/**
 *
 * @file OnePIC_16bit_pot.c
 * @brief configures the potentiometer and takes a reading
 * @defgroup sixteen_bit_pot Potentiometer
 * @ingroup sixteen_bit
 * @{
 */

#include "OnePIC_pot.h"
#include "TimeDelay.h"
#include "bsp.h"
#include "OnePIC_int.h"

#define FILTER_MEDIAN_LENGTH 3 
#define FILTER_MIDDLE_VALUE (FILTER_MEDIAN_LENGTH-1)/2

/**
 * @brief compares two values
 * @param e1
 * @param e2
 * @return bool indicating which is greater:
 *  - -1 -> v1 < v2
 *  - 1  -> v1 = v2
 *  - 0  v1 > v2
 */
static int comp(const void *e1, const void *e2)
{
    const int * a1 = e1;
    const int * a2 = e2;
    if (*a1 < *a2)
        return -1;
    else if (*a1 == *a2)
        return 0;
    else
        return 1;
}

/**
 * @brief Initilizes the POT to take a reading
 */
void EDF_potInit(void)
{
 //   _pbState = 1;
    AD1PCFGL &= ~0x10; // Clear PCFG8
}  


/**
 * @brief experiencing some dropped readings, so the median filter takes three samples and passes the median of the three
 * @param d adc value
 * @return 16bit average of the ADC
 */
uint16_t filterMedian(uint16_t d)
{
    static uint16_t mVal[FILTER_MEDIAN_LENGTH];
    static uint8_t mIndex = 0; 
        
    mVal[mIndex++] = d;
    
    if(mIndex == FILTER_MEDIAN_LENGTH) {
        mIndex = 0;
    }
     
    qsort(mVal, FILTER_MEDIAN_LENGTH, sizeof(uint16_t), comp);
    
    return mVal[FILTER_MIDDLE_VALUE];
}
/**
 * @brief takes the reading
 * @return the 16bit digital representation of the analog voltage
 */
uint16_t EDF_potRead(void)
{
    uint16_t filteredPotReading;
    EDF_INT_SourceDisable ( INT_TMR_1 );
    AD1CHS = POT_CHANNEL;
    AD1CON1bits.SAMP  = 1;
    Delay10us(10);
    AD1CON1bits.SAMP  = 0;    
    while(!AD1CON1bits.DONE);
    filteredPotReading = /*filterMedian*/(ADC1BUF0);
    EDF_INT_SourceEnable ( INT_TMR_1 );
    return filteredPotReading;
}

/** @}*/
