/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    PIC18F mTouch CVD and CTMU Framework
 *  FileName:        mTouchCTMU_Acquisition.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Acquisition Module Implements
 *                   the CTMU acquisition process.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
 /****************************************************************************
 * Revision History
 * Date		Rev	 Author  Comments
 * 07.07.11
 ****************************************************************************/
#include "mTouch.h"

INT16 ACQ_CTMU(void)
{
UINT16* tris;
UINT16* lat;
UINT16  ioMask;
UINT16  channelBits;
INT16  result;

////////////////////////////////////////////////////////////////////
// Read sensor info
////////////////////////////////////////////////////////////////////
    tris = sensors[currentSensor].trisReg;
    lat = sensors[currentSensor].latReg;
    ioMask = sensors[currentSensor].ioMask;
    channelBits = sensors[currentSensor].channelBits;

////////////////////////////////////////////////////////////////////
// CTMU setup
////////////////////////////////////////////////////////////////////
    // Set current source value to 55 uA 
    CTMUICONbits.ITRIM = -30;
	CTMUICONbits.IRNG  = 0x03;
	CTMUCONbits.CTMUEN = 1;	

    // Disable edges 
	CTMUCONbits.EDGEN = 0;
	// Rearm the CTMU (must clear CTMUCON.EDGE1STAT and CTMUCON.EDGE2STAT bits simultaneously)
	CTMUCON = CTMUCON & 0xFFFC;

    // Switch channel to the current sensor
    AD1CHS = channelBits;
    // Set current sensor to GND (discharge external)
    *lat &= ~ioMask; *tris &= ~ioMask;
	// Ground current source and start A2D sampling to discharge internal A2D capasitor
    AD1CON1bits.SAMP  = 1;
	CTMUCONbits.IDISSEN = 1; 
    // Wait to discharge
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    // Enable the current source
	CTMUCONbits.IDISSEN = 0;
    // Disconnect current sensor from GND (HI-Z)
    *tris |= ioMask;

    // Start charging
	CTMUCONbits.EDG1STAT = 1;

    // Wait for the sensor charge
    Nop();Nop(); Nop();Nop(); Nop();Nop(); Nop();Nop();
    Nop();Nop(); Nop();Nop(); Nop();Nop(); Nop();Nop();
    Nop();Nop(); Nop();Nop(); Nop();Nop(); Nop();Nop();
    Nop();Nop(); Nop();Nop(); Nop();Nop(); Nop();Nop();
    Nop();Nop(); Nop();Nop(); Nop();Nop(); Nop();Nop();

    // Stop charging
	CTMUCONbits.EDG2STAT = 1;

////////////////////////////////////////////////////////////////////
// Get result
////////////////////////////////////////////////////////////////////  
    AD1CON1bits.SAMP  = 0;    
 	while(!AD1CON1bits.DONE);
    // Ground sensor
    *tris &= ~ioMask;

    // Read result
    result = ADC1BUF0;

#if defined(A2D_12_BITS)

    result = 4096 - result;
    result >>= 2;

#else

    result = 1024 - result;

#endif

    return result;
}
