/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    PIC18F mTouch CVD and CTMU Framework
 *  FileName:        mTouchCVD_Acquisition.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Acquisition Module Implements
 *                   the CVD acquisition process.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
 /****************************************************************************
 * Revision History
 * Date		Rev	 Author  Comments
 * 07.07.10
 ****************************************************************************/
#include "mTouch.h"

INT16 ACQ_CVD(void)
{
UINT16* prevTRIS;
UINT16* prevLAT;
UINT16  prevIOMask;
UINT16* curTRIS;
UINT16* curLAT;
UINT16  curIOMask;
UINT16  prevChannelBits;
UINT16  curChannelBits;
INT16  result;

////////////////////////////////////////////////////////////////////
// Read sensor info
////////////////////////////////////////////////////////////////////
    prevTRIS = sensors[previousSensor].trisReg;
    prevLAT = sensors[previousSensor].latReg;
    prevIOMask = sensors[previousSensor].ioMask;
    curTRIS = sensors[currentSensor].trisReg;
    curLAT = sensors[currentSensor].latReg;
    curIOMask = sensors[currentSensor].ioMask;
    prevChannelBits = sensors[previousSensor].channelBits;
    curChannelBits = sensors[currentSensor].channelBits;

////////////////////////////////////////////////////////////////////
// Scan A
////////////////////////////////////////////////////////////////////
    // Set channel to the previous sensor
    AD1CHS = prevChannelBits;
    // Set previous sensor to Vdd (charge internal)
    *prevLAT |= prevIOMask; *prevTRIS &= ~prevIOMask;
    // Start sampling
    AD1CON1bits.SAMP  = 1; 
    // Wait
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    // Stop sampling
 //   ADCON0bits.ADON = 0; ADCON0bits.ADON = 1; 

    // Set current sensor to GND (discharge external)
    *curLAT &= ~curIOMask; *curTRIS &= ~curIOMask;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();

    // Disconnect current sensor from GND (HI-Z)
    *curTRIS |= curIOMask;
    // Switch channel to the current sensor
    AD1CHS = curChannelBits;

    // Start ADC 
    AD1CON1bits.SAMP  = 0;
    while(!AD1CON1bits.DONE);

    *curTRIS &= ~curIOMask;
    *prevLAT &= ~prevIOMask;

    // Store the result
    result = ADC1BUF0;

////////////////////////////////////////////////////////////////////
// Scan B
////////////////////////////////////////////////////////////////////
    // Switch channel to the previous sensor
    AD1CHS = prevChannelBits;
    // Set previous sensor to GND (discharge internal)
    *prevLAT &= ~prevIOMask; *prevTRIS &= ~prevIOMask;
    // Start sampling
    AD1CON1bits.SAMP  = 1;
    // Wait
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    // Stop sampling 
 //   ADCON0bits.ADON = 0; ADCON0bits.ADON = 1; 

    // Set current sensor to Vdd (charge external)
    *curLAT |= curIOMask; *curTRIS &= ~curIOMask;
    Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();Nop();
    // Disconnect current sensor from Vdd (HI-Z)
    *curTRIS |= curIOMask;
    // Switch channel to the current sensor
    AD1CHS = curChannelBits;

    // Start ADC
    AD1CON1bits.SAMP  = 0;
	while(!AD1CON1bits.DONE);

    // Ground sensor
    *curLAT &= ~curIOMask;
    *curTRIS &= ~curIOMask;

    // Store the result by subtracting
    // it from the value stored in result. This creates the 'differential'
    // value that will be accumulated in the next step.
    result -= ADC1BUF0;

    if(result < 0)
    {
        result =  - result;
    }

    return result;
}
