/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    PIC18F mTouch CVD and CTMU Framework
 *  FileName:        mTouch_Acquisition.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Acquisition Module
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and INT16ellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
 /****************************************************************************
 * Revision History
 * Date		Rev	 Author  Comments
 * 07.07.11
 ****************************************************************************/

#include "mTouch.h"

volatile UINT8  mTouchDataReady;                               // Flag to indicate that new data available for processing 
volatile UINT32 oversampleFilterData [MTOUCH_NUMBER_SENSORS];  // Accumulator array for sensor oversampling
volatile INT16  sampleCounter = MTOUCH_SAMPLES_PER_SCAN;       // Counter variable for oversampling
                               
volatile INT16  decimateFilterData [MTOUCH_NUMBER_SENSORS];    // Decimate-filtered result register
volatile UINT16 sensorData [MTOUCH_NUMBER_SENSORS];            // Latch register storing the latest acquisition data (output)
            
volatile UINT8  jitter = 0x55;                                 // Stores the current random seed value for jittering

volatile UINT8  currentSensor  = 0;                            // Currently scanned sensor index value
volatile UINT8  previousSensor = 1;                            // Previously scanned sensor index value


void MTouchAcquisition(void)
{
INT16  data; 
INT8 i;  

    if(MTouchIsDataReady())
    {
       return;
    }

    for(i=0; i<MTOUCH_NUMBER_SENSORS; i++)
    {
        // Move indexes to the next sensor.
        previousSensor = currentSensor;
        currentSensor = i;

 //       data = ACQ_CTMU();
       data = ACQ_CVD();

        // Add the last_result to decimate filter
        if(data >= decimateFilterData[currentSensor])
        {
            decimateFilterData[currentSensor] += 2;
        } else {
            decimateFilterData[currentSensor]--;
        }

        // Add the decimate filter to the accumulator.
        oversampleFilterData[currentSensor] += decimateFilterData[currentSensor];

    }

    sampleCounter--;

    // Set the scan done flag and pass result to the decode module.
    if(sampleCounter < 0)
    {
        sampleCounter = MTOUCH_SAMPLES_PER_SCAN;
        
        for(i=0; i<MTOUCH_NUMBER_SENSORS; i++)
        {
            #if   (MTOUCH_SCALING == 1)
                sensorData[i] = (UINT16)oversampleFilterData[i];
            #elif (MTOUCH_SCALING == 2)
                sensorData[i] = (UINT16)(oversampleFilterData[i] >> 1);
            #elif (MTOUCH_SCALING == 4)
                sensorData[i] = (UINT16)(oversampleFilterData[i] >> 2);
            #elif (MTOUCH_SCALING == 8)
                sensorData[i] = (UINT16)(oversampleFilterData[i] >> 3);
            #elif (MTOUCH_SCALING == 16)
                sensorData[i] = (UINT16)(oversampleFilterData[i] >> 4);
            #else
                sensorData[i] = (UINT16)(oversampleFilterData[i] >> 8);
            #endif   
            oversampleFilterData[i] = 0; 
        }
        mTouchDataReady = 1;
    }
}


UINT8 MTouchJitter(void) 
{
 	if(jitter & 0x80){
       jitter <<= 1;
       jitter ^= 0x07; // Polynomial: X^8+X^2+X+1.
    }else{
       jitter <<= 1;
    }
    return jitter;
}
