/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch CVD and CTMU Framework
 *  FileName:        mTouch_Config.h
 *  Dependencies:    
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework Configuration File
 *************************************************************************/
/***********************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
 /****************************************************************************
 * Revision History
 * Date		Rev	 Author  Comments
 * 07.07.11
 ****************************************************************************/


//**********************************************
//
// Debug and GUI Communication Configuration
//
//**********************************************

                                                    //====================================================
//#define MTOUCH_DEBUG                                // Enable CVD Data Logging through RS232, comment 
                                                    // this out to disable RS232 output.
                                            
//#define MTOUCH_DEBUG_OUTPUT_HEX                     // Enabled: Output in Hex notation
                                                    // Disabled: Output in Decimal notation
                                            
#define MTOUCH_DEBUG_SPEED              115200      // 115200 or 9600

#define MTOUCH_DEBUG_OUTPUT_RAW                     // You may output the raw sensor values, the sensor average
//#define MTOUCH_DEBUG_OUTPUT_AVG                   // values, or both. Simply uncomment/comment one or more options.

#define MTOUCH_JITTER_ENABLE

//**********************************************
//
// Typical-Use Configuration Options
//
//**********************************************

#define MTOUCH_USE_NEGATIVE_PULSE

                                            //====================================================
#define MTOUCH_NUMBER_SENSORS          5    // Number of CVD Sensor Inputs

                                            //====================================================
#define MTOUCH_SAMPLES_PER_SCAN        32   // This is the number of times each sensor will be 
                                            // scanned before asserting the mTouchDataReady 
                                            // flag. If the application requires a specific
                                            // response time, there are several variables that 
                                            // will be important to the calculation of this
                                            // parameter.
                                            
                                            
                                            //====================================================
#define MTOUCH_SCALING                 8    // This option will determine the amount of post-scaling
                                            // that is implemented on the acquisition's accumulator
                                            // register before being stored in the results register.
                                            // The scaling factor should be set so that it is not
                                            //  possible for the result register to overflow 
                                            //      (Max output value is 65535.)
                                            // When in doubt, 16 is usually the best option.
                                            //
                                            //
                                            // Options:
                                            //
                                            //   1 - No scaling is performed.
                                            //       1:1 relationship between accumulator and result
                                            //
                                            //  16 - Accumulator is divided by 16.
                                            //       16:1 relationship between accumulator and result
                                            //
                                            // 256 - Accumulator is divided by 256.
                                            //       256:1 relationship between accumulator and result
                                            //

//**********************************************
//
// Advanced Configuration Options
//
//**********************************************

                                            //====================================================
#define POWER_UP_SAMPLES            100 // This is the number of total scans that should be
                                            //   taken across ALL SENSORS before they will be 
                                            //   considered initialized.
                                            //
                                            // For example:
                                            //      Let's assume the dataReady flag is set once
                                            //      every 100ms due to the MTOUCH_SAMPLES_PER_SCAN
                                            //      option and there are 8 sensors in our system.
                                            //      If we want a 3 second initialization time,
                                            //      we set POWER_UP_SAMPLES to 240.
                                            //
                                            //  (dataReady Toggle Rate / MTOUCH_NUMBER_SENSORS) * POWER_UP_SAMPLES
                                            //  = Initialization Duration
                                            //
                                            //   Allowable Range :: [ 1 to 65535 ]

                                            //====================================================
#define	BUTTON_TIMEOUT              200     // This feature will reset the sensor's state if it
                                            // remains in the 'pressed' state for too long.
                                            // This number is based off the amount of time between
                                            // dataReady flag toggles.
                                            //
                                            // For example: 
                                            //      Let's assume the dataReady flag is set once
                                            //      every 100ms due to the MTOUCH_SAMPLES_PER_SCAN
                                            //      option. If we want a timeout of 10sec, the
                                            //      BUTTON_TIMEOUT value should be set to 100.
                                            //      If we want a timeout of 7.4 seconds, then
                                            //      BUTTON_TIMEOUT should be set to 74.
                                            //
                                            // dataReady Toggle Rate * BUTTON_TIMEOUT = Timeout Duration
                                            //
                                            //
                                            //   Allowable Range :: [ 2 to 255 ]
                                            //
                                            // Setting to '0' or commenting the #define will
                                            //   disable the timeout feature.
                                            
                                            //====================================================
#define AVG_ENABLE                          // Leaving this uncommented will enable 
                                            //   implementation of an average (or 'baseline')
                                            //   to track environmental changes in the system.
                                            // NOTE!: Turning this off is highly discouraged.
                                            
                                            
                                            //====================================================
#define AVG_UPDATE                     8   // When the average updates itself using a new 
                                            //   reading, this value determines what weight is 
                                            //   given to the reading variable in the 
                                            //   calculation of the new average.
                                            // The reading variable will have a weight of 
                                            //   1/AVG_UPDATE in the average calculation. 
                                            // Ex: If it is 4, the average calculation would be 
                                            //   average = (1/4)*reading + (3/4)*average
                                            // Allowed Values :: 1, 2, 4, 8, 16

                                            
                                            
                                            //====================================================
#define AVG_RATE                       4   // The update rate of the sensors' average values 
                                            //   when the current sensor value is farther away 
                                            //   from the average value than AVG_RATE_THRESHOLD. 
                                            // (Fastest = 1)
                                            // Allowable Range :: [ 1 to 255 ]
                                          
                                            //====================================================
#define	DEBOUNCE_COUNT_RELEASE         1    // Number of consecutive scans a button must be seen
                                            //   as released before a 'released state' is declared
                                            //
                                            //   Allowable Range :: [ 1 to 255 ]
 
 
                                            //====================================================
#define	DEBOUNCE_COUNT_PRESS    	   1    // Number of consecutive scans a button must be seen 
                                            // as pressed before a 'pressed state' is declared
                                            //
                                            //   Allowable Range :: [ 1 to 255 ]                                            

                                            
#define NEGATIVE_CAPACITANCE           0    //====================================================
                                            // This option determines the method used to handle
                                            //   a negative capacitance shift. In other words, how
                                            //   the system will respond to a reading value shift
                                            //   in the opposite direction of a normal shift
                                            //   caused by a finger.
                                            // 
                                            // These shifts occur for a variety of reasons and you 
                                            //   may want to adjust this behavior based on your 
                                            //   system.
                                            //
                                            // Options:
                                            //
                                            //   0 - Normal Average Behavior
                                            //       (DEFAULT) No special behavior. The average 
                                            //       will update at the normal rate, as always.
                                            //       For Metal-over-Capacitive systems, a negative
                                            //       shift usually means a neighboring sensor is
                                            //       being pressed, so this is the best option to
                                            //       keep the current state of the sensor and
                                            //       allow it to be pressed quickly after a
                                            //       neighbor has been pressed.
                                            //
                                            //   1 - Increase Average Speed
                                            //       The average will ignore the AVG_RATE counter
                                            //       and update itself after each new reading
                                            //       value. This has the effect of speeding up
                                            //       the average until it catches up with the
                                            //       negative shift. 
                                            //
                                            //   2 - Average Directly Follows Reading
                                            //       The average will instantly follow a negative
                                            //       shift. This behavior is best for a system
                                            //       that needs a very fast response time. It
                                            //       will help prevent quick, repeated presses 
                                            //       from being missed due to the average not
                                            //       updating quickly enough.
                                            

//////////////////////////////////////////////////////////////////////
//                                                                  //
//                          ERRORS CHECKING                         //
//                                                                  //
//====================================================================

#if defined(AVG_UPDATE)
    #if ((AVG_UPDATE != 0) && (AVG_UPDATE != 1) && (AVG_UPDATE != 2) && (AVG_UPDATE != 4) && (AVG_UPDATE != 8) && (AVG_UPDATE != 16))
        #warning Invalid value for AVG_UPDATE. AVG_ENABLE has been turned off.
        #undef AVG_UPDATE
        #undef AVG_ENABLE
    #elif ((AVG_UPDATE == 0) || (AVG_UPDATE == 1))
        #undef AVG_ENABLE
    #endif
#endif

#if !defined(POWER_UP_SAMPLES)
    #warning POWER_UP_SAMPLES must be defined in the configuration file. Setting to '50'.
    #define POWER_UP_SAMPLES 50
#else
    #if POWER_UP_SAMPLES > 65535
        #warning POWER_UP_SAMPLES exceeds the maximum limit of 65535. Setting to '65535'.
        #undef POWER_UP_SAMPLES
        #define POWER_UP_SAMPLES 65535
    #endif
    #if POWER_UP_SAMPLES < 1
        #warning POWER_UP_SAMPLES must be at least '1'. Setting to '1'.
        #undef POWER_UP_SAMPLES
        #define POWER_UP_SAMPLES 1
    #endif
#endif

#if defined (BUTTON_TIMEOUT)
    #if BUTTON_TIMEOUT > 255
        #warning BUTTON_TIMEOUT exceeds the maximum limit of 255. Setting to '255'.
        #undef BUTTON_TIMEOUT
        #define BUTTON_TIMEOUT 255
    #endif
#endif

#if !defined(DEBOUNCE_COUNT_RELEASE)
    #define DEBOUNCE_COUNT_RELEASE 1
#else
    #if DEBOUNCE_COUNT_RELEASE > 255
        #warning DEBOUNCE_COUNT_RELEASE exceeds the maximum limit of 255. Setting to '255'.
        #undef DEBOUNCE_COUNT_RELEASE
        #define DEBOUNCE_COUNT_RELEASE 255
    #endif
    #if DEBOUNCE_COUNT_RELEASE < 1
        #warning DEBOUNCE_COUNT_RELEASE must be at least '1'. Setting to '1'.
        #undef DEBOUNCE_COUNT_RELEASE
        #define DEBOUNCE_COUNT_RELEASE 1
    #endif
#endif

#if !defined(DEBOUNCE_COUNT_PRESS)
    #define DEBOUNCE_COUNT_PRESS 1
#else
    #if DEBOUNCE_COUNT_PRESS > 255
        #warning DEBOUNCE_COUNT_PRESS exceeds the maximum limit of 255. Setting to '255'.
        #undef DEBOUNCE_COUNT_PRESS
        #define DEBOUNCE_COUNT_PRESS 255
    #endif
    #if DEBOUNCE_COUNT_PRESS < 1
        #warning DEBOUNCE_COUNT_PRESS must be at least '1'. Setting to '1'.
        #undef DEBOUNCE_COUNT_PRESS
        #define DEBOUNCE_COUNT_PRESS 1
    #endif
#endif

#if !defined(MTOUCH_SAMPLES_PER_SCAN)
    #warning MTOUCH_SAMPLES_PER_SCAN must be defined in the configuration file. Defaulting to 20ms timing.
    #define MTOUCH_SAMPLES_PER_SCAN MTOUCH_20ms_Timing
#endif

#if !defined(NEGATIVE_CAPACITANCE)
    #define NEGATIVE_CAPACITANCE 0
#endif
#if (NEGATIVE_CAPACITANCE != 0) && (NEGATIVE_CAPACITANCE != 1) && (NEGATIVE_CAPACITANCE != 2)
    #warning NEGATIVE_CAPACITANCE is set to an invalid value. Defaulting to 0.
    #undef NEGATIVE_CAPACITANCE
    #define NEGATIVE_CAPACITANCE 0
#endif
