/********************************************************************
* FileName:		Controller.c
* Dependencies: none   
* Processor:	PIC18 tested with 18F87J11	
* Hardware:		PIC18 Explorer
* Complier:     Microchip C18 v3.30 or higher
* Company:		Microchip Technology, Inc.
*
* Copyright and Disclaimer Notice for P2P Software:
*
* Copyright  2009 Microchip Technology Inc.  All rights reserved.
*
* Microchip licenses to you the right to use, modify, copy and distribute 
* Software only when embedded on a Microchip microcontroller or digital 
* signal controller and used with a Microchip radio frequency transceiver, 
* which are integrated into your product or third party product (pursuant 
* to the terms in the accompanying license agreement).  
*
* You should refer to the license agreement accompanying this Software for 
* additional information regarding your rights and obligations.
*
* SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY 
* KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY 
* WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A 
* PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE 
* LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, 
* CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY 
* DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO 
* ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, 
* LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, 
* TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT 
* NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*
*********************************************************************
* File Description:
*
*  This is the sample application that demonstrate a controller device
*  that runs ZigBee Remote Control (ZRC) profile on top of Microchip 
*  certified ZigBee(R) RF4CE protocol stack.
*
* Change History:
*  Rev          Date            Author      Description
*  1.0-1.1.2    12/10/2009      yfy         Initial Release
********************************************************************/


/************************ HEADERS **********************************/
#include "Common\Console.h"
#include "Transceivers\Transceivers.h"
#include "Common\SymbolTime.h"
#include "Common\LCDBlocking.h"
#include "Common\NVM.h"
#include "WirelessProtocols\RF4CE\RF4CE.h"
#include "OnePIC_lcd.h"
#include "OnePIC_led.h"
#include "OnePIC_pot.h"
#include "Controller.h"
#include "mTouch.h"
#include "OnePIC_mTouch.h"
#include "RF4CEController.h"

#define ZRC_USER_CONTROL_PRESSED        1
#define ZRC_USER_CONTROL_REPEATED       2
#define ZRC_USER_CONTROL_RELEASED       3
#define ZRC_COMMAND_DISCOVERY_REQUEST   4
#define ZRC_COMMAND_DISCOVERY_RESPONSE  5

extern BYTE myDefaultChannel;

extern CONTROLLER_TV_CMD GetTerminalCommand();
    
void CONTROLLERSendTVCommand(CONTROLLER_TV_CMD cmd);
void volumeTasks(void);
/*********************************************************************
* Function:         void main(void)
*
* PreCondition:     none
*
* Input:		    none
*
* Output:		    none
*
* Side Effects:	    none
*
* Overview:		    This is the main function that runs the simple 
*                   example demo. The purpose of this example is to
*                   demonstrate the simple application programming
*                   interface for the RF4CE stack. 
*
* Note:			    
********************************************************************/

//#if defined(__18CXX)
//void main(void)
//#elif defined __PIC24FJ__
//int main(void)
//#elif defined EDF_DEMO
int demo_main_loop(void)
//#endif
{   
    BYTE i;
    BYTE DiscoveryNum;
    unsigned char Command;
    TX_OPTIONS TxOptions;
    CONTROLLER_TV_CMD tvCmd;

    BoardInit();    // Initialize ports, timers, interrupts...
    ConsoleInit();
 
    //*** turn on interrupts INTCONbits.GIEH = 1;    
    //printf("\r\nPIC24 RF4CE Controller");
    
    sprintf(LCDBuffer[0],"Pairing....     ");
    LCDUpdate();
    RF4CEControllerInit();
    while(!RF4CEControllerPairing());
    sprintf(LCDBuffer[0],"Paired       ");
    LCDUpdate();
    LED_1RF = 1;


    while(1)
    {
        /**************************************************************************
         * RF4CE API RF4CE_MessageAvailable is the function call to check if a new
         * message for application layer has been received.
         * The signature of the RF4CE_MessageAvailable function is
         *
         *      BOOL RF4CE_MessageAvailable(void)
         *
         * The return value is a boolean to indicate if a message has been received
         ****************************************************************************/
        if( RF4CE_MessageAvailable() )
        {
            // A remote control is not supposed to receive any message in this demo,
            // so ignore any received messages
            /*****************************************************************************
             * RF4CE API RF4CE_DiscardMessage is the function call to notify the RF4CE
             * stack that it can discard the current message and the application layer
             * is ready to process the next message.
             * The signature of the RF4CE_DiscardMessage function is
             *
             *      void RF4CE_DiscardMessage(void)
             ******************************************************************************/
            RF4CE_DiscardMessage();
        }
        volumeTasks();
        mTouch_ProcessKeys();

    } // end while(1)
} // end main

void volumeTasks()
{
    static uint16_t newReading, oldReading, diffReading;
    static uint8_t upTick=0,downTick=0;
    uint8_t potTick;
    newReading = EDF_potRead();
    if(newReading < 10 && oldReading >= 10)
    {
        for(potTick = 0; potTick < 100; potTick++)
        {
            RF4CEControllerVolumnDown();
        }
    }
    if(newReading > oldReading)
    {
                RF4CEControllerVolumnUp();
    }
    else if(newReading < oldReading)
    {
                RF4CEControllerVolumnDown();
    }
    else
    {}
    oldReading = newReading;
   // RF4CEControllerVolumnDown();
   // RF4CEControllerVolumnUp();
}
// XXX 11/14/2011 10:45:56 AM: added ProcessKeys
void mTouch_ProcessKeys(void)
{

    if(0)//MTouchGetPressReleaseButtonState(VOL_PLUS) == MTOUCH_PRESSED)
    {
        RF4CEControllerVolumnUp();
    }
    else if(0)//MTouchGetPressReleaseButtonState(VOL_MINUS) == MTOUCH_PRESSED)
    {
        RF4CEControllerVolumnDown();
    }
    else  if(0)//MTouchGetPressReleaseButtonState(CH_PLUS) == MTOUCH_PRESSED)
    {
        RF4CEControllerChannelUp();
    }
    else  if(0)//MTouchGetPressReleaseButtonState(CH_MINUS) == MTOUCH_PRESSED)
    {
        RF4CEControllerChannelDown();
    }
    else  if(EDF_CapTouchGetButtonState(CAP_UP) == MTOUCH_PRESSED)
    {

        RF4CEControllerUp();
        EDF_LEDSet(LED_UP);
    }
    else  if(EDF_CapTouchGetButtonState(CAP_DOWN) == MTOUCH_PRESSED)
    {

        EDF_LEDSet(LED_DOWN);

        RF4CEControllerDown();
    }  
    else  if(EDF_CapTouchGetButtonState(CAP_LEFT) == MTOUCH_PRESSED)
    {

        EDF_LEDSet(LED_LEFT);
        RF4CEControllerLeft();
    }
    else  if(EDF_CapTouchGetButtonState(CAP_RIGHT) == MTOUCH_PRESSED)
    {

        EDF_LEDSet(LED_RIGHT);
        RF4CEControllerRight();
    } 
    else  if(EDF_CapTouchGetButtonState(CAP_ENTER) == MTOUCH_PRESSED)
    {

        RF4CEControllerSelect();
    }   
    else
    {
        EDF_LEDClr(LED_UP);
        EDF_LEDClr(LED_DOWN);
        EDF_LEDClr(LED_LEFT);
        EDF_LEDClr(LED_RIGHT);
    }
} 


/******************************************************************************
 * void RF4CEDiscoveryCallBack(void)
 ******************************************************************************/ 
void RF4CEDiscoveryCallBack(void)
{
}

/*************************************************************************
  Function:
           void RF4CE_CB_StatusReport(BYTE status, BYTE PairIndex)
    
  Summary:
    This is the callback function from the RF4CE stack to the application
    to report communication status, especially communication status in the
    pairing process.
  Description:
    This is the callback function from the RF4CE stack to the application
    to report communication status, especially in the pairing process.
    Application layer generally can ignore this status report, unless the
    status is not SUCCESS. In the case a communication fails during the
    pairing process, either because of timeout in the process of sending a
    command, or timeout to receive certain command, the status will be set
    to represent failure reason.
  Conditions:
    Transceiver initialization has been finished.
  Input:
    BYTE status -     The status of the communication status.
    BYTE PairIndex -  The index of the communication destination device in
                      the pair table
  Return:
    None
  Example:
    <code>
    void RF4CE_CB_StatusReport(BYTE status, BYTE PairIndex)
    {
        // handle the communication error if pairing process fail.
        if( status != SUCCESS )
        {
            // handle the error
        }
    }
    </code>
  Remarks:
    None                                                                  
  *************************************************************************/ 
void RF4CE_CB_StatusReport(BYTE status, BYTE PairIndex)
{
    sprintf(LCDBuffer[0],"RF4CE Comm Status: %x ", status);
    LCDUpdate();
    //PrintChar(status);
    sprintf(LCDBuffer[0],"Pair Index: %x      ", PairIndex);
    LCDUpdate();
    if( status == SUCCESS )
    {
        PrintChar(PairIndex);
    }   
    else
    {
        PrintChar(0xFF);
    }        
}    

/*************************************************************************
 CONTROLLERSendTVCommand
  *************************************************************************/ 
void CONTROLLERSendTVCommand(CONTROLLER_TV_CMD cmd)
{
    TX_OPTIONS TxOptions;

    TxData = 0;
    TxBuffer[TxData++] = ZRC_USER_CONTROL_PRESSED;
    TxBuffer[TxData++] = (BYTE)cmd;
    
    /*************************************************************************************
    * TxOptions decides how to transmit the data
    * The defintion of TxOptions structure is
    *
    *      typedef union
    *      {
    *          BYTE Val;
    *          struct
    *          {
    *              BYTE    broadcast       :1;     // 1: broadcast, 0: unicast
    *              BYTE    IEEEAddress     :1;     // 1: IEEE destination address, 
    *                                              // 0: Network destination address
    *              BYTE    ackReq          :1;     // 1: required acknowledgement, 
    *                                              // 0: acknowledgement not required
    *              BYTE    secEn           :1;     // 1: secure the outgoing packet, only 
    *                                              // valid in unicast
    *                                              // 0: not secure the outgoing packet
    *              BYTE    singleChannel   :1;     // 1: single channel transmission
    *                                              // 0: transmit on all 3 channels in 
    *                                              // broadcast, or transmit on all 3 
    *                                              // channels if acknowledgement is not 
    *                                              // received in unicast with acknowledgement
    *              BYTE    channelDsgtor   :1;     // 1: Channel designator on, 
    *                                              // 0: Channel designator off
    *              BYTE    vendorFrame     :1;     // 1: Vendor specific frame, 
    *                                              // 0: normal profile frame
    *          } bits;        
    *      } TX_OPTIONS;
    *
    ************************************************************************************/
    TxOptions.Val = 0;
    TxOptions.bits.ackReq = 1;
    //TxOptions.bits.secEn = 1;    
    
    /************************************************************************************
    * RF4CE API RF4CE_SendFrame is the function to send message
    * The sigunature of function RF4CE_SendFrame is
    *
    *      BYTE RF4CE_SendFrame(   BYTE PairIndex, BYTE ProfileID, WORD VendorID, 
    *                              TX_OPTIONS TxOptions )
    *
    * Input parameter PairIndex is the index of the pair table that represents the 
    * destination node.
    * Input parameter ProfileID is the profile identifier of the message belongs to
    * Input parameter VendorID is the vendor identifier. It is only valid for vendor
    * specific messages.
    * Input parameter TxOptions is the options to notify RF4CE stack how to transmit
    * the message.
    *
    * The return value is the status byte to indicate the transmission status.
    *************************************************************************************/
    RF4CE_SendFrame(0, ZRC_PROFILE_ID, VENDOR_IDENTIFIER, TxOptions);
}

/*****************************************************************************
 * void Get64BitNodeAddress(void)
 *****************************************************************************/
void RF4CE_CB_Get64BitNodeAddress(BYTE *address)
{
    ////MCHP25AA02E48GetEUI64NodeAddress(address);

    // check to make sure the node was loaded properly, IEEE assigned OUI
    if((address[0] != 0x00) || (address[1] != 0x04) || (address[2] != 0xA3))
    {
        address[0] = EUI_0;
        address[1] = EUI_1;
        address[2] = EUI_2;
        address[3] = EUI_3;
        address[4] = EUI_4;
        address[5] = EUI_5;
        address[6] = EUI_6;
        address[7] = EUI_7;
    }
}

/*****************************************************************************
 * DWORD RF4CE_CB_GetFrameCounter(void)
 *****************************************************************************/
DWORD RF4CE_CB_GetFrameCounter(void)
{
    DWORD frameCounter;
    frameCounter = 0;
    //MCHP25AA02E48ReadArray(RF4CE_FRAME_COUNTER_EEPROM_NVM_ADDRESS, (BYTE *)&frameCounter, 4);

    return frameCounter;
}

/*****************************************************************************
 * void RF4CE_CB_PutFrameCounter(DWORD frameCounter)
 *****************************************************************************/
void RF4CE_CB_PutFrameCounter(DWORD frameCounter)
{
    //MCHP25AA02E48WriteArray(RF4CE_FRAME_COUNTER_EEPROM_NVM_ADDRESS, (BYTE *)&frameCounter, 4);
}

/*****************************************************************************
 * BYTE RF4CE_CB_GetDefaultChannel(void)
 *****************************************************************************/
BYTE RF4CE_CB_GetDefaultChannel(void)
{
    return 0;//MCHP25AA02E48ReadByte(RF4CE_DEFAULT_CHANNEL_EEPROM_NVM_ADDRESS);
}

/*****************************************************************************
 * void RF4CE_CB_PutDefaultChannel(BYTE defaultChannel)
 *****************************************************************************/
void RF4CE_CB_PutDefaultChannel(BYTE defaultChannel)
{
    //MCHP25AA02E48WriteByte(RF4CE_DEFAULT_CHANNEL_EEPROM_NVM_ADDRESS, defaultChannel);
}

/*****************************************************************************
 * BYTE RF4CE_CB_GetPowerSaveMode(void)
 *****************************************************************************/
BYTE RF4CE_CB_GetPowerSaveMode(void)
{
    return 0;//MCHP25AA02E48ReadByte(RF4CE_POWER_SAVE_MODE_EEPROM_NVM_ADDRESS);
}

/*****************************************************************************
 * void RF4CE_CB_PutPowerSaveMode(BYTE powerSaveMode)
 *****************************************************************************/
void RF4CE_CB_PutPowerSaveMode(BYTE powerSaveMode)
{
    //MCHP25AA02E48WriteByte(RF4CE_POWER_SAVE_MODE_EEPROM_NVM_ADDRESS, powerSaveMode);
}

/*****************************************************************************
 * WORD RF4CE_CB_GetShortAddress(void)
 *****************************************************************************/
WORD RF4CE_CB_GetShortAddress(void)
{
    return 0;//SST25ReadWord(RF4CE_SHORT_ADDRESS_NOR_NVM_ADDRESS);
}

/*****************************************************************************
 * void RF4CE_CB_PutShortAddress(WORD shortAddress)
 *****************************************************************************/
void RF4CE_CB_PutShortAddres(WORD shortAddress)
{
    //SST25WriteWord(shortAddress, RF4CE_SHORT_ADDRESS_NOR_NVM_ADDRESS);
}

/*****************************************************************************
 * WORD RF4CE_CB_GetShortAddress(void)
 *****************************************************************************/
WORD RF4CE_CB_GetPanID(void)
{
    return 0;//SST25ReadWord(RF4CE_PAN_ID_NOR_NVM_ADDRESS);
}

/*****************************************************************************
 * void RF4CE_CB_PutShortAddress(WORD shortAddress)
 *****************************************************************************/
void RF4CE_CB_PutPanID(WORD panID)
{
    //SST25WriteWord(panID, RF4CE_PAN_ID_NOR_NVM_ADDRESS);
}

/*****************************************************************************
 * void RF4CE_CB_GetPairTable(BYTE *pairTable)
 *****************************************************************************/
void RF4CE_CB_GetPairTable(BYTE *pairTable)
{
    //SST25ReadArray(RF4CE_PAIR_TABLE_NOR_NVM_ADDRESS, pairTable, (WORD)MAX_PAIR_SIZE * sizeof(PAIR_TABLE_ENTRY));
}

/*****************************************************************************
 * void RF4CE_CB_PutShortAddress(WORD shortAddress)
 *****************************************************************************/
void RF4CE_CB_PutPairTableIndex(BYTE *pairTable, WORD index)
{
    //SST25WriteArray(RF4CE_PAIR_TABLE_NOR_NVM_ADDRESS + ((DWORD)index * sizeof(PAIR_TABLE_ENTRY)), pairTable, sizeof(PAIR_TABLE_ENTRY)); 
}

/*****************************************************************************
 * void RF4CE_CB_NVMClear(void)
 *****************************************************************************/
void RF4CE_CB_NVMClear(void)
{
    //SST25SectorErase(RF4CE_NOR_NVM_START_ADDR);
}

/*****************************************************************************
 * void RF4CE_ControllerNVM_PutSelectedDevice(BYTE *pData)
 *****************************************************************************/
void RF4CE_ControllerNVM_PutSelectedDevice(BYTE *pData)
{
    //SST25WriteArray(RF4CE_NVM_ACTIVE_DEVICE_ADDRESS, pData, sizeof(RF4CE_SELECTED_DEVICE)); 
}

/*****************************************************************************
 * void RF4CE_ControllerNVM_GetSelectedDevice(BYTE *pData)
 *****************************************************************************/
void RF4CE_ControllerNVM_GetSelectedDevice(BYTE *pData)
{
    //SST25ReadArray(RF4CE_NVM_ACTIVE_DEVICE_ADDRESS, pData, sizeof(RF4CE_SELECTED_DEVICE)); 
}

/*****************************************************************************
 * void RF4CE_ControllerNVM_Clear(void)
 *****************************************************************************/
void RF4CE_ControllerNVM_Clear(void)
{
    //SST25SectorErase(RF4CE_NVM_ACTIVE_DEVICE_ADDRESS);
}
