/*****************************************************************************
 * FileName:        Controller.h
 * Dependencies:
 * Processor:       PIC18, PIC24
 * Compiler:        See Release Notes
 * Linker:          See Release Notes
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * Copyright  2010 Microchip Technology Inc.  All rights reserved.
 *
 * Microchip licenses to you the right to use, modify, copy and distribute 
 * Software only when embedded on a Microchip microcontroller or digital 
 * signal controller and used with a Microchip radio frequency transceiver, 
 * which are integrated into your product or third party product (pursuant 
 * to the terms in the accompanying license agreement).   
 *
 * You should refer to the license agreement accompanying this Software for 
 * additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY 
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A 
 * PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE 
 * LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, 
 * CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY 
 * DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO 
 * ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, 
 * LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, 
 * TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT 
 * NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *****************************************************************************
 *****************************************************************************/

#ifndef CONTROLLER_HEADER_FILE
#define CONTROLLER_HEADER_FILE
#define EDF_DEMO

#ifdef EDF_DEMO
int demo_main_loop(void);
#endif
typedef enum
{
    CONTROLLER_TV_NUMBER_0_CMD      = 0x20,
    CONTROLLER_TV_NUMBER_1_CMD      = 0x21,
    CONTROLLER_TV_NUMBER_2_CMD      = 0x22,
    CONTROLLER_TV_NUMBER_3_CMD      = 0x23,
    CONTROLLER_TV_NUMBER_4_CMD      = 0x24,
    CONTROLLER_TV_NUMBER_5_CMD      = 0x25,
    CONTROLLER_TV_NUMBER_6_CMD      = 0x26,
    CONTROLLER_TV_NUMBER_7_CMD      = 0x27,
    CONTROLLER_TV_NUMBER_8_CMD      = 0x28,
    CONTROLLER_TV_NUMBER_9_CMD      = 0x29,
                                    
    CONTROLLER_TV_VOL_UP_CMD        = 0x41,
    CONTROLLER_TV_VOL_DOWN_CMD      = 0x42,

    CONTROLLER_TV_CHANNEL_UP_CMD    = 0x30,
    CONTROLLER_TV_CHANNLE_DOWN_CMD  = 0x31,

    CONTROLLER_TV_PAIRING           = 0xFE

}CONTROLLER_TV_CMD;


void RF4CEDiscoveryCallBack(void);
void mTouch_ProcessKeys(void);
UINT8 MTouchGetPressReleaseButtonState(UINT8 buttonNumber);
/*****************************************************************************
 * void Get64BitNodeAddress(void)
 *****************************************************************************/
void RF4CE_CB_Get64BitNodeAddress(BYTE *address);

/*****************************************************************************
 * DWORD RF4CE_CB_GetFrameCounter(void)
 *****************************************************************************/
DWORD RF4CE_CB_GetFrameCounter(void);

/*****************************************************************************
 * void RF4CE_CB_PutFrameCounter(DWORD frameCounter)
 *****************************************************************************/
void RF4CE_CB_PutFrameCounter(DWORD frameCounter);

/*****************************************************************************
 * BYTE RF4CE_CB_GetDefaultChannel(void)
 *****************************************************************************/
BYTE RF4CE_CB_GetDefaultChannel(void);

/*****************************************************************************
 * void RF4CE_CB_PutDefaultChannel(BYTE defaultChannel)
 *****************************************************************************/
void RF4CE_CB_PutDefaultChannel(BYTE defaultChannel);

/*****************************************************************************
 * BYTE RF4CE_CB_GetPowerSaveMode(void)
 *****************************************************************************/
BYTE RF4CE_CB_GetPowerSaveMode(void);

/*****************************************************************************
 * void RF4CE_CB_PutPowerSaveMode(BYTE powerSaveMode)
 *****************************************************************************/
void RF4CE_CB_PutPowerSaveMode(BYTE powerSaveMode);

/*****************************************************************************
 * WORD RF4CE_CB_GetShortAddress(void)
 *****************************************************************************/
WORD RF4CE_CB_GetShortAddress(void);

/*****************************************************************************
 * void RF4CE_CB_PutShortAddress(WORD shortAddress)
 *****************************************************************************/
void RF4CE_CB_PutShortAddres(WORD shortAddress);

/*****************************************************************************
 * WORD RF4CE_CB_GetShortAddress(void)
 *****************************************************************************/
WORD RF4CE_CB_GetPanID(void);

/*****************************************************************************
 * void RF4CE_CB_PutShortAddress(WORD shortAddress)
 *****************************************************************************/
void RF4CE_CB_PutPanID(WORD panID);

/*****************************************************************************
 * void RF4CE_CB_GetPairTable(BYTE *pairTable)
 *****************************************************************************/
void RF4CE_CB_GetPairTable(BYTE *pairTable);

/*****************************************************************************
 * void RF4CE_CB_PutShortAddress(WORD shortAddress)
 *****************************************************************************/
void RF4CE_CB_PutPairTableIndex(BYTE *pairTable, WORD index);

/*****************************************************************************
 * void RF4CE_CB_NVMClear(void)
 *****************************************************************************/
void RF4CE_CB_NVMClear(void);

/*****************************************************************************
 * void RF4CE_ControllerNVM_PutSelectedDevice(BYTE *pData)
 *****************************************************************************/
void RF4CE_ControllerNVM_PutSelectedDevice(BYTE *pData);

/*****************************************************************************
 * void RF4CE_ControllerNVM_GetSelectedDevice(BYTE *pData)
 *****************************************************************************/
void RF4CE_ControllerNVM_GetSelectedDevice(BYTE *pData);

/*****************************************************************************
 * void RF4CE_ControllerNVM_Clear(void)
 *****************************************************************************/
void RF4CE_ControllerNVM_Clear(void);

/*************************************************************************
  Function:
           void RF4CE_CB_StatusReport(BYTE status, BYTE PairIndex)
    
  Summary:
    This is the callback function from the RF4CE stack to the application
    to report communication status, especially communication status in the
    pairing process.
  Description:
    This is the callback function from the RF4CE stack to the application
    to report communication status, especially in the pairing process.
    Application layer generally can ignore this status report, unless the
    status is not SUCCESS. In the case a communication fails during the
    pairing process, either because of timeout in the process of sending a
    command, or timeout to receive certain command, the status will be set
    to represent failure reason.
  Conditions:
    Transceiver initialization has been finished.
  Input:
    BYTE status -     The status of the communication status.
    BYTE PairIndex -  The index of the communication destination device in
                      the pair table
  Return:
    None
  Example:
    <code>
    void RF4CE_CB_StatusReport(BYTE status, BYTE PairIndex)
    {
        // handle the communication error if pairing process fail.
        if( status != SUCCESS )
        {
            // handle the error
        }
    }
    </code>
  Remarks:
    None                                                                  
  *************************************************************************/ 
void RF4CE_CB_StatusReport(BYTE status, BYTE PairIndex);


#endif
