/*****************************************************************************
 *
 *              HardwareProfile.c -- Hardware Configuration
 *
 *****************************************************************************
 * FileName:        HardwareProfile.c
 * Dependencies:
 * Processor:       PIC18F
 * Compiler:        C18 02.20.00 or higher
 *                  C30 02.03.00 or higher
 *                  C32 01.00.02 or higher
 * Linker:          MPLINK 03.40.00 or higher
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * Copyright  2007-2009 Microchip Technology Inc.  All rights reserved.
 *
 * Microchip licenses to you the right to use, modify, copy and distribute 
 * Software only when embedded on a Microchip microcontroller or digital 
 * signal controller and used with a Microchip radio frequency transceiver, 
 * which are integrated into your product or third party product (pursuant 
 * to the terms in the accompanying license agreement).   
 *
 * You should refer to the license agreement accompanying this Software for 
 * additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY 
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A 
 * PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE 
 * LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, 
 * CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY 
 * DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO 
 * ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, 
 * LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, 
 * TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT 
 * NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *****************************************************************************
 * File Description:
 *
 *   This file provides configuration and basic hardware functionality 
 *   based on chosen hardware demo boards.
 *
 * Change History:
 *  Rev         Date         Description
 *  0.1         2/17/2009    Initial revision
 *  2.0         04/24/2009   Modified for MiApp interface
 *  2.1         06/20/2009   Add LCD support
 *  1.0-1.1.2   12/10/2009   Modified for RF4CE
 *****************************************************************************/
#include "SystemProfile.h"
#include "Compiler.h"
#include "Common\SymbolTime.h"
#include "Common\LCDBlocking.h"
#include "Common\timer.h"
#include "Controller.h"
#include "OnePIC_mtouch.h"
#include "OnePIC_switch.h"
#include "bsp.h"
#include "OnePIC_switch.h"

// Configuration Bit Settings //
#if defined(__PIC24FJ256GB110__)
    _CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2)
    _CONFIG2(0xF7FF & IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL & PLLDIV_DIV2 & IOL1WAY_OFF)
#endif
#if defined(__PIC24FJ256GA110__)
    _CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & COE_OFF & FWDTEN_OFF & ICS_PGx2)
    _CONFIG2(IESO_OFF & FCKSM_CSDCMD & OSCIOFNC_OFF & POSCMOD_HS & FNOSC_PRIPLL & IOL1WAY_OFF)
#endif
#if defined(__PIC24FJ128GA010__)
    _CONFIG2(FNOSC_PRIPLL & POSCMOD_XT) // Primary XT OSC with PLL
    _CONFIG1(JTAGEN_OFF & FWDTEN_OFF)   // JTAG off, watchdog timer off
#endif
#if defined (__PIC24FJ256GB210__)
    _CONFIG1( WDTPS_PS32768 & FWPSA_PR128 & ALTVREF_ALTVREDIS & WINDIS_OFF & FWDTEN_OFF & ICS_PGx2 & GWRP_OFF & GCP_OFF & JTAGEN_OFF) 
    _CONFIG2( POSCMOD_HS & IOL1WAY_OFF & OSCIOFNC_OFF & OSCIOFNC_OFF & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
    _CONFIG3( WPFP_WPFP255 & SOSCSEL_SOSC & WUTSEL_LEG & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM) 
#endif
#if defined (__PIC24FJ256DA210__)
    _CONFIG1( WDTPS_PS32768 & FWPSA_PR128 & ALTVREF_ALTVREDIS & WINDIS_OFF & FWDTEN_OFF & ICS_PGx2 & GWRP_OFF & GCP_OFF & JTAGEN_OFF) 
    _CONFIG2( POSCMOD_HS & IOL1WAY_OFF & OSCIOFNC_OFF & OSCIOFNC_OFF & FCKSM_CSDCMD & FNOSC_PRIPLL & PLL96MHZ_ON & PLLDIV_DIV2 & IESO_OFF)
    _CONFIG3( WPFP_WPFP255 & SOSCSEL_SOSC & WUTSEL_LEG & ALTPMP_ALTPMPEN & WPDIS_WPDIS & WPCFG_WPCFGDIS & WPEND_WPENDMEM) 
#endif	

#ifdef __18CXX
//#pragma config FOSC = HSPLL, WDTEN = OFF, WDTPS = 16, XINST = ON
//#pragma romdata CONFIG1L = 0x300000

#pragma romdata CONFIG1H = 0x300001
const rom unsigned char config1H = 0b00001000;      // Internal Oscillator

#pragma romdata CONFIG2L = 0x300002
const rom unsigned char config2L = 0b00011000;      // Brown-out reset enabled @ 2.7V  //FRED

#pragma romdata CONFIG2H = 0x300003
const rom unsigned char config2H = 0b00001000;      // HW WDT disabled

//#pragma romdata CONFIG3L = 0x300004

#pragma romdata CONFIG3H = 0x300005
const rom unsigned char config3H = 0b00000100;      // 

#pragma romdata CONFIG4L = 0x300006
const rom unsigned char config4L = 0b11000000;      // 

//#pragma romdata CONFIG4H = 0x300007
//#pragma romdata CONFIG5L = 0x300008
#endif

/* User Profile will define the control code sent based on the button press.
*/
unsigned char UserTVProfile [4][8] = {
{0xff, 0x80, 0x0A, 0x21, 0x24, 0x27, 0x43, 0x42},
{0x81, 0x22, 0x25, 0x28, 0x20, 0x30, 0x2B, 0x31},
{0x40, 0x82, 0x0D, 0x23, 0x26, 0x29, 0x32, 0x41},
{0x35, 0xfD, 0xfD, 0xfD, 0xfD, 0x09, 0xfD, 0xfD}};

unsigned char UserDVDProfile [4][8] = {
{0xff, 0x80, 0xfD, 0xfD, 0xfD, 0xfD, 0xfD, 0xfD},
{0x81, 0xfD, 0xfD, 0xfD, 0xfD, 0xfD, 0xfD, 0xfD},
{0x40, 0x82, 0xfD, 0xfD, 0xfD, 0xfD, 0xfD, 0xfD},
{0xfD, 0x44, 0x46, 0x45, 0x49, 0xfD, 0x48, 0x47}};


/*********************************************************************
 * Function:        void BoardInit( void )
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    Board is initialized for P2P usage
 *
 * Overview:        This function configures the board 
 *
 * Note:            This routine needs to be called before the function 
 *                  to initialize P2P stack or any other function that
 *                  operates on the stack
 ********************************************************************/
void BoardInit(void)
{
	#if defined(__PIC24F__)
		CLKDIVbits.RCDIV = 0;		// Set 1:1 8MHz FRC postscalar
	#endif
	
	// ADC
    #if defined(__PIC24FJ256DA210__) || defined(__PIC24FJ256GB210__)
    	// Disable analog on all pins
    	ANSA = 0x0000;
    	ANSB = 0x0000;
    	ANSC = 0x0000;
    	ANSD = 0x0000;
    	ANSE = 0x0000;
    	ANSF = 0x0000;
    	ANSG = 0x0000;
    #endif
    #if defined(__PIC24FJ256GA106__) 
    	__builtin_write_OSCCONL(OSCCON & 0xBF);  // Unlock PPS
        #ifdef HARDWARE_SPI
            // Configure SPI1 PPS pins (ENC28J60/ENCX24J600/MRF24WB0M or other PICtail Plus cards)
            // RPOR0bits.RP0R = 8;		// Assign RP0 to SCK1 (output)
            // RPOR7bits.RP15R = 7;	// Assign RP15 to SDO1 (output)
            // RPINR20bits.SDI1R = 23;	// Assign RP23 to SDI1 (input)
            /*** SPI **********************************************************************/  
            // SDO1 RP23 n+1 = 23, n/2 = 11
            RPOR11bits.RP23R = 7; // SDO1
            // SDI1 RP24
            RPINR20bits.SDI1R = 24;
            // SCK
            RPOR9bits.RP18R = 8;
        #endif

        // Configure SPI2 PPS pins (25LC256 EEPROM on Explorer 16)
        //	RPOR10bits.RP21R = 11;	// Assign RG6/RP21 to SCK2 (output)
        //	RPOR9bits.RP19R = 10;	// Assign RG8/RP19 to SDO2 (output)
        //	RPINR22bits.SDI2R = 26;	// Assign RG7/RP26 to SDI2 (input)
        //	
        //	RPINR19bits.U2RXR = 10;	// Assign RF4/RP10 to U2RX (input)
        //	RPOR8bits.RP17R = 5;	// Assign RF5/RP17 to U2TX (output)
        //
        //	// Configure INT1 PPS pin (MRF24WB0M Wi-Fi PICtail Plus interrupt signal when in SPI slot 1)
        //	RPINR0bits.INT1R = 33;	// Assign RE8/RPI33 to INT1 (input)

        __builtin_write_OSCCONL(OSCCON | 0x40); // Lock PPS
    #endif

    
    // Setup LED
    LED_1RF = 0;
    LED_1RF_TRIS = 0;
    
    
    // Setup RF Module pins
    PHY_CS = 1;
    PHY_CS_TRIS = 0;
    PHY_RESETn = 1;
    PHY_RESETn_TRIS = 0;
    PHY_WAKE = 1;
    PHY_WAKE_TRIS = 0;
    
    // Set up the SPI
    
    SDI_TRIS = 1;
    SDO_TRIS = 0;
    SCK_TRIS = 0;
    
    #if defined(HARDWARE_SPI)
        RF_SPICON1 = 0x033B;    // Fcy Primary prescaler 1:1, secondary prescaler 2:1, CKP=0, CKE=1, SMP=1
        RF_SPICON2 = 0x0000;
        RF_SPISTAT = 0x8000;    // Enable the module
        //***SSPSTAT = 0xC0;
        //***SSPCON1 = 0x21;
    #else
        SPI_SDO = 0;        
        SPI_SCK = 0;    
    #endif   
   
    
    
    #ifdef USE_EXTERNAL_EEPROM
        EE_nCS_TRIS = 0;
        EE_nCS = 1;
    #endif
    
    RF_INT_PIN = 1;
    RF_INT_TRIS = 1;
    RFIE_EDGE = 1;
    
    RFIF = 0;                   // enable RF INT
    RFIE = 1;

    BUTTON_ROW_TRIS = 1;
    BUTTON_COL_TRIS = 1;
    BUTTON_CMD_TRIS = 1;
    BUTTON_PAIRING_TRIS = 1;

}


BYTE backupPLL;

/************************************************************************************
 * Function:
 *      void EnablePLL(void)
 *
 * Summary:
 *      This is the callback function from the RF4CE stack to enable PLL of MCU to run
 *      it full speed during secured pairing process.
 *
 * Description:        
 *      According to ZigBee RF4CE specification, for secured pairing, the reception side needs
 *      to streaming security key count number of seed packets to the joining device. Depending
 *      on the target device, the data rate for this seed procedure may be very high. Although
 *      the RF4CE stack already has build-in modules to recover from packet overflow, it is 
 *      still recommended that the joining device enables PLL to run higher speed in the short
 *      period of key seeding process. The RF4CE stack will call EnablePLL to enable PLL
 *      temporarily. After key seed procedure is over, callback function RestorePLL will be 
 *      called to restore to the original PLL setting. These functions are only need to implement
 *      if the device needs to actively paired with another device in security mode. In addition, 
 *      if the device runs with PLL already enabled, these two functions can be left unimplemented.
 *
 * PreCondition:    
 *      MCU has been properly initialized. 
 *
 * Parameters: 
 *      None
 *
 * Returns: 
 *      None
 *
 * Example:
 *      <code>
 *      BYTE backupPLL;
 *
 *      void EnablePLL(void)
 *      {
 *          backupPLL = OSCTUNEbits.PLLEN;
 *          OSCTUNEbits.PLLEN = 1;   
 *      }    
 *
 *      void RestorePLL(void)
 *      {
 *          OSCTUNEbits.PLLEN = backupPLL;
 *      } 
 *      </code>
 *
 * Remarks:    
 *      None
 *
 *****************************************************************************************/
void EnablePLL(void)
{
//    backupPLL = OSCTUNEbits.PLLEN;
//    OSCTUNEbits.PLLEN = 1;   
}    


/************************************************************************************
 * Function:
 *      void RestorePLL(void)
 *
 * Summary:
 *      This is the callback function from the RF4CE stack to enable PLL of MCU to run
 *      it full speed during secured pairing process.
 *
 * Description:        
 *      According to ZigBee RF4CE specification, for secured pairing, the reception side needs
 *      to streaming security key count number of seed packets to the joining device. Depending
 *      on the target device, the data rate for this seed procedure may be very high. Although
 *      the RF4CE stack already has build-in modules to recover from packet overflow, it is 
 *      still recommended that the joining device enables PLL to run higher speed in the short
 *      period of key seeding process. The RF4CE stack will call EnablePLL to enable PLL
 *      temporarily. After key seed procedure is over, callback function RestorePLL will be 
 *      called to restore to the original PLL setting. These functions are only need to implement
 *      if the device needs to actively paired with another device in security mode. In addition, 
 *      if the device runs with PLL already enabled, these two functions can be left unimplemented.
 *
 * PreCondition:    
 *      MCU has been properly initialized. 
 *
 * Parameters: 
 *      None
 *
 * Returns: 
 *      None
 *
 * Example:
 *      <code>
 *      BYTE backupPLL;
 *
 *      void EnablePLL(void)
 *      {
 *          backupPLL = OSCTUNEbits.PLLEN;
 *          OSCTUNEbits.PLLEN = 1;   
 *      }    
 *
 *      void RestorePLL(void)
 *      {
 *          OSCTUNEbits.PLLEN = backupPLL;
 *      } 
 *      </code>
 *
 * Remarks:    
 *      None
 *
 *****************************************************************************************/
void RestorePLL(void)
{
//    OSCTUNEbits.PLLEN = backupPLL;
}  

void BoostModeInit(void)
{
    // Setup PWM
    // Setup Timer2
    // TRIS C2 drives FET
    
    
    /***  Look into adding this back in with some better understanding of the PWM
    PSTRCON = 0x01;
    PR2 = 0x13;
    CCP1CON = 0x0C;
    CCPR1L = 0x0A;
    PIR1bits.TMR2IF = 0;
    T2CON = 0x00;
    T2CONbits.TMR2ON = 1;
    while(PIR1bits.TMR2IF = 0)
    PIR1bits.TMR2IF = 0;
    
    // Setup Comparator for AutoShutdown
    
    ECCP1AS = 0x10;     // C1 autoshutdown, drive P1A low!
    PWM1CON = 0x80;     // Auto clear the shutdown
    CM1CON0 = 0x96;     // C1Vref+, C12IN2-, ON
    CM2CON1 = 0x00;     // 1.2 FVR
    
    TRISCbits.TRISC2 = 0;       // Turn it ON.
    ****/
}


unsigned char ButtonPress(void)
{
    unsigned char command = 0x02;
    static unsigned char row = 0;
    static unsigned char column = 0;
    static unsigned char rowPress = 0;
    static unsigned char colPress = 0;
    static unsigned char cmdPress = 0;
    
    if(PUSH_BUTTON_ROW == 0)
    {
        rowPress = 1;
    }else
    {
        if(rowPress)
            row++;

        rowPress = 0;
    }

    if(PUSH_BUTTON_COL == 0)
    {
        colPress = 1;
    }else
    {
        if(colPress)
            column++;

        colPress = 0;
    }

    if(row >= 4)
        row = 0;

    if(column >= 8)
        column = 0;

        if(PUSH_BUTTON_CMD == 0)
    {
        cmdPress = 1;
    }else
    {
        if(cmdPress)
            command = UserTVProfile[row][column];

        cmdPress = 0;
    }
    
//    if((PUSH_BUTTON_CMD == 0) && (PUSH_BUTTON_ROW == 0))
     
    if(switchPressed())
    {
        command = 0xFE;
    } 
    return command;
            
}    
/*
CONTROLLER_TV_CMD GetTerminalCommand()
{
    char rxChar;
    CONTROLLER_TV_CMD cmd = 0xFF;

    if(IFS1bits.U2RXIF == 0)
        return cmd;

    rxChar = U2RXREG;
    IFS1bits.U2RXIF = 0;

    if((rxChar >= '0') && (rxChar <= '9'))
    {
        rxChar -= '0';

        cmd = CONTROLLER_TV_NUMBER_0_CMD + rxChar;
    }    

    if(rxChar == 'q')
        cmd = CONTROLLER_TV_VOL_UP_CMD;

    if(rxChar == 'a')
        cmd = CONTROLLER_TV_VOL_DOWN_CMD;
    
    if(rxChar == '+')
        cmd = CONTROLLER_TV_CHANNEL_UP_CMD;

    if(rxChar == '-')
        cmd = CONTROLLER_TV_CHANNLE_DOWN_CMD;
    
    if(rxChar == 'p')
        cmd = CONTROLLER_TV_PAIRING;

    return cmd;
}*/


CONTROLLER_TV_CMD GetTerminalCommand()
{
    char rxChar;
    CONTROLLER_TV_CMD cmd = 0xFF;

/*    if(IFS1bits.U2RXIF == 0)
    {
            return cmd;
    }
    rxChar = U2RXREG;
    IFS1bits.U2RXIF = 0;
    
*/
/*    
    if(MTouchGetButtonState(CAP_UP) == MTOUCH_PRESSED)
    {
        cmd = CONTROLLER_TV_VOL_UP_CMD;
    }    
    if((rxChar >= '0') && (rxChar <= '9'))
    {
        rxChar -= '0';

        cmd = CONTROLLER_TV_NUMBER_0_CMD + rxChar;
    }    

    if(rxChar == 'q')
        

    if(rxChar == 'a')
        cmd = CONTROLLER_TV_VOL_DOWN_CMD;
    
    if(rxChar == '+')
        cmd = CONTROLLER_TV_CHANNEL_UP_CMD;

    if(rxChar == '-')
        cmd = CONTROLLER_TV_CHANNLE_DOWN_CMD;
    
    if(rxChar == 'p')
        cmd = CONTROLLER_TV_PAIRING;

*/

    static uint8_t next = 0;
    switch(next++)
    {
        case 1: cmd = CONTROLLER_TV_VOL_UP_CMD; break;
        case 2: cmd = CONTROLLER_TV_VOL_DOWN_CMD; break; 
        case 3: cmd = CONTROLLER_TV_CHANNLE_DOWN_CMD; break;
        case 4: cmd = CONTROLLER_TV_CHANNEL_UP_CMD; break;
        default: next = 0;
    }
    return cmd;
}
