/*****************************************************************************
 *
 *              HardwareProfile.h -- Hardware Configuration 
 *
 *****************************************************************************
 * FileName:        HardwareProfile.h
 * Dependencies:
 * Processor:       PIC18F
 * Compiler:        C18 v3.20 or higher
 * Linker:          MPLINK 3.40 or higher
 * Company:         Microchip Technology Incorporated
 *
 * Software License Agreement
 *
 * Copyright  2007-2009 Microchip Technology Inc.  All rights reserved.
 *
 * Microchip licenses to you the right to use, modify, copy and distribute 
 * Software only when embedded on a Microchip microcontroller or digital 
 * signal controller and used with a Microchip radio frequency transceiver, 
 * which are integrated into your product or third party product (pursuant 
 * to the terms in the accompanying license agreement).   
 *
 * You should refer to the license agreement accompanying this Software for 
 * additional information regarding your rights and obligations.
 *
 * SOFTWARE AND DOCUMENTATION ARE PROVIDED AS IS WITHOUT WARRANTY OF ANY 
 * KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY 
 * WARRANTY OF MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A 
 * PARTICULAR PURPOSE. IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE 
 * LIABLE OR OBLIGATED UNDER CONTRACT, NEGLIGENCE, STRICT LIABILITY, 
 * CONTRIBUTION, BREACH OF WARRANTY, OR OTHER LEGAL EQUITABLE THEORY ANY 
 * DIRECT OR INDIRECT DAMAGES OR EXPENSES INCLUDING BUT NOT LIMITED TO 
 * ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR CONSEQUENTIAL DAMAGES, 
 * LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, 
 * TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES (INCLUDING BUT 
 * NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *****************************************************************************
 * File Description:
 *
 *   This file provides configuration and basic hardware functionality 
 *   based on chosen hardware demo boards.
 *
 * Change History:
 *  Rev         Date         Description
 *  0.1         2/17/2009    Initial revision
 *  2.0         04/24/2009   Modified for MiApp interface
 *  2.1         06/20/2009   Add LCD support
 *  1.0-1.1.2   12/10/2009   Modified for RF4CE
 *****************************************************************************/
#ifndef _HARDWARE_PROFILE_H
    #define _HARDWARE_PROFILE_H

    #include "GenericTypeDefs.h"
    #include "ConfigApp.h"
        
    void BoardInit(void);
    void BoostModeInit(void);
    void ControllerPowerDown(void);
    void ControllerPowerUp(void);
    unsigned char ButtonPress(void);


    #define CLOCK_FREQ 32000000

    // Transceiver Configuration

    // Transceiver Configuration
    #define RFIF        IFS1bits.INT1IF
    #define RFIE        IEC1bits.INT1IE
    #define RFIE_EDGE   INTCON2bits.INT1EP
    #define RF_INT_PIN  PORTGbits.RG2
    #define RF_INT_TRIS TRISGbits.TRISG2
    
    #define PHY_CS          LATEbits.LATE7
    #define PHY_CS_TRIS     TRISEbits.TRISE7
    #define PHY_RESETn      LATEbits.LATE4
    #define PHY_RESETn_TRIS TRISEbits.TRISE4
    #define PHY_WAKE        LATEbits.LATE6
    #define PHY_WAKE_TRIS   TRISEbits.TRISE6


    #define SPI_SDI     PORTDbits.RD2               
    #define SDI_TRIS    TRISDbits.TRISD2
    #define SPI_SDO     LATDbits.LATD1               
    #define SDO_TRIS    TRISDbits.TRISD1
    #define SPI_SCK     LATBbits.LATB5               
    #define SCK_TRIS    TRISBbits.TRISB5

    #define RF_SPICON1      SPI1CON1
    #define RF_SPICON2      SPI1CON2
    #define RF_SPISTAT      SPI1STAT

    
    #define PUSH_BUTTON_ROW   PORTDbits.RD6
    #define BUTTON_ROW_TRIS   TRISDbits.TRISD6
   
    #define PUSH_BUTTON_COL   PORTDbits.RD7
    #define BUTTON_COL_TRIS   TRISDbits.TRISD7

    #define PUSH_BUTTON_CMD   PORTDbits.RD9
    #define PUSH_BUTTON_RESET PORTFbits.RF3
    #define BUTTON_CMD_TRIS   TRISDbits.TRISD10

    #define PUSH_BUTTON_PAIRING PORTDbits.RD7
    #define BUTTON_PAIRING_TRIS TRISDbits.TRISD7

    #define LED_1RF       LATDbits.LATD11
    #define LED_1RF_TRIS  TRISDbits.TRISD11
        
    #define EEPROM_CS_LAT   LATGbits.LATG0 = LATDbits.LATD6
    #define EEPROM_CS_TRIS  TRISGbits.TRISG0 = TRISDbits.TRISD6

    #ifdef USE_EXTERNAL_EEPROM
        #define EE_nCS_TRIS         TRISFbits.TRISF1       
        #define EE_nCS              LATFbits.LATF1
    #endif
    
    #define TMRL TMR1

#endif
