/**
 * @file OnePIC_32bit_int.c
 * @brief Controls all interrupt related tasks for the 32bit such as:
 *  - Enable interrupt
 *  - Disable interrupt
 *  - Clear interrupt flag
 *  - Set interrupt flag
 *
 * @defgroup thirtytwo_bit 32bit
 * @{
 */

#include "Compiler.h"
#include "OnePIC_int.h"

/**
 * @brief enables a specific interrupt
 * @param source the source of the interrupt to enable
 */
void EDF_INT_SourceEnable ( EDF_INT_SOURCE source )
{
    switch(source)
    {
        case INT_TMR_1:
            IEC0bits.T1IE = 1;
            break;
        default:
            break;
    }
}

/**
 *
 * @param source
 * @return a bool indicating if the interrupt is enabled or not
 */
bool EDF_INT_SourceEnableGet ( EDF_INT_SOURCE source )
{
    switch(source)
    {
        case INT_TMR_1:
            return IEC0bits.T1IE;
            break;
        default:
            break;
    }
}
/**
 * @brief disables a specific interrupt
 * @param source interrupt source to disable
 */
void EDF_INT_SourceDisable ( EDF_INT_SOURCE source )
{
    switch(source)
    {
        case INT_TMR_1:
            IEC0bits.T1IE = 0;
            break;
        default:
            break;
    }
}
/**
 * @brief clears a specific interrupt
 * @param source interrupt flag to clear
 */
void EDF_INT_SourceFlagClear ( EDF_INT_SOURCE source )
{
    switch(source)
    {
        case INT_TMR_1:
            IFS0bits.T1IF = 0;
            break;
        default:
            break;
    }
}
/**
 * @brief retrieves the interrupt flag
 * @param source source of interrupt flag to get
 * @return bool indicating if flag is set or not
 */
bool EDF_INT_SourceFlagGet ( EDF_INT_SOURCE source )
{
    switch(source)
    {
        case INT_TMR_1:
            return IFS0bits.T1IF;
            break;
        default:
            break;
    }
}

/**
 * @brief
 */
void EDF_INT_Enable(void) {
}

/**
 * @brief
 */
void EDF_INT_Disable(void) {
}


