/**
 * @file OnePIC_32bit_mtouch.c
 * @brief Initilizes the 5 touch buttons as well as board init
 * @defgroup thirtytwo_bit_mtouch mTouch
 * @ingroup thirtytwo_bit
 * @{
 */

#include "OnePIC_mtouch.h"
#include "Compiler.h"
#include "mTouchCapAPI.h"
#include "OnePIC_lcd.h"
#include "mTouchCapLED.h"
#include "OnePIC_timer.h"
#include "OnePIC_int.h"
#include "mTouchCapApp_DirectKeys.h"


/**
 * @brief init the board and touch buttons
 */
void EDF_mTouchInit(void)
{

    DDPCONbits.JTAGEN = 0; // Disable JTAG

    // Initalize global interrupt enable
    INTEnableSystemMultiVectoredInt();

    // Configure the device for maximum performance
    // Given the options, this function will change the flash wait states, RAM
    // wait state and enable prefetch cache but will not change the PBDIV.
    // The PBDIV value is already set via the pragma FPBDIV option.
    SYSTEMConfig(SYS_FREQ, SYS_CFG_WAIT_STATES | SYS_CFG_PCACHE);

    // Setup data structures for using Direct Keys
    mTouchCapApp_DirectKeys_Create();

    mTouchCapAPI_Init();

    // Start button measurements
    Set_ScanTimer_IE_Bit_State(ENABLE);  //Enable interrupt
    Set_ScanTimer_ON_Bit_State(ENABLE);  //Run timer
}
/** @}*/