/**
 *
 * @file OnePIC_32bit_pot.c
 * @brief configures the potentiometer and takes a reading
 * @defgroup thirtytwo_bit_pot Potentiometer
 * @ingroup thirtytwo_bit
 * @{
 */


#include "OnePIC_pot.h"
#include "TimeDelay.h"
#include "bsp.h"
#include "OnePIC_int.h"
#include "Compiler.h"

#define FILTER_MEDIAN_LENGTH 3
#define FILTER_MIDDLE_VALUE (FILTER_MEDIAN_LENGTH-1)/2


/**
 * @brief compares two values
 * @param e1
 * @param e2
 * @return bool indicating which is greater:
 *  - -1 -> v1 < v2
 *  - 1  -> v1 = v2
 *  - 0  v1 > v2
 */
static int comp(const void *e1, const void *e2)
{
    const int * a1 = e1;
    const int * a2 = e2;
    if (*a1 < *a2)
        return -1;
    else if (*a1 == *a2)
        return 0;
    else
        return 1;
}
/**
 * @brief no init needed for 32
 */
void EDF_potInit(void)
{
}  

/**
 * @brief experiencing some dropped readings, so the median filter takes three samples and passes the median of the three
 * @param d adc value
 * @return 16bit average of the ADC
 */
uint16_t filterMedian(uint16_t d)
{
    static uint16_t mVal[FILTER_MEDIAN_LENGTH];
    static uint8_t mIndex = 0; 
        
    mVal[mIndex++] = d;
    
    if(mIndex == FILTER_MEDIAN_LENGTH) {
        mIndex = 0;
    }
     
    qsort(mVal, FILTER_MEDIAN_LENGTH, sizeof(uint16_t), comp);
    
    return mVal[FILTER_MIDDLE_VALUE];
}

/**
 * @brief takes the reading
 * @return the 16bit digital representation of the analog voltage
 */
uint16_t EDF_potRead(void)
{
    uint16_t filteredPotReading;
    AD1CON1bits.ON = 0;
    TRISBbits.TRISB11 = 1;
    asm("di"); // disable interrupts
    AD1CHSbits.CH0SA = 11; 
    AD1CON1bits.ON = 1;
    //Bring up ADC in known state
    AD1CON1bits.SAMP = 1;        //Start Chold charging Process
    asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop");
    asm("nop"); asm("nop"); asm("nop"); asm("nop");
    AD1CON1bits.SAMP = 0; //Start ADC conversion
    asm("ei"); //re-enable interrupts
    while ( !AD1CON1bits.DONE );
    AD1CON1bits.DONE = 0;
    filteredPotReading = ADC1BUF0;
    return filteredPotReading;
}
/**@}*/

