/**
 * @brief board specific defines here
 * @file 32bit/include/bsp.h
 */


#ifndef __BSP_H
#define __BSP_H
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include "OnePIC_lcd.h"
#include "OnePIC_pot.h"
#include "OnePIC_switch.h"

#include "TimeDelay.h"

#define LED_ON  1
#define LED_OFF 0

#include <p32xxxx.h>
#include <plib.h>
#include <sys/attribs.h>

// Clocks
#define GetSystemClock()     20000000UL
#define SYS_FREQ            (20000000UL)
#define GetPeripheralClock()        (GetSystemClock())
#define GetInstructionClock()       (GetSystemClock())

// ADC channels available
#define MAX_ADC_CHANNELS 16

// Key to ADC Channel assignment
#define DIRECTKEY1_CHANNEL  CHANNEL_AN0
#define DIRECTKEY2_CHANNEL  CHANNEL_AN1
#define DIRECTKEY3_CHANNEL  CHANNEL_AN2
#define DIRECTKEY4_CHANNEL  CHANNEL_AN3
#define DIRECTKEY5_CHANNEL  CHANNEL_AN4

// GPIO initialization values
#define INIT_TRISB_VALUE 0x0000
#define INIT_TRISC_VALUE 0x0000
#define INIT_TRISD_VALUE 0x0000
#define INIT_TRISE_VALUE 0x0000
#define INIT_TRISF_VALUE 0x0000
#define INIT_TRISG_VALUE 0x0000

#define INIT_PORTB_VALUE 0x0000
#define INIT_PORTC_VALUE 0x0000
#define INIT_PORTD_VALUE 0x0000
#define INIT_PORTE_VALUE 0x0000
#define INIT_PORTF_VALUE 0x0000 // Not used
#define INIT_PORTG_VALUE 0x0000

/******************************************************************************
* ADC
******************************************************************************/
#define ADC_BUFFER						
#define ADC_OPERATING_MODE      	AD1CON1bits.ON
#define ADC_STOP_IN_IDLE        	AD1CON1bits.SIDL
#define ADC_DATA_OUTPUT_FORMAT  	AD1CON1bits.FORM
#define ADC_CONV_SOURCE         	AD1CON1bits.SSRC
#define ADC_AUTO_SAMPLE         	AD1CON1bits.ASAM
#define ADC_SAMPLE_ENABLE       	AD1CON1bits.SAMP
#define ADC_CONV_DONE           	AD1CON1bits.DONE

#define ADC_VOLTAGE_REF_CONFIG  	AD1CON2bits.VCFG
#define ADC_MUX_A_CH_SEL        	AD1CON2bits.CSCNA
#define ADC_BUFFER_FILL_STAT    	AD1CON2bits.BUFS
#define ADC_SAMPLE_SEQ_INT_SEL  	AD1CON2bits.SMPI
#define ADC_BUFFER_MODE_SEL     	AD1CON2bits.BUFM
#define ADC_ALT_SAMPLE_MODE     	AD1CON2bits.ALTS

#define ADC_CONV_CLK_SRC        	AD1CON3bits.ADRC
#define ADC_AUTO_SAMPLE_TIME    	AD1CON3bits.SAMC
#define ADC_CONV_CLK_PERIOD_SEL 	AD1CON3bits.ADCS


/****************************************************************************** 
* ADC
* *****************************************************************************/
// 10/5/2011 10:25:22 AM: PIC32 ADC:
/*** POT RB11 AN11    ***********************************************************/
#define ADC_POT_IO_BIT      11
#define ADC_POT_IO_CH_BIT   11
#define ADC_POT_LATCH       LATB
#define ADC_POT_TRIS        TRISB
/*** MIC RB8 AN8   ***********************************************************/
#define ADC_MIC_IO_BIT      8
#define ADC_MIC_IO_CH_BIT   8
#define ADC_POT_LATCH       LATB
#define ADC_POT_TRIS        TRISB
/*** UP RB4 AN4    ***********************************************************/
#define ADC_UP_IO_BIT       4
#define ADC_UP_IO_CH_BIT    4
#define ADC_POT_LATCH       LATB
#define ADC_POT_TRIS        TRISB
/*** DOWN RB1 AN1  ***********************************************************/
#define ADC_DOWN_IO_BIT     1
#define ADC_DOWN_IO_CH_BIT  1
#define ADC_POT_LATCH       LATB
#define ADC_POT_TRIS        TRISB
/*** LEFT RB0 AN0  ***********************************************************/
#define ADC_LEFT_IO_BIT     0
#define ADC_LEFT_IO_CH_BIT  0
#define ADC_POT_LATCH       LATB
#define ADC_POT_TRIS        TRISB
/*** RIGHT RB3 AN3 ***********************************************************/
#define ADC_RIGHT_IO_BIT    3   
#define ADC_RIGHT_IO_CH_BIT 3  
#define ADC_POT_LATCH       LATB
#define ADC_POT_TRIS        TRISB
/*** ENTER RB2 AN2 ***********************************************************/
#define ADC_ENTER_IO_BIT    2   
#define ADC_ENTER_IO_CH_BIT 2  
#define ADC_POT_LATCH       LATB
#define ADC_POT_TRIS        TRISB

/******************************************************************************
 * SPI
 ******************************************************************************/ 
#define SPI1_BUFFER                              
#define SPI1_ENABLE                 SPI2CONbits.ON
#define SPI1_STOP_IN_IDLE           SPI2CONbits.SIDL
// XXX 10/10/2011 3:13:33 PM: 24/32 have different buffer structers, use simplest non-enhanced mode
#define SPI1_BUFFER_ELEMENT_CNT     
#define SPI1_SR_EMPTY               SPI2STATbits.SRMT 
#define SPI1_RX_OVERFLOW            SPI2STATbits.SPIROV
#define SPI1_RX_FIFO_EMPTY          SPI2STATbits.RXBUFELM
#define SPI1_INT_MODE_SEL           // XXX 10/10/2011 3:15:12 PM Different int selections, use Polled mode
#define SPI1_TX_BUFFER_FULL         SPI2STATbits.SPITBF
#define SPI1_RX_BUFFER_FULL         SPI2STATbits.SPIRBF

#define SPI1_SCK_DISABLE            
#define SPI1_SDO_DISABLE            SPI2CONbits.DISSDO
// XXX 10/10/2011 2:35:36 PM:       PIC32x has 8/16/32 option, PIC24 has 8/16bit options, config not the same
#define SPI1_MODE16_SEL             
#define SPI1_SAMPLE_PHASE           SPI2CONbits.SMP
#define SPI1_CLK_EDGE_SEL           SPI2CONbits.CKE
#define SPI1_SS_ENABLE              SPI2CONbits.SSEN
#define SPI1_CLK_POL_SEL            SPI2CONbits.CKP
#define SPI1_MASTER_ENABLE          SPI2CONbits.MSTEN
#define SPI1_SEC_PRESCALE           
#define SPI1_PRI_PRESCALE           
        
#define SPI1_FRAMED_MODE_ENABLE      SPI2CONbits.FRMEN
#define SPI1_FRAME_SYNC_PULSE_DIR    SPI2CONbits.FRMSYNC
#define SPI1_FRAME_SYNC_PULSE_POL    SPI2CONbits.FRMPOL
#define SPI1_FRAME_SYNC_PULSE_EDGE   SPI2CONbits.SPIFE
#define SPI1_ENHANCE_BUFFER_ENABLE   SPI2CONbits.ENHBUF

// NA on PIC24:
// XXX 10/10/2011 2:41:02 PM: Master Mode Slave Select enable
// XXX 10/10/2011 2:41:17 PM: FRame sync pulse width
// XXX 10/10/2011 2:51:32 PM: FRZ
// XXX 10/10/2011 2:57:34 PM: TX buffer interrupt mode sel
// XXX 10/10/2011 2:57:49 PM: RX buffer int mode sel
/**********************************************************************
 * LEDs
 **********************************************************************/
#define gpLED_8_TRIS TRISDbits.TRISD6
#define gpLED_7_TRIS TRISDbits.TRISD5
#define gpLED_6_TRIS TRISDbits.TRISD12
#define gpLED_5_TRIS TRISDbits.TRISD11
#define gpLED_4_TRIS TRISDbits.TRISD10
#define gpLED_3_TRIS TRISDbits.TRISD9
#define gpLED_2_TRIS TRISDbits.TRISD8
#define gpLED_1_TRIS TRISDbits.TRISD7

#define gpLED_8 LATDbits.LATD6
#define gpLED_7 LATDbits.LATD5
#define gpLED_6 LATDbits.LATD12
#define gpLED_5 LATDbits.LATD11
#define gpLED_4 LATDbits.LATD10
#define gpLED_3 LATDbits.LATD9
#define gpLED_2 LATDbits.LATD8
#define gpLED_1 LATDbits.LATD7

#define gpLED_UP_TRIS      TRISCbits.TRISC1 
#define gpLED_DOWN_TRIS    TRISCbits.TRISC4 
#define gpLED_RIGHT_TRIS   TRISCbits.TRISC2 
#define gpLED_LEFT_TRIS    TRISCbits.TRISC3 

#define gpLED_UP      LATCbits.LATC1 
#define gpLED_DOWN    LATCbits.LATC4 
#define gpLED_RIGHT   LATCbits.LATC2 
#define gpLED_LEFT    LATCbits.LATC3 

/**********************************************************************
* Switch
**********************************************************************/
#define SW1_TRIS        gpLED_1_TRIS /**<@brief shared with LED1*/
#define SW1_PORT         PORTDbits.RD7 /**<@brief */
#define SW1_LATCH        LATDbits.LATD7 /**<@brief */
/**********************************************************************
* I2C
**********************************************************************/
#define I2C_MODULE_ENABLE               I2C1CONbits.ON 
#define I2C_DISABLE_SLEW_CONTROL        I2C1CONbits.DISSLW
#define I2C_INTERRUPT_FLAG 	            IFS0bits.I2C1MIF
#define I2C_INTERRUPT_ENABLE            IEC0bits.I2C1MIE
#define I2C_BRG                         I2C1BRG
#define I2C_BCL_MASTER_BUS_COLLISION 	I2C1STATbits.BCL
#define I2C_SEN_START_CONDITION_ENABLE  I2C1CONbits.SEN
#define I2C_TRANSMIT_REGISTER           I2C1TRN
#define I2C_RECIEVE_REGISTER            I2C1RCV
#define I2C_REPEATED_START_ENABLE       I2C1CONbits.RSEN
#define I2C_ACK_STATUS                  I2C1STATbits.ACKSTAT
#define I2C_STOP_CONDITION_ENABLE       I2C1CONbits.PEN
#define I2C_RECIEVE_ENABLE              I2C1CONbits.RCEN
#define I2C_ACK_DATA                    I2C1CONbits.ACKDT
#define I2C_ACK_SEQ_ENABLE              I2C1CONbits.ACKEN

// 10/10/2011 2:14:02 PM removed TRIS/Data pins for SDA/SCL
// 10/5/2011 10:42:28 AM bits not needed in p32
/**********************************************************************
* LCD
**********************************************************************/
#define LCD_RS          LATEbits.LATE9
#define LCD_RS_TRIS	    TRISEbits.TRISE9
    
#define LCD_ENABLE      LATBbits.LATB5
#define LCD_ENABLE_TRIS TRISBbits.TRISB5

#define LCD_D0          LATEbits.LATE0
#define LCD_D1          LATEbits.LATE1
#define LCD_D2          LATEbits.LATE2
#define LCD_D3          LATEbits.LATE3

#define LCD_D0_TRIS     TRISEbits.TRISE0
#define LCD_D1_TRIS     TRISEbits.TRISE1
#define LCD_D2_TRIS     TRISEbits.TRISE2
#define LCD_D3_TRIS     TRISEbits.TRISE3

/**********************************************************************
* mTouch
**********************************************************************/
#define EDF_CapTouchGetButtonState(i)  0//MTouchGetButtonState(i)
#define EDF_MTOUCH_PRESSED 1
/******************************************************************************
* PTP
* *****************************************************************************/
#define PTP1_LAT    LATGbits.LATG15
#define PTP2_LAT    LATAbits.LATA0
#define PTP3_LAT    LATEbits.LATE4
#define PTP4_LAT    LATEbits.LATE5
#define PTP5_LAT    LATEbits.LATE6
#define PTP6_LAT    LATEbits.LATE7
#define PTP7_LAT    LATGbits.LATG9
#define PTP8_LAT    LATGbits.LATG1
#define PTP9_LAT    LATGbits.LATG0
#define PTP10_LAT   LATGbits.LATG14
#define PTP11_LAT   LATGbits.LATG12
#define PTP12_LAT   LATBbits.LATB12
#define PTP13_LAT   LATBbits.LATB13
#define PTP14_LAT   // NOT AVAILABLE   
#define PTP15_LAT   // NOT AVAILABLE   
#define PTP16_LAT   LATDbits.LATD14
#define PTP17_LAT   LATDbits.LATD15
#define PTP18_LAT   LATBbits.LATB14
#define PTP19_LAT   LATBbits.LATB15
#define PTP20_LAT   LATEbits.LATE8
#define PTP21_LAT   LATGbits.LATG13

#define PTP1_TRIS   TRISGbits.TRISG15
#define PTP2_TRIS   TRISAbits.TRISA0
#define PTP3_TRIS   TRISEbits.TRISE4
#define PTP4_TRIS   TRISEbits.TRISE5
#define PTP5_TRIS   TRISEbits.TRISE6
#define PTP6_TRIS   TRISEbits.TRISE7
#define PTP7_TRIS   TRISGbits.TRISG9
#define PTP8_TRIS   TRISGbits.TRISG1
#define PTP9_TRIS   TRISGbits.TRISG0
#define PTP10_TRIS  TRISGbits.TRISG14
#define PTP11_TRIS  TRISGbits.TRISG12
#define PTP12_TRIS  TRISBbits.TRISB12
#define PTP13_TRIS  TRISBbits.TRISB13
#define PTP14_TRIS  // NOT AVAILABLE   
#define PTP15_TRIS  // NOT AVAILABLE   
#define PTP16_TRIS  TRISDbits.TRISD14
#define PTP17_TRIS  TRISDbits.TRISD15
#define PTP18_TRIS  TRISBbits.TRISB14
#define PTP19_TRIS  TRISBbits.TRISB15
#define PTP20_TRIS  TRISEbits.TRISE8
#define PTP21_TRIS  TRISGbits.TRISG13

#define PTP1_PORT   PORTGbits.RG15
#define PTP2_PORT   PORTAbits.RA0
#define PTP3_PORT   PORTEbits.RE4
#define PTP4_PORT   PORTEbits.RE5
#define PTP5_PORT   PORTEbits.RE6
#define PTP6_PORT   PORTEbits.RE7
#define PTP7_PORT   PORTGbits.RG9
#define PTP8_PORT   PORTGbits.RG1
#define PTP9_PORT   PORTGbits.RG0
#define PTP10_PORT  PORTGbits.RG14
#define PTP11_PORT  PORTGbits.RG12
#define PTP12_PORT  PORTBbits.RB12
#define PTP13_PORT  PORTBbits.RB13
#define PTP14_PORT  // NOT AVAILABLE   
#define PTP15_PORT  // NOT AVAILABLE   
#define PTP16_PORT  PORTDbits.RD14
#define PTP17_PORT  PORTDbits.RD15
#define PTP18_PORT  PORTBbits.RB14
#define PTP19_PORT  PORTBbits.RB15
#define PTP20_PORT  PORTEbits.RE8
#define PTP21_PORT  PORTGbits.RG13




#endif //__BSP_H
