/******************************************************************************
* File Name:       mTouchCapAdc.c
* Includes:
*   void mTouchCapADC_InitADC(void )
*
* Dependencies:    None
* Processor:       PIC32MX795F512H
* Compiler:        C32
* Company:         Microchip Technology, Inc.
* Software License Agreement
*
* Copyright  2011 Microchip Technology Inc.
* Microchip licenses this software to you solely for use with Microchip
* products, according to the terms of the accompanying click-wrap software
* license. Microchip and its licensors retain all right, title and interest in
* and to the software.  All rights reserved. This software and any accompanying
* information is for suggestion only. It shall not be deemed to modify
* Microchips standard warranty for its products.  It is your responsibility to
* ensure that this software meets your requirements.
*
* SOFTWARE IS PROVIDED AS IS WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESS OR
* IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF MERCHANTABILITY, TITLE,
* NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT WILL
* MICROCHIP OR ITS LICENSORS BE LIABLE FOR ANY DIRECT OR INDIRECT DAMAGES OR
* EXPENSES INCLUDING BUT NOT LIMITED TO INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE
* OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
* SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, OR ANY CLAIMS BY THIRD PARTIES
* (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
*
* The aggregate and cumulative liability of Microchip and its licensors for
* damages related to the use of the software will in no event exceed $1000 or
* the amount you paid Microchip for the software, whichever is greater.
*
* MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF THESE
* TERMS AND THE TERMS OF THE ACCOMPANYING CLICK-WRAP SOFTWARE LICENSE.
*
* Author    Date          Comments
*--------------------------------------------------------------------------------
* MC        22-Jan-2010   First release of ADCDriver.c, ADCDriver.h
*                         Porting for PIC32MX795F512H
* MWM       28 Mar 2011   Removed ADC ISR, capture now in Timer 4 ISR.
********************************************************************************/
#include "mTouchConfig.h"
#include "HardwareProfile.h"
#include "GenericTypeDefs.h"
#include "mTouchCapADC.h"
#include "mTouchCapAPI.h"

#define ADC_CHAN_SELECT AD1CHS
#define ADC_PORT_CONFIG AD1PCFG

UINT16 PortPinADC[MAX_ADC_CHANNELS] = { 0xB0, 0xB1, 0xB2, 0xB3,
                                        0xB4, 0xB5, 0xB6, 0xB7,
                                        0xB8, 0xB9, 0xBA, 0xBB,
                                        0xBC, 0xBD, 0xBE, 0xBF };


void mTouchCapADC_InitADC(void)
{
//*****************************************************************************
// ADC Setup
//*****************************************************************************

    // configure and enable the ADC
    AD1CON1 = 0x0;

    /* do not scan inputs */
    /* interrupt at completion of each conversion */
    /* always use MUXA */
    AD1CON2 = 0x0;

    /* clock derived from periph clock */
    /* Tadc = 2*(AD1CON<7:0>+1)*Tpbus */
    /* Tadc = 2*(   2       +1)*Tpbus */
    /* Tadc = 6 * Tpbus = 6 * 12.5 ns = 75 ns > 65 ns required */
    AD1CON3 = 0x0002;

    /* reset value for channel select at this point */
    ADC_CHAN_SELECT = 0x0;

    ADC_PORT_CONFIG =
              ~( (1<<DIRECTKEY1_CHANNEL) |
                 (1<<DIRECTKEY2_CHANNEL) |
                 (1<<DIRECTKEY3_CHANNEL) |
                 (1<<DIRECTKEY4_CHANNEL) |
                 (1<<DIRECTKEY5_CHANNEL) );

    /* no channel is scanned */
    AD1CSSL = 0x0;
    /* DIS-ENABLE interrupt */
    IEC1bits.AD1IE = 0;
}

