 /*********************************************************************
 *
 *	Hardware specific definitions for:
 *    - Explorer 16
 *    - PIC32MX
 *    - MRF24WB0MA Wi-Fi PICtail Plus
 *
 *********************************************************************
 * FileName:        HardwareProfile.h
 * Dependencies:    Compiler.h
 * Processor:       PIC32
 * Compiler:        Microchip C32 v1.11 or higher
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * Copyright (C) 2002-2010 Microchip Technology Inc.  All rights
 * reserved.
 *
 * Microchip licenses to you the right to use, modify, copy, and
 * distribute:
 * (i)  the Software when embedded on a Microchip microcontroller or
 *      digital signal controller product ("Device") which is
 *      integrated into Licensee's product; or
 * (ii) ONLY the Software driver source files ENC28J60.c, ENC28J60.h,
 *		ENCX24J600.c and ENCX24J600.h ported to a non-Microchip device
 *		used in conjunction with a Microchip ethernet controller for
 *		the sole purpose of interfacing with the ethernet controller.
 *
 * You should refer to the license agreement accompanying this
 * Software for additional information regarding your rights and
 * obligations.
 *
 * THE SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT
 * WARRANTY OF ANY KIND, EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT
 * LIMITATION, ANY WARRANTY OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT SHALL
 * MICROCHIP BE LIABLE FOR ANY INCIDENTAL, SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF
 * PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS
 * BY THIRD PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE
 * THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER
 * SIMILAR COSTS, WHETHER ASSERTED ON THE BASIS OF CONTRACT, TORT
 * (INCLUDING NEGLIGENCE), BREACH OF WARRANTY, OR OTHERWISE.
 *
 *
 * Author               Date		Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Howard Schlunder		09/16/2010	Regenerated for specific boards
 ********************************************************************/
#ifndef HARDWARE_PROFILE_H
#define HARDWARE_PROFILE_H

#include "Compiler.h"

// Define a macro describing this hardware set up (used in other files)
#define EXPLORER_16

// Set configuration fuses (but only in MainDemo.c where THIS_IS_STACK_APPLICATION is defined)
#if defined(THIS_IS_STACK_APPLICATION)
	#pragma config FPLLODIV = DIV_1, FPLLMUL = MUL_20, FPLLIDIV = DIV_2, FWDTEN = OFF, FPBDIV = DIV_1, POSCMOD = HS, FNOSC = FRCPLL, CP = OFF
#endif


// Clock frequency values
// These directly influence timed events using the Tick module.  They also are used for UART and SPI baud rate generation.
#define GetSystemClock()		(80000000ul)			// Hz
#define GetInstructionClock()	(GetSystemClock()/1)	// Normally GetSystemClock()/4 for PIC18, GetSystemClock()/2 for PIC24/dsPIC, and GetSystemClock()/1 for PIC32.  Might need changing if using Doze modes.
#define GetPeripheralClock()	(GetSystemClock()/1)	// Normally GetSystemClock()/4 for PIC18, GetSystemClock()/2 for PIC24/dsPIC, and GetSystemClock()/1 for PIC32.  Divisor may be different if using a PIC32 since it's configurable.

//#define MRF24WB0M_IN_SPI1
#define MRF24WB0M_IN_SPI2

// Hardware I/O pin mappings
#if defined (MRF24WB0M_IN_SPI1)
// LEDs
#define LED0_TRIS			(TRISAbits.TRISA0)	// Ref D3
#define LED0_IO				(LATAbits.LATA0)
#define LED1_TRIS			(TRISAbits.TRISA1)	// Ref D4
#define LED1_IO				(LATAbits.LATA1)
#define LED2_TRIS			(TRISAbits.TRISA2)	// Ref D5
#define LED2_IO				(LATAbits.LATA2)
#define LED3_TRIS			(TRISAbits.TRISA3)	// Ref D6
#define LED3_IO				(LATAbits.LATA3)
#define LED4_TRIS			(TRISAbits.TRISA4)	// Ref D7
#define LED4_IO				(LATAbits.LATA4)
#define LED5_TRIS			(TRISAbits.TRISA5)	// Ref D8
#define LED5_IO				(LATAbits.LATA5)
#define LED6_TRIS			(TRISAbits.TRISA6)	// Ref D9
#define LED6_IO				(LATAbits.LATA6)
#define LED7_TRIS			(LATAbits.LATA7)	// Ref D10;  Note: This is multiplexed with BUTTON1, so this LED can't be used.  However, it will glow very dimmly due to a weak pull up resistor.
#define LED7_IO				(LATAbits.LATA7)
#define LED_GET()			(*((volatile unsigned char*)(&LATA)))
#define LED_PUT(a)			(*((volatile unsigned char*)(&LATA)) = (a))

#else
// LEDs
#define LED0_TRIS			(TRISDbits.TRISD5)	// Ref D3
#define LED0_IO				(LATDbits.LATD5)
#define LED1_TRIS			(TRISDbits.TRISD6)	// Ref D4
#define LED1_IO				(LATDbits.LATD6)
#define LED2_TRIS			(TRISDbits.TRISD7)	// Ref D5
#define LED2_IO				(LATDbits.LATD7)
#define LED3_TRIS			(TRISDbits.TRISD8)	// Ref D6
#define LED3_IO				(LATDbits.LATD8)
#define LED4_TRIS			(TRISDbits.TRISD9)	// Ref D7
#define LED4_IO				(LATDbits.LATD9)
#define LED5_TRIS			(TRISDbits.TRISD10)	// Ref D8
#define LED5_IO				(LATDbits.LATD10)
#define LED6_TRIS			(TRISDbits.TRISD11)	// Ref D9
#define LED6_IO				(LATDbits.LATD11)
#define LED7_TRIS			(TRISDbits.TRISD12)	// Ref D10;  Note: This is multiplexed with BUTTON1, so this LED can't be used.  However, it will glow very dimmly due to a weak pull up resistor.
#define LED7_IO				(LATDbits.LATD12)
#define LED_GET()			1//(*((volatile unsigned char*)(&LATD)))
#define LED_PUT(a)			Nop()//(*((volatile unsigned char*)(&LATD))= (a))
#endif
// Momentary push buttons
#define BUTTON0_TRIS		(TRISDbits.TRISD7)	// Ref S4
#define	BUTTON0_IO			(PORTDbits.RD7)
#define BUTTON1_TRIS		(TRISDbits.TRIS7)	// Ref S5;  Note: This is multiplexed with LED7
#define	BUTTON1_IO			(PORTDbits.RD7)
#define BUTTON2_TRIS		(TRISDbits.TRISD7)	// Ref S6
#define	BUTTON2_IO			(PORTDbits.RD7)
#define BUTTON3_TRIS		(TRISDbits.TRISD7)	// Ref S3
#define	BUTTON3_IO			(PORTDbits.RD7)

#if defined(MRF24WB0M_IN_SPI1) && (defined(__32MX460F512L__) || defined(__32MX795F512L__))
	// MRF24WB0M in SPI1 slot
	#define WF_CS_TRIS			(TRISDbits.TRISD9)
	#define WF_CS_IO			(LATDbits.LATD9)
	#define WF_SDI_TRIS			(TRISCbits.TRISC4)
	#define WF_SCK_TRIS			(TRISDbits.TRISD10)
	#define WF_SDO_TRIS			(TRISDbits.TRISD0)
	#define WF_RESET_TRIS		(TRISFbits.TRISF0)
	#define WF_RESET_IO			(LATFbits.LATF0)
	#define WF_INT_TRIS			(TRISEbits.TRISE8)  // INT1
	#define WF_INT_IO			(PORTEbits.RE8)
	#define WF_HIBERNATE_TRIS	(TRISFbits.TRISF1)
	#define WF_HIBERNATE_IO		(PORTFbits.RF1)
	#define WF_INT_EDGE			(INTCONbits.INT1EP)
	#define WF_INT_IE			(IEC0bits.INT1IE)
	#define WF_INT_IF			(IFS0bits.INT1IF)
	#define WF_INT_IE_CLEAR		IEC0CLR
	#define WF_INT_IF_CLEAR		IFS0CLR
	#define WF_INT_IE_SET		IEC0SET
	#define WF_INT_IF_SET		IFS0SET
	#define WF_INT_BIT			0x00000080
	#define WF_INT_IPCSET		IPC1SET
	#define WF_INT_IPCCLR		IPC1CLR
	#define WF_INT_IPC_MASK		0xFF000000
	#define WF_INT_IPC_VALUE	0x0C000000

	#define WF_SSPBUF			(SPI1BUF)
	#define WF_SPISTAT			(SPI1STAT)
	#define WF_SPISTATbits		(SPI1STATbits)
	#define WF_SPICON1			(SPI1CON)
	#define WF_SPICON1bits		(SPI1CONbits)
	#define WF_SPI_IE_CLEAR		IEC0CLR
	#define WF_SPI_IF_CLEAR		IFS0CLR
	#define WF_SPI_INT_BITS		0x03800000
	#define WF_SPI_BRG			(SPI1BRG)
	#define WF_MAX_SPI_FREQ		(10000000ul)	// Hz

#elif defined(MRF24WB0M_IN_SPI2) && (defined(__32MX460F512L__) || defined(__32MX795F512L__))
    #define WF_CS_TRIS	        (TRISBbits.TRISB14) //EDF
	#define WF_CS_IO			(LATBbits.LATB14)   //EDF
	#define WF_SDI_TRIS			(TRISGbits.TRISG7)  //EDF 
	#define WF_SCK_TRIS			(TRISGbits.TRISG6)  //EDF
	#define WF_SDO_TRIS			(TRISGbits.TRISG8)  //EDF
	#define WF_RESET_TRIS		(TRISAbits.TRISA14) //EDF
	#define WF_RESET_IO			(LATAbits.LATA14)   //EDF
	#define WF_INT_TRIS			(TRISEbits.TRISE8)  // INT1 EDF
	#define WF_INT_IO			(PORTEbits.RE8)     //EDF
	#define WF_HIBERNATE_TRIS	(TRISAbits.TRISA15) //EDF
	#define WF_HIBERNATE_IO		(PORTAbits.RA15)    //EDF
	#define WF_INT_EDGE			(INTCONbits.INT1EP)
	#define WF_INT_IE			(IEC0bits.INT1IE)
	#define WF_INT_IF			(IFS0bits.INT1IF)
	#define WF_INT_IE_CLEAR		IEC0CLR
	#define WF_INT_IF_CLEAR		IFS0CLR
	#define WF_INT_IE_SET		IEC0SET
	#define WF_INT_IF_SET		IFS0SET
	#define WF_INT_BIT			0x00000080
	#define WF_INT_IPCSET		IPC1SET
	#define WF_INT_IPCCLR		IPC1CLR
	#define WF_INT_IPC_MASK		0xFF000000
	#define WF_INT_IPC_VALUE	0x0C000000

	#define WF_SSPBUF			(SPI2BUF)
	#define WF_SPISTAT			(SPI2STAT)
	#define WF_SPISTATbits		(SPI2STATbits)
	#define WF_SPICON1			(SPI2CON)
	#define WF_SPICON1bits		(SPI2CONbits)
	#define WF_SPI_IE_CLEAR		 IEC1CLR     //EDF
	#define WF_SPI_IF_CLEAR		 IFS1CLR     //EDF
	#define WF_SPI_INT_BITS		0x000000E0  //EDF
	#define WF_SPI_BRG			(SPI2BRG)
	#define WF_MAX_SPI_FREQ		(10000000ul)	// Hz
#endif


// Select which UART the STACK_USE_UART and STACK_USE_UART2TCP_BRIDGE 
// options will use.  You can change these to U1BRG, U1MODE, etc. if you 
// want to use the UART1 module instead of UART2.
#define UBRG				U1BRG
#define UMODE				U1MODE
#define USTA				U1STA
#define BusyUART()			BusyUART2()
#define CloseUART()			CloseUART2()
#define ConfigIntUART(a)	ConfigIntUART2(a)
#define DataRdyUART()		DataRdyUART2()
#define OpenUART(a,b,c)		OpenUART2(a,b,c)
#define ReadUART()			ReadUART2()
#define WriteUART(a)		WriteUART2(a)
#define getsUART(a,b,c)		getsUART2(a,b,c)
#define putsUART(a)			putsUART2(a)
#define getcUART()			getcUART2()
#define putcUART(a)			do{while(BusyUART()); WriteUART(a); while(BusyUART()); }while(0)
#define putrsUART(a)		Nop()//(a)//putrsUART2(a)


#endif // #ifndef HARDWARE_PROFILE_H
