/**
 *
 * @file OnePIC_8bit_pot.c
 * @brief configures the potentiometer and takes a reading
 * @defgroup eight_bit_pot Potentiometer
 * @ingroup eight_bit
 * @{
 */
#include "OnePIC_pot.h"
#include <htc.h>
#include "bsp.h"
#include "TimeDelay.h"
#include "OnePIC_int.h"
/**
 * @brief compares two values
 * @param e1
 * @param e2
 * @return bool indicating which is greater:
 *  - -1 -> v1 < v2
 *  - 1  -> v1 = v2
 *  - 0  v1 > v2
 */
static int comp(const void *e1, const void *e2)
{
    const int * a1 = e1;
    const int * a2 = e2;
    if (*a1 < *a2)
        return -1;
    else if (*a1 == *a2)
        return 0;
    else
        return 1;
}

/**
 * @brief Initilizes the POT to take a reading
 * @note must call this every time before processing ADC due to mTouch configureing ADCON
 */
void EDF_potInit(void)
{
    ADC_CONTROL_REGISTER0 = 0b00110100; //AN13 = POT :: ANXX = MIC
    ADC_CONTROL_REGISTER1 = 0b11010000; //Right justified using Vdd/Vss as reference voltages
    ADC_ON;
    DelayMs(5); //init delay 
}  


/**
 * @brief takes the reading
 * @return the 16bit digital representation of the analog voltage
 * @note ADC result is 10bits wide
 */
uint16_t EDF_potRead(void)
{
    volatile uint16_t pot_voltage;

    EDF_INT_Disable(); //Turn off interrupts
    EDF_potInit(); //need to init again because of mTouch
    GO = 1; //start the ADC
    while (ADC_WAITING) {
        Nop();
    } //Wait for conversion to complete
    pot_voltage = ADC_RESULT_HIGH; //grab the top 2 MSBs
    pot_voltage = pot_voltage << 8; //make room for lower 8 bits
    pot_voltage |= ADC_RESULT_LOW;
    EDF_INT_Enable(); //entable


    return pot_voltage;
}
/** @}*/
