/**
 * @file 8bit/include/bsp.h
 * @brief all board specifics are defined in here:
 *  - LEDs
 *  - i2c
 *  - mtouch
 *  - switch button
 *  - UART
 *  - PWM
 *  - ADC
 * @ingroup eight_bit
 * @{
 */
#ifndef __BSP_H
#define __BSP_H

#include "Compiler.h"
#include <stdint.h>
#include <stdbool.h>
#define LED_ON  1 /**<@brief */
#define LED_OFF 0 /**<@brief */

#define Nop() NOP() /**<@brief */

#define _XTAL_FREQ   32000000 /**<@brief */

#define GetSystemClock()            32000000UL /**<@brief */
#define GetPeripheralClock()        32000000UL /**<@brief */
#define GetInstructionClock()       (GetSystemClock() / 4)  /**<@brief */

/**********************************************************************
 * LEDs
 **********************************************************************/
#define gpLED_8_TRIS TRISAbits.TRISA5 /**<@brief */
#define gpLED_7_TRIS TRISAbits.TRISA5 /**<@brief */
#define gpLED_6_TRIS TRISAbits.TRISA5 /**<@brief */
#define gpLED_5_TRIS TRISAbits.TRISA4 /**<@brief */
#define gpLED_4_TRIS TRISAbits.TRISA3 /**<@brief */
#define gpLED_3_TRIS TRISAbits.TRISA2 /**<@brief */
#define gpLED_2_TRIS TRISAbits.TRISA1 /**<@brief */
#define gpLED_1_TRIS TRISAbits.TRISA0 /**<@brief */

#define gpLED_8 LATAbits.LATA5 /**<@brief */
#define gpLED_7 LATAbits.LATA5 /**<@brief */
#define gpLED_6 LATAbits.LATA5 /**<@brief */
#define gpLED_5 LATAbits.LATA4 /**<@brief */
#define gpLED_4 LATAbits.LATA3 /**<@brief */
#define gpLED_3 LATAbits.LATA2 /**<@brief */
#define gpLED_2 LATAbits.LATA1 /**<@brief */
#define gpLED_1 LATAbits.LATA0 /**<@brief */

#define gpLED_UP_TRIS      TRISDbits.TRISD4 /**<@brief */
#define gpLED_DOWN_TRIS    TRISDbits.TRISD7 /**<@brief */
#define gpLED_RIGHT_TRIS   TRISDbits.TRISD6 /**<@brief */
#define gpLED_LEFT_TRIS    TRISDbits.TRISD5 /**<@brief */

#define gpLED_UP      LATDbits.LATD4 /**<@brief */
#define gpLED_DOWN    LATDbits.LATD7 /**<@brief */
#define gpLED_RIGHT   LATDbits.LATD6 /**<@brief */
#define gpLED_LEFT    LATDbits.LATD5 /**<@brief */

/**********************************************************************
 * Switch
 **********************************************************************/
#define SW1_TRIS    gpLED_1_TRIS /**<@brief */
#define SW1_PORT         PORTAbits.RA0 /**<@brief */
#define SW1_LATCH        LATAbits.LATA0 /**<@brief */
/**********************************************************************
 * I2C
 **********************************************************************/
#define I2C_SCL_TRIS TRISCbits.TRISC3 /**<@brief */
#define I2C_SDA_TRIS TRISCbits.TRISC4 /**<@brief */

#define I2C_MODULE_SETUP_MASTER_3       SSPCON1bits.SSPM3 /**<@brief */
#define I2C_MODULE_SETUP_MASTER_2       SSPCON1bits.SSPM2 /**<@brief */
#define I2C_MODULE_SETUP_MASTER_1       SSPCON1bits.SSPM1 /**<@brief */
#define I2C_MODULE_SETUP_MASTER_0       SSPCON1bits.SSPM0 /**<@brief */
#define I2C_MODULE_ENABLE               SSPCON1bits.SSPEN /**<@brief */
#define I2C_DISABLE_SLEW_CONTROL        SSPSTATbits.SMP /**<@brief */
#define I2C_INTERRUPT_FLAG 	        PIR1bits.SSPIF /**<@brief */
#define I2C_INTERRUPT_ENABLE            PIE1bits.SSPIE /**<@brief */
#define I2C_BRG                         SSPADD /**<@brief */
#define I2C_BCL_MASTER_BUS_COLLISION    SSPCON1bits.WCOL  /**<@brief *///only write conditions to SSPBUF
#define I2C_SEN_START_CONDITION_ENABLE  SSPCON2bits.SEN /**<@brief */
#define I2C_TRANSMIT_REGISTER           SSPBUF /**<@brief */
#define I2C_RECIEVE_REGISTER            SSPBUF /**<@brief */
#define I2C_REPEATED_START_ENABLE       SSPCON2bits.RSEN /**<@brief */
#define I2C_ACK_STATUS                  SSPCON2bits.ACKSTAT; /**<@brief */
#define I2C_STOP_CONDITION_ENABLE       SSPCON2bits.PEN /**<@brief */
#define I2C_RECIEVE_ENABLE              SSPCON2bits.RCEN /**<@brief */
#define I2C_ACK_DATA                    SSPCON2bits.ACKDT /**<@brief */
#define I2C_ACK_SEQ_ENABLE              SSPCON2bits.ACKEN /**<@brief */
/**********************************************************************
 * LCD
 **********************************************************************/
#define LCD_RS          LATCbits.LATC0 /**<@brief */
#define LCD_RS_TRIS	TRISCbits.TRISC0 /**<@brief */

#define LCD_ENABLE      LATAbits.LATA6 /**<@brief */
#define LCD_ENABLE_TRIS TRISAbits.TRISA6 /**<@brief */

#define LCD_D0 LATDbits.LATD0 /**<@brief */
#define LCD_D1 LATDbits.LATD1 /**<@brief */
#define LCD_D2 LATDbits.LATD2 /**<@brief */
#define LCD_D3 LATDbits.LATD3 /**<@brief */

#define LCD_D0_TRIS TRISDbits.TRISD0 /**<@brief */
#define LCD_D1_TRIS TRISDbits.TRISD1 /**<@brief */
#define LCD_D2_TRIS TRISDbits.TRISD2 /**<@brief */
#define LCD_D3_TRIS TRISDbits.TRISD3 /**<@brief */
/**********************************************************************
 * mTouch
 **********************************************************************/
#define EDF_CapTouchGetButtonState(i)  mTouch_GetButtonState(i) /**<@brief */
#define EDF_MTOUCH_PRESSED MTOUCH_PRESSED /**<@brief */

/**********************************************************************
 * UART
 **********************************************************************/
#define UART_TX_INTERRUPT  PIR1bits.TXIF /**<@brief */
#define UART_RX_INTERRUPT  PIR1bits.RCIF /**<@brief */
#define UART_TX_REGISTER   TXREG /**<@brief */
#define UART_RX_REGISTER   RCREG /**<@brief */

#define UART_TRIS_RX        TRISCbits.TRISC7 /**<@brief */
#define UART_TRIS_TX        TRISCbits.TRISC6 /**<@brief */

#define UART_RX        PORTCbits.RC7 /**<@brief */
#define UART_TX        PORTCbits.RC6 /**<@brief */

#define UART_BAUD_REGISTER  SPBRG /**<@brief */
#define UART_RC_STATUS      TXSTA /**<@brief UART RX*/
#define UART_TX_STATUS      RCSTA /**<@brief UART tx*/


///**********************************************************************
// * PWM
// **********************************************************************/
//#define     TMR2_REGISTER       T2CON /**<@brief TMR2 controller*/
//#define     TMR2_TIME           PR2 /**<@brief period register*/
//#define     TMR2_INTERRUPT      PIE3bits.TMR6IE /**<@brief */
//#define     PWM2_CONTROL_REGISTER       CCP2CON /**<@brief */
//#define     PWM2_TIME_REGISTER       CCPR2L /**<@brief */

/**********************************************************************
 * ADC
 **********************************************************************/
#define     ADC_ON                    ADON = 1; /**<@brief turn on the ADC*/
#define     ADC_CONTROL_REGISTER0     ADCON0 /**<@brief ADC control register0*/
#define     ADC_CONTROL_REGISTER1     ADCON1 /**<@brief ADC control register1*/
#define     ADC_RESULT_HIGH           ADRESH /**<@brief upper 2 bits of result*/
#define     ADC_RESULT_LOW             ADRESL /**<@brief lower 8 bits of result*/
#define     ADC_WAITING             ADCON0bits.GO /**<@brief wait for conversion*/

#define PTP1_LAT   LATCbits.LATC5  /**<@brief IR PICtail pin*/
#define PTP2_LAT   LATCbits.LATC1  /**<@brief IR PICtail pin*/
#define PTP3_LAT   LATEbits.LATE1  /**<@brief IR PICtail pin*/

#define PTP1_PORT  PORTCbits.RC5  /**<@brief IR PICtail pin*/
#define PTP2_PORT  PORTCbits.RC1  /**<@brief IR PICtail pin*/
#define PTP3_PORT  PORTEbits.RE1  /**<@brief IR PICtail pin*/

#define PTP1_TRIS  TRISCbits.TRISC5  /**<@brief IR PICtail pin*/
#define PTP2_TRIS  TRISCbits.TRISC1  /**<@brief IR PICtail pin*/
#define PTP3_TRIS  TRISEbits.TRISE1  /**<@brief IR PICtail pin*/

#endif // BSP_H
/** @}*/