/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_HardwareProfile_16F184x.h
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Hardware Profile Header File
 *                   - Provides information about correct SFR addresses,
 *                     pin assignments, module availability, and any other
 *                     part-specific data.
 *                   - See the documentation for more information.
 *************************************************************************/
 /**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file mTouch_HardwareProfile_12F184x.h
* @brief Support for: PIC12F1840, PIC12LF1840
*/
/// @cond
#if !defined(__MTOUCH_HARDWARE_12F184X_H)
#define __MTOUCH_HARDWARE_12F184X_H

#if !defined(_12F1840) && !defined(_12LF1840)
    #error The current mTouch hardware include file does not support this PIC microcontroller.
#endif

#if defined(_12F1840) || defined(_12LF1840)
    #if MTOUCH_NUMBER_SENSORS > 4
        #error The PIC12F/LF1840 is not able to support more than 4 sensors due to pinout limitations.
    #endif
    #if MTOUCH_NUMBER_OF_SLEEP_SENSORS > 4
        #error The PIC12F/LF1840 is not able to support more than 4 sleep-mode sensors due to pinout limitations.
    #endif
#endif

#if (defined(MTOUCH_V1_COMMS) && !defined(MTOUCH_V1_COMM_BITBANGED)) || defined(MTOUCH_V2_COMMS)
#if !defined(APFCON_INITIALIZED)
    #warning The mTouch comm module does not automatically set the APFCON# register. The TX pin may be selectable. This warning can be removed by defining the precompiler variable 'APFCON_INITIALIZED' in the mTouch_config.h file - verifying that you've initialized this register.
#endif
#endif

#define MTOUCH_DEBUG_UART_AVAILABLE

#define MTOUCH_DEBUG_COMM_INIT()                                       \
    do {                                                            \
        TXSTA       = MTOUCH_DEBUG_TXSTA;                              \
        RCSTA       = MTOUCH_DEBUG_RCSTA;                              \
        BAUDCON     = MTOUCH_DEBUG_BAUDCON;                            \
        SPBRGL      = MTOUCH_DEBUG_SPBRGL;                             \
        SPBRGH      = MTOUCH_DEBUG_SPBRGH;                             \
    } while (0)
   
#define MTOUCH_DEBUG_COMM_TXIF             TXIF
#define MTOUCH_DEBUG_COMM_TXREG            TXREG

#define MTOUCH_TIMER0_AVAILABLE         
#define MTOUCH_TIMER1_AVAILABLE         
#define MTOUCH_TIMER2_AVAILABLE   
#define MTOUCH_SET_GIE()                   GIE    = 1    

#define MTOUCH_SET_ADC_CLK_32MHZ()         ADCON1 = 0b10100000 // 32
#define MTOUCH_SET_ADC_CLK_16MHZ()         ADCON1 = 0b11010000 // 16
#define MTOUCH_SET_ADC_CLK_8MHZ()          ADCON1 = 0b10010000 // 8
#define MTOUCH_SET_ADC_CLK_4MHZ()          ADCON1 = 0b11000000 // 4
#define MTOUCH_SET_ADC_CLK_2MHZ()          ADCON1 = 0b10000000 // 2
#define MTOUCH_SET_ADC_CLK_1MHZ()          ADCON1 = 0b10000000 // 2

#define CVD_UNIMPLEMENTED_AVAILABLE     1
#define MTOUCH_ADC_BITS                 10

#if defined(MTOUCH_V1_COMMS)
#if !(MTOUCH_V1_COMM_SPEED == 9600 || MTOUCH_V1_COMM_SPEED == 115200)
    #error MTOUCH_V1_COMM_SPEED must be set to either 9600 or 115200 in the configuration file.
#endif
#endif

#if     _XTAL_FREQ == 32000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_32MHZ()    
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    68
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100000
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00000000
        #define MTOUCH_DEBUG_SPBRGL    12
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00000000
        #define MTOUCH_DEBUG_SPBRGL    207
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ == 16000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_16MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    33
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    103
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00000000
        #define MTOUCH_DEBUG_SPBRGL    103
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  8000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_8MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    16
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    51
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    207
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  4000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_4MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    8
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    25
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    103
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  2000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_2MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #error The 115.2kbps UART baudrate option cannot be used with a 2MHz Fosc. Please select '9600' or '38400'.
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    12
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    51
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  1000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_1MHZ()
    #define CVD_TAD             2
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #error The 115.2kbps UART baudrate option cannot be used with a 1MHz Fosc. Please select '9600' or '38400'.
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00000000
        #define MTOUCH_DEBUG_SPBRGL    5
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    25
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#else
    #error _XTAL_FREQ is not set to a valid value for this processor.
#endif


#define CVD_ADCON0_BANK     1
#define CVD_DACOUT_AVAILABLE

#define CVD_DACOUT_TRIS_C               TRISA0
#define CVD_DACOUT_LAT_C                LATA0
#define CVD_DACOUT_LAT                  _LATA
#define CVD_DACOUT_PIN                  0

#define CVD_DACCON0_VDD                 0xC0
#define CVD_DACCON1_VDD                 0x1F
#define CVD_DACCON0_VSS                 0x80
#define CVD_DACCON1_VSS                 0x00

#define CVD_DACOUT_DACCON0_VSS          0xA0
#define CVD_DACOUT_DACCON0_1_3RD_VDD    0xA0
#define CVD_DACOUT_DACCON0_HALF_VDD     0xA0
#define CVD_DACOUT_DACCON0_2_3RD_VDD    0xE0
#define CVD_DACOUT_DACCON0_VDD          0xE0

#define CVD_DACOUT_DACCON1_VSS          0x00
#define CVD_DACOUT_DACCON1_1_3RD_VDD    0x0B
#define CVD_DACOUT_DACCON1_HALF_VDD     0x10
#define CVD_DACOUT_DACCON1_2_3RD_VDD    0x16
#define CVD_DACOUT_DACCON1_VDD          0x1F

#define MTOUCH_WDT_OFF()                   SWDTEN = 0
#define MTOUCH_WDT_ON()                    SWDTEN = 1

// A/D MUX selection for each A/D button, do not change this, refer to datasheet if curious how these are derived
// Right Justified, Vdd as reference, A/D on, Go/Done  asserted, do not change this, refer to datasheet if curious how these are derived 
#define MTOUCH_AD_AN0           0x01
#define MTOUCH_AD_AN1           0x05
#define MTOUCH_AD_AN2           0x09
#define MTOUCH_AD_AN3           0x0D

#define MTOUCH_AD_FVR_AND_GO    0x7F // Selects the FVR as the ADC mux option and sets the GO/DONE bit.
#define MTOUCH_AD_DAC_AND_GO    0x7B // Selects the DAC as the ADC mux option and sets the GO/DONE bit.
#define MTOUCH_AD_DAC_NOGO      0x79 // Selects the DAC as the ADC mux option w/o setting the GO/DONE bit.
#define MTOUCH_AD_ISO_AND_GO    0x43 // Selects an unimplemented, isolated ADC mux option and sets the GO/DONE bit.
#define MTOUCH_AD_ISO_NOGO      0x41 // Selects an unimplemented, isolated ADC mux option w/o setting the GO/DONE bit.

#define MTOUCH_PIN_AN0          0
#define MTOUCH_PIN_AN1          1
#define MTOUCH_PIN_AN2          2
#define MTOUCH_PIN_AN3          4

#define MTOUCH_LETTER_AN0         A
#define MTOUCH_LETTER_AN1         A
#define MTOUCH_LETTER_AN2         A
#define MTOUCH_LETTER_AN3         A


#endif
/// @endcond

