/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_HardwareProfile_16F152x.h
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Hardware Profile Header File
 *                   - Provides information about correct SFR addresses,
 *                     pin assignments, module availability, and any other
 *                     part-specific data.
 *                   - See the documentation for more information.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file   mTouch_HardwareProfile_16F152x.h
*   @brief  Support for: PIC16F1526, PIC16F1527
*/
/// @cond
#if !defined(__MTOUCH_HARDWARE_16F152X_H)
#define __MTOUCH_HARDWARE_16F152X_H

#if !defined(_16F1526) && !defined(_16LF1526)
#if !defined(_16F1527) && !defined(_16LF1527)
    #error The current mTouch hardware include file does not support this PIC microcontroller.
#endif
#endif

#if defined(_16F1526) || defined(_16LF1526)
    #if MTOUCH_NUMBER_SENSORS > 22
        #error The PIC16F/LF1526 is not able to support more than 22 sensors due to memory limitations.
    #endif
#endif
#if defined(_16F1527) || defined(_16LF1527)
    #if MTOUCH_NUMBER_SENSORS > 22
        #error The PIC16F/LF1527 is not able to support more than 22 sensors due to memory limitations.
    #endif
#endif

#define MTOUCH_DEBUG_UART_AVAILABLE

#define MTOUCH_DEBUG_COMM_INIT()                                       \
    do {                                                            \
        TXSTA       = MTOUCH_DEBUG_TXSTA;                              \
        RCSTA       = MTOUCH_DEBUG_RCSTA;                              \
        BAUDCON     = MTOUCH_DEBUG_BAUDCON;                            \
        SPBRGL      = MTOUCH_DEBUG_SPBRGL;                             \
        SPBRGH      = MTOUCH_DEBUG_SPBRGH;                             \
    } while (0)

#define MTOUCH_DEBUG_COMM_TXIF             TXIF
#define MTOUCH_DEBUG_COMM_TXREG            TXREG
    

#define MTOUCH_TIMER0_AVAILABLE         
#define MTOUCH_TIMER1_AVAILABLE         
#define MTOUCH_TIMER3_AVAILABLE         
#define MTOUCH_TIMER5_AVAILABLE         
#define MTOUCH_TIMER2_AVAILABLE         
#define MTOUCH_TIMER4_AVAILABLE      
#define MTOUCH_TIMER6_AVAILABLE     
#define MTOUCH_TIMER8_AVAILABLE     
#define MTOUCH_TIMER10_AVAILABLE
#define MTOUCH_SET_GIE()                   GIE    = 1  
    
#define MTOUCH_SET_ADC_CLK_32MHZ()         ADCON1 = 0b10100000 // 32
#define MTOUCH_SET_ADC_CLK_16MHZ()         ADCON1 = 0b11010000 // 16
#define MTOUCH_SET_ADC_CLK_8MHZ()          ADCON1 = 0b10010000 // 8
#define MTOUCH_SET_ADC_CLK_4MHZ()          ADCON1 = 0b11000000 // 4
#define MTOUCH_SET_ADC_CLK_2MHZ()          ADCON1 = 0b10000000 // 2
#define MTOUCH_SET_ADC_CLK_1MHZ()          ADCON1 = 0b10000000 // 2

#define CVD_UNIMPLEMENTED_AVAILABLE     0

#if defined(MTOUCH_V1_COMMS)
#if !(MTOUCH_V1_COMM_SPEED == 9600 || MTOUCH_V1_COMM_SPEED == 115200)
    #error MTOUCH_V1_COMM_SPEED must be set to either 9600 or 115200 in the configuration file.
#endif
#endif

#if     _XTAL_FREQ == 16000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_16MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    33
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    103
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00000000
        #define MTOUCH_DEBUG_SPBRGL    103
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  8000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_8MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    16
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    51
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    207
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  4000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_4MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    8
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    25
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    103
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  2000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_2MHZ()
    #define CVD_TAD             1  
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #error The 115.2kbps UART baudrate option cannot be used with a 2MHz Fosc. Please select '9600' or '38400'.
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    12
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    51
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#elif   _XTAL_FREQ ==  1000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_1MHZ()
    #define CVD_TAD             2
    #if     defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 115200)
        #error The 115.2kbps UART baudrate option cannot be used with a 1MHz Fosc. Please select '9600' or '38400'.
    #elif   (defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 38400)) || defined(MTOUCH_V2_COMMS)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00000000
        #define MTOUCH_DEBUG_SPBRGL    5
        #define MTOUCH_DEBUG_SPBRGH    0
    #elif   defined(MTOUCH_V1_COMMS) && (MTOUCH_V1_COMM_SPEED == 9600)
        #define MTOUCH_DEBUG_TXSTA     0b00100100
        #define MTOUCH_DEBUG_RCSTA     0b10010000
        #define MTOUCH_DEBUG_BAUDCON   0b00001000
        #define MTOUCH_DEBUG_SPBRGL    25
        #define MTOUCH_DEBUG_SPBRGH    0
    #endif
#else
    #error _XTAL_FREQ is not set to a valid value for this processor.
#endif

#if MTOUCH_NUMBER_SENSORS == 1
    #error Single-sensor scanning code not currently compatible with the PIC16F152x family due to the lack of a DAC module.
#endif
#define MTOUCH_ADC_BITS                 10

#define MTOUCH_WDT_OFF()           SWDTEN = 0
#define MTOUCH_WDT_ON()            SWDTEN = 1

// A/D MUX selection for each A/D button, do not change this, refer to datasheet if curious how these are derived
// Right Justified, Vdd as reference, A/D on, Go/Done  asserted, do not change this, refer to datasheet if curious how these are derived 
#define	MTOUCH_AD_AN0       0x01
#define	MTOUCH_AD_AN1       0x05
#define	MTOUCH_AD_AN2       0x09
#define	MTOUCH_AD_AN3       0x0D
#define MTOUCH_AD_AN4       0x11
#define MTOUCH_AD_AN5       0x15
#define MTOUCH_AD_AN6       0x19
#define MTOUCH_AD_AN7       0x1D
#define MTOUCH_AD_AN8       0x21
#define MTOUCH_AD_AN9       0x25
#define MTOUCH_AD_AN10      0x29
#define MTOUCH_AD_AN11      0x2D
#define MTOUCH_AD_AN12      0x31
#define MTOUCH_AD_AN13      0x35
#define MTOUCH_AD_AN14      0x39
#define MTOUCH_AD_AN15      0x3D
#define MTOUCH_AD_AN16      0x41
#define MTOUCH_AD_AN17      0x45
#define MTOUCH_AD_AN18      0x49
#define MTOUCH_AD_AN19      0x4D
#define MTOUCH_AD_AN20      0x51
#define MTOUCH_AD_AN21      0x55
#define MTOUCH_AD_AN22      0x59
#define MTOUCH_AD_AN23      0x5D
#define MTOUCH_AD_AN24      0x61
#define MTOUCH_AD_AN25      0x65
#define MTOUCH_AD_AN26      0x69
#define MTOUCH_AD_AN27      0x6D
#define MTOUCH_AD_AN28      0x71
#define MTOUCH_AD_AN29      0x75

#define MTOUCH_AD_FVR_AND_GO   0x7F // Selects the FVR as the ADC mux option and sets the GO/DONE bit.

#define MTOUCH_PIN_AN0      0
#define MTOUCH_PIN_AN1      1
#define MTOUCH_PIN_AN2      2
#define MTOUCH_PIN_AN3      3
#define MTOUCH_PIN_AN4      5
#define	MTOUCH_PIN_AN5      7
#define	MTOUCH_PIN_AN6      1
#define	MTOUCH_PIN_AN7      2
#define MTOUCH_PIN_AN8      3
#define MTOUCH_PIN_AN9      4
#define MTOUCH_PIN_AN10     5
#define MTOUCH_PIN_AN11     6
#define MTOUCH_PIN_AN12     4
#define MTOUCH_PIN_AN13     3
#define MTOUCH_PIN_AN14     2
#define MTOUCH_PIN_AN15     1
#define MTOUCH_PIN_AN16     0
#define MTOUCH_PIN_AN17     0
#define MTOUCH_PIN_AN18     1
#define MTOUCH_PIN_AN19     2
#define MTOUCH_PIN_AN20     3
#define MTOUCH_PIN_AN21     4
#define MTOUCH_PIN_AN22     5
#define MTOUCH_PIN_AN23     0
#define MTOUCH_PIN_AN24     1
#define MTOUCH_PIN_AN25     2
#define MTOUCH_PIN_AN26     3
#define MTOUCH_PIN_AN27     0
#define MTOUCH_PIN_AN28     1
#define MTOUCH_PIN_AN29     2

#define MTOUCH_LETTER_AN0     A
#define MTOUCH_LETTER_AN1     A
#define MTOUCH_LETTER_AN2     A
#define MTOUCH_LETTER_AN3     A
#define MTOUCH_LETTER_AN4     A
#define MTOUCH_LETTER_AN5     F
#define MTOUCH_LETTER_AN6     F
#define MTOUCH_LETTER_AN7     F
#define MTOUCH_LETTER_AN8     F
#define MTOUCH_LETTER_AN9     F
#define MTOUCH_LETTER_AN10    F
#define MTOUCH_LETTER_AN11    F
#define MTOUCH_LETTER_AN12    G
#define MTOUCH_LETTER_AN13    G
#define MTOUCH_LETTER_AN14    G
#define MTOUCH_LETTER_AN15    G
#define MTOUCH_LETTER_AN16    F
#define MTOUCH_LETTER_AN17    B
#define MTOUCH_LETTER_AN18    B
#define MTOUCH_LETTER_AN19    B
#define MTOUCH_LETTER_AN20    B
#define MTOUCH_LETTER_AN21    B
#define MTOUCH_LETTER_AN22    B
#define MTOUCH_LETTER_AN23    D
#define MTOUCH_LETTER_AN24    D
#define MTOUCH_LETTER_AN25    D
#define MTOUCH_LETTER_AN26    D
#define MTOUCH_LETTER_AN27    E
#define MTOUCH_LETTER_AN28    E
#define MTOUCH_LETTER_AN29    E


#endif
/// @endcond

