/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch CVD Framework
 *  FileName:        mTouch_HardwareProfile_16F61x.h
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH PRO Ver. 9.80 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Hardware Profile Header File
 *                   - Provides information about correct SFR addresses,
 *                     pin assignments, module availability, and any other
 *                     part-specific data.
 *                   - See the documentation for more information.
 *************************************************************************/
 /**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file mTouch_HardwareProfile_16F61x.h
* @brief Support for: PIC16F616, PIC16HV616
*/
/// @cond
#if !defined(__MTOUCHCVD_HARDWARE_16F61X_H)
#define __MTOUCHCVD_HARDWARE_16F61X_H

#if !defined(_16F616) && !defined(_16HV616)
    #error The current mTouch hardware include file does not support this PIC microcontroller.
#endif

#if defined(_16F616) || defined(_16HV616)
    #if MTOUCH_NUMBER_SENSORS > 8
        #error The PIC16F/HV616 is not able to support more than 8 sensors due to pinout limitations.
    #endif
#endif


#define MTOUCH_DEBUG_COMM_INIT()                                       \
    do {                                                            \
    } while (0)

#define MTOUCH_DEBUG_COMM_TXIF             TXIF
#define MTOUCH_DEBUG_COMM_TXREG            TXREG

#define MTOUCH_TIMER0_AVAILABLE         
#define MTOUCH_TIMER1_AVAILABLE         
#define MTOUCH_TIMER2_AVAILABLE  
#define MTOUCH_SET_GIE()                   GIE    = 1
    
#define CVD_TAD 2   
#define MTOUCH_SET_ADC_CLK_8MHZ()          ADCON1 = 0b01010000 // 16
#define MTOUCH_SET_ADC_CLK_4MHZ()          ADCON1 = 0b00010000 // 8
#define MTOUCH_SET_ADC_CLK_1MHZ()          ADCON1 = 0b00000000 // 2

#if defined(MTOUCH_V1_COMMS)
#if !(MTOUCH_V1_COMM_SPEED == 9600 || MTOUCH_V1_COMM_SPEED == 115200)
    #error MTOUCH_V1_COMM_SPEED must be set to either 9600 or 115200 in the configuration file.
#endif
#endif

#if     _XTAL_FREQ == 8000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_8MHZ()    
#elif   _XTAL_FREQ == 4000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_4MHZ()    
#elif   _XTAL_FREQ == 1000000 
    #define MTOUCH_ADC_INIT()   MTOUCH_SET_ADC_CLK_1MHZ()    
#else
    #error Chosen Fosc frequency is not supported by the mTouch framework on this device.
#endif

#define MTOUCH_USE_PORT
#define CVD_ADCON0_BANK         0
#define MTOUCH_ADC_BITS         10
    
#define MTOUCH_WDT_OFF()          
#define MTOUCH_WDT_ON()            

// A/D MUX selection for each A/D button, do not change this, refer to datasheet if curious how these are derived
// Right Justified, Vdd as reference, A/D on, Go/Done  asserted, do not change this, refer to datasheet if curious how these are derived 
#define MTOUCH_AD_AN0           0x81
#define MTOUCH_AD_AN1           0x85
#define MTOUCH_AD_AN2           0x89
#define MTOUCH_AD_AN3           0x8D
#define MTOUCH_AD_AN4           0x91
#define MTOUCH_AD_AN5           0x95
#define MTOUCH_AD_AN6           0x99
#define MTOUCH_AD_AN7           0x9D

#define MTOUCH_AD_ISO_AND_GO    0xBF // Selects an unimplemented, isolated ADC mux option and sets the GO/DONE bit.
#define MTOUCH_AD_ISO_NOGO      0xBD // Selects an unimplemented, isolated ADC mux option w/o setting the GO/DONE bit.

#define MTOUCH_PIN_AN0      0
#define MTOUCH_PIN_AN1      1
#define MTOUCH_PIN_AN2      2
#define MTOUCH_PIN_AN3      4
#define MTOUCH_PIN_AN4      0
#define MTOUCH_PIN_AN5      1
#define MTOUCH_PIN_AN6      2
#define MTOUCH_PIN_AN7      3


#define MTOUCH_LETTER_AN0     A
#define MTOUCH_LETTER_AN1     A
#define MTOUCH_LETTER_AN2     A
#define MTOUCH_LETTER_AN3     A
#define MTOUCH_LETTER_AN4     C
#define MTOUCH_LETTER_AN5     C
#define MTOUCH_LETTER_AN6     C
#define MTOUCH_LETTER_AN7     C


#endif
/// @endcond
