/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_Comm.c
 *  Dependencies:    mTouch.h
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework Communication Module
 *                   - Implements the communication between the mTouch
 *                     PC GUI program and the firmware. May also be used
 *                     to output directly to a PC's terminal.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file   mTouch_comm.h
*   @brief  Internal framework communications header file implementing the
*           bit-banged UART timing in assembly and the internal macros used
*           implement the 'V2' communications.
*/
#ifndef __MTOUCH_COMM_H
#define __MTOUCH_COMM_H

#if defined(MTOUCH_V1_COMMS) || defined(MTOUCH_V2_COMMS)

    #if defined(MTOUCH_V1_COMMS) && defined(MTOUCH_V2_COMMS)
        #error Both V1 and V2 comms have been enabled in the configuration file. Please choose one and comment-out the other.
    #endif

    #if defined(MTOUCH_V2_COMMS)
    
        extern enum     mTouch_CommState
                        {
                            MTOUCH_COMM_IDLE,          ///< RX communications are idling
                            MTOUCH_COMM_START,         ///< New RX packet is starting
                            MTOUCH_COMM_BUFFER,        ///< RX packet is being stored in the buffer
                            MTOUCH_COMM_PROCESS_DATA   ///< RX packet is ready to be processed by the state machine
                        };  
                        
        #define MTOUCH_V2_STREAMBYTESperSENSOR      4
        #define MTOUCH_V2_COMMS_BUFFER_SIZE         6
        #define MTOUCH_COMM_SEND_CHECKSUM()         put_Char(checksum)
        
        #define MTOUCH_COMM_READandSEND_EE(addr)    \
            do {                                    \
                MTOUCH_EEPROM_read(addr);           \
                temp    = EEDATL;                   \
                checksum ^= temp;                   \
                put_Char(temp);                     \
            } while(0)
        
        #define MTOUCH_COMM_SEND_OPCODE(value,size) checksum = v2Comm_sendOpcode(value, size)
        
        // #define MTOUCH_COMM_SEND_OPCODE(value,size)             \
            // do {                                                \
                // SENDB   = 1;                                    \
                // TXREG   = size;                                 \
                // while(SENDB);                                   \
                // put_Char(size);                                 \
                // put_Char(value);                                \
                // checksum  = value;                              \
            // } while (0)
            
        #define MTOUCH_COMM_SEND_BYTE(byte)                     \
            do {                                                \
                put_Char(byte);                                 \
                checksum ^= byte;                               \
            } while (0)

        #define MTOUCH_COMM_PUT_CHAR(_char)         checksum ^= v2Comm_putChar(_char)
            
        // #define MTOUCH_COMM_PUT_CHAR(_char)                     \
            // do {                                                \
                // if(!mTouch_state.commNewCommand)                \
                // {                                               \
                    // put_Char(_char);                            \
                    // checksum ^= _char;                          \
                // }                                               \
                // else                                            \
                // {                                               \
                    // mTouch_state.commNewCommand = 0;            \
                    // return;                                     \
                // }                                               \
            // } while (0)

        #define MTOUCH_COMM_SEND_OP_MAINLOOP(value,size)        \
            do {                                                \
                /* Send Break Character to Begin New Packet */  \
                SENDB   = 1;                                    \
                if(!mTouch_state.commNewCommand)                \
                {                                               \
                    TXREG   = size;                             \
                    while(SENDB);                               \
                }                                               \
                else                                            \
                {                                               \
                    mTouch_state.commNewCommand = 0;            \
                    return;                                     \
                }                                               \
                                                                \
                /* Send Packet Length */                        \
                if(!mTouch_state.commNewCommand)                \
                {                                               \
                    put_Char(size);                             \
                }                                               \
                else                                            \
                {                                               \
                    mTouch_state.commNewCommand = 0;            \
                    return;                                     \
                }                                               \
                                                                \
                /* Send OPCODE */                               \
                if(!mTouch_state.commNewCommand)                \
                {                                               \
                    put_Char(value);                            \
                    checksum = (value);                         \
                }                                               \
                else                                            \
                {                                               \
                    mTouch_state.commNewCommand = 0;            \
                    return;                                     \
                }                                               \
            } while (0)
        
        
        #define MTOUCH_COMM_OPCODE_A                    0x0A
        #define MTOUCH_COMM_OPCODE_C                    0x0C
        #define MTOUCH_COMM_OPCODE_D                    0x0D
        #define MTOUCH_COMM_OPCODE_F                    0x0F
        #define MTOUCH_COMM_OPCODE_5                    0x05
        #define MTOUCH_COMM_OPCODE_6                    0x06
        #define MTOUCH_COMM_OPCODE_8                    0x08
        #define MTOUCH_COMM_OPCODE_9                    0x09

        #define MTOUCH_COMM_READ_CONFIG                 0xA0
        #define MTOUCH_COMM_RESTORE_FAC_SETTINGS        0xA1
        #define MTOUCH_COMM_CONFIG_RAW_ACTIVE           0xC0
        #define MTOUCH_COMM_CONFIG_RAW_SLEEP            0xD0
        #define MTOUCH_COMM_CONFIG_MASK_ACTIVE          0xC1
        #define MTOUCH_COMM_CONFIG_MASK_SLEEP           0xD1
        #define MTOUCH_COMM_MANUAL_MODE_CHANGE          0xF0
        #define MTOUCH_COMM_CHANGE_EDIT_MODE_ACTIVE     0xF2
        #define MTOUCH_COMM_CHANGE_EDIT_MODE_SLEEP      0xF3

        #define MTOUCH_COMM_READ_pTHRESHOLD             0x80
        #define MTOUCH_COMM_WRITE_pTHRESHOLD            0x50
        //
        //
        #define MTOUCH_COMM_READ_rTHRESHOLD_PER         0x90
        #define MTOUCH_COMM_WRITE_rTHRESHOLD_PER        0x60
        #define MTOUCH_COMM_READ_pDEBOUNCE              0x91
        #define MTOUCH_COMM_WRITE_pDEBOUNCE             0x61
        #define MTOUCH_COMM_READ_rDEBOUNCE              0x92
        #define MTOUCH_COMM_WRITE_rDEBOUNCE             0x62
        #define MTOUCH_COMM_READ_SAMPLESperSCAN         0x93
        #define MTOUCH_COMM_WRITE_SAMPLESperSCAN        0x63
        #define MTOUCH_COMM_READ_JITTER                 0x94
        #define MTOUCH_COMM_WRITE_JITTER                0x64
        #define MTOUCH_COMM_READ_SCALING_FACTOR         0x95
        #define MTOUCH_COMM_WRITE_SCALING_FACTOR        0x65
        #define MTOUCH_COMM_READ_pTIMEOUT               0x96
        #define MTOUCH_COMM_WRITE_pTIMEOUT              0x66
        #define MTOUCH_COMM_READ_AVG_UPDATE             0x97
        #define MTOUCH_COMM_WRITE_AVG_UPDATE            0x67
        #define MTOUCH_COMM_READ_AVG_RATE               0x98
        #define MTOUCH_COMM_WRITE_AVG_RATE              0x68
        #define MTOUCH_COMM_READ_BOARD_ID               0x99
        #define MTOUCH_COMM_WRITE_BOARD_ID              0x69
        #define MTOUCH_COMM_READ_NEG_CAPACITANCE        0x9A
        #define MTOUCH_COMM_WRITE_NEG_CAPACITANCE       0x6A
        #define MTOUCH_COMM_READ_DEC_STEPSIZE           0x9B
        #define MTOUCH_COMM_WRITE_DEC_STEPSIZE          0x6B
        #define MTOUCH_COMM_READ_INACTIVE               0x9C
        #define MTOUCH_COMM_WRITE_INACTIVE              0x6C
        #define MTOUCH_COMM_READ_BUTTON_STATE           0x9D
        //
        //

        #define MTOUCH_COMM_STREAM_ACTIVE               0xC0
        #define MTOUCH_COMM_STREAM_SLEEP                0xD0
        #define MTOUCH_COMM_STREAM_MASK_ACTIVE          0xC1
        #define MTOUCH_COMM_STREAM_MASK_SLEEP           0xD1
        #define MTOUCH_COMM_STREAM_COUNTER_ACTIVE       0xC3
        #define MTOUCH_COMM_STREAM_COUNTER_SLEEP        0xD3
        #define MTOUCH_COMM_SEND_CONFIGURATION          0xA0
        #define MTOUCH_COMM_SEND_SINGLE_VALUE_ACTIVE    0xCC
        #define MTOUCH_COMM_SEND_SINGLE_VALUE_SLEEP     0xDD
        #define MTOUCH_COMM_SEND_BUTTON_STATE_ACTIVE    0xCD
        #define MTOUCH_COMM_SEND_BUTTON_STATE_SLEEP     0xDE
        #define MTOUCH_COMM_SEND_MODE_CHANGE_ACTIVE     0xF0
        #define MTOUCH_COMM_SEND_MODE_CHANGE_SLEEP      0xF1

        #define MTOUCH_COMM_ACK                         0xAA
        #define MTOUCH_COMM_NACK                        0xA5
    #endif

    #if defined(MTOUCH_V2_COMMS)
        #if !defined(MTOUCH_DEBUG_UART_AVAILABLE)
            #error The Framework's "V2" communications are not available on processors without a hardware UART. Please disable MTOUCH_V2_COMMS.
        #else
            #if defined(MTOUCH_V1_COMM_BITBANGED)
                #undef MTOUCH_V1_COMM_BITBANGED
            #endif
        #endif
    #endif
    
    #if !defined(MTOUCH_DEBUG_UART_AVAILABLE) && !defined(MTOUCH_V2_COMMS)
        #if !defined(MTOUCH_V1_COMM_BITBANGED)
            #warning MTOUCH_V1_COMM_BITBANGED is not enabled in the configuration file, but no hardware UART is available for this processor. MTOUCH_V1_COMM_BITBANGED is being automatically enabled for you.
            #define MTOUCH_V1_COMM_BITBANGED
        #endif
    #endif

    #if defined(MTOUCH_V1_COMM_BITBANGED)
        #define MTOUCH_RS232_BRG_9600_32MHz     208
        #define MTOUCH_RS232_BRG_9600_16MHz     104
        #define MTOUCH_RS232_BRG_9600_8MHz       52
        #define MTOUCH_RS232_BRG_9600_4MHz       26
        #define MTOUCH_RS232_BRG_9600_2MHz       13

        #define MTOUCH_RS232_BRG_19200_32MHz    104
        #define MTOUCH_RS232_BRG_19200_16MHz     52
        #define MTOUCH_RS232_BRG_19200_8MHz      26
        #define MTOUCH_RS232_BRG_19200_4MHz      13

        #define MTOUCH_RS232_BRG_38400_32MHz     52
        #define MTOUCH_RS232_BRG_38400_16MHz     26
        #define MTOUCH_RS232_BRG_38400_8MHz      13

        #define MTOUCH_RS232_BRG_57600_32MHz     35

        #if _XTAL_FREQ == 32000000
            #if     MTOUCH_V1_COMM_SPEED == 9600
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_9600_32MHz
            #elif   MTOUCH_V1_COMM_SPEED == 19200
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_19200_32MHz
            #elif   MTOUCH_V1_COMM_SPEED == 38400
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_38400_32MHz
            #elif   MTOUCH_V1_COMM_SPEED == 57600
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_57600_32MHz
            #endif
        #elif _XTAL_FREQ == 16000000
            #if     MTOUCH_V1_COMM_SPEED == 9600
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_9600_16MHz
            #elif   MTOUCH_V1_COMM_SPEED == 19200
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_19200_16MHz
            #elif   MTOUCH_V1_COMM_SPEED == 38400
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_38400_16MHz
            #endif
        #elif _XTAL_FREQ == 8000000
            #if     MTOUCH_V1_COMM_SPEED == 9600
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_9600_8MHz
            #elif   MTOUCH_V1_COMM_SPEED == 19200
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_19200_8MHz
            #elif   MTOUCH_V1_COMM_SPEED == 38400
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_38400_8MHz
            #endif
        #elif _XTAL_FREQ == 4000000
            #if     MTOUCH_V1_COMM_SPEED == 9600
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_9600_8MHz
            #elif   MTOUCH_V1_COMM_SPEED == 19200
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_19200_8MHz
            #elif   MTOUCH_V1_COMM_SPEED == 38400
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_38400_8MHz
            #endif
        #elif _XTAL_FREQ == 2000000
            #if     MTOUCH_V1_COMM_SPEED == 9600
            #define MTOUCH_RS232_FWBRG  MTOUCH_RS232_BRG_9600_2MHz
            #endif
        #elif _XTAL_FREQ == 1000000
        #endif

        #if !defined(MTOUCH_RS232_FWBRG)
        #error The Fosc / Baudrate combination you have chosen is not currently compatible with the default mTouch communication implementation.
        #endif


        //  _____       _                              _   __  __ _     _                                  
        // | ____|_ __ | |__   __ _ _ __   ___ ___  __| | |  \/  (_) __| |      _ __ __ _ _ __   __ _  ___ 
        // |  _| | '_ \| '_ \ / _` | '_ \ / __/ _ \/ _` | | |\/| | |/ _` |_____| '__/ _` | '_ \ / _` |/ _ \
        // | |___| | | | | | | (_| | | | | (_|  __/ (_| | | |  | | | (_| |_____| | | (_| | | | | (_| |  __/
        // |_____|_| |_|_| |_|\__,_|_| |_|\___\___|\__,_| |_|  |_|_|\__,_|     |_|  \__,_|_| |_|\__, |\___|
        //                                                                                      |___/  
        #if defined(_PIC14E)

            #define MTOUCH_COMM_232_TXSET()         asm("bsf " ___mkstr(_INDF0) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN))
            #define MTOUCH_COMM_232_TXCLEAR()       asm("bcf " ___mkstr(_INDF0) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN))

            #define MTOUCH_COMM_232_INIT()                                                                          \
                do {                                                                                                \
                    asm("BANKSEL    "   ___mkstr(__paste(_,MTOUCH_SOFT_UART_TXPORT))                                           );  \
                    asm("bsf        "   ___mkstr(NOBANK(__paste(_,MTOUCH_SOFT_UART_TXPORT))) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN) );  \
                    asm("BANKSEL    "   ___mkstr(__paste(_,MTOUCH_SOFT_UART_TXTRIS))                                           );  \
                    asm("bcf        "   ___mkstr(NOBANK(__paste(_,MTOUCH_SOFT_UART_TXTRIS))) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN) );  \
                } while(0)
            
            #define MTOUCH_COMM_232_TRANSMIT()                                          \
                do {                                                                    \
                    mTouch_comm_shiftReg = output;                                      \
                    asm("movlw LOW  "   ___mkstr(NOBANK(__paste(_,MTOUCH_SOFT_UART_TXPORT)))        ); \
                    asm("movwf      "   ___mkstr(_FSR0L)                             ); \
                    asm("movlw HIGH "   ___mkstr(NOBANK(__paste(_,MTOUCH_SOFT_UART_TXPORT)))        ); \
                    asm("movwf      "   ___mkstr(_FSR0H)                             ); \
                    asm("BANKSEL    _mTouch_comm_bitCount                           "); \
                    asm("movlw      0x08                                            "); \
                    asm("movwf      _mTouch_comm_bitCount                           "); \
                    MTOUCH_COMM_232_TXCLEAR();                                          \
                    asm("call       MTOUCH_232_BITDELAY                             "); \
                    asm("goto       $+1                                             "); \
                    asm("nop                                                        "); \
                                                                                        \
                    asm("MTOUCH_232_TXNEXTBIT:                          "); \
                        asm("rrf    _mTouch_comm_shiftReg, F            "); \
                        asm("btfss  "   ___mkstr(NOBANK(_STATUS))   ", 0"); \
                        asm("goto       MTOUCH_232_TXZERO               "); \
                    asm("MTOUCH_232_TXONE:                              "); \
                        asm("nop                                        "); \
                        MTOUCH_COMM_232_TXSET();                            \
                        asm("call       MTOUCH_232_BITDELAY             "); \
                        asm("decfsz     _mTouch_comm_bitCount, F        "); \
                        asm("goto       MTOUCH_232_TXNEXTBIT            "); \
                        asm("goto       MTOUCH_232_TXSTOP               "); \
                    asm("MTOUCH_232_TXZERO:                             "); \
                        MTOUCH_COMM_232_TXCLEAR();                          \
                        asm("call       MTOUCH_232_BITDELAY             "); \
                        asm("decfsz     _mTouch_comm_bitCount, F        "); \
                        asm("goto       MTOUCH_232_TXNEXTBIT            "); \
                        asm("goto       MTOUCH_232_TXSTOP               "); \
                    asm("MTOUCH_232_TXSTOP:                             "); \
                        asm("goto       $+1                             "); \
                        asm("nop                                        "); \
                        MTOUCH_COMM_232_TXSET();                            \
                        asm("call       MTOUCH_232_BITDELAY             "); \
                        asm("return                                     "); \
                } while(0)
                
            #define MTOUCH_COMM_232_BITDELAY()                          \
                do {                                                    \
                    asm("MTOUCH_232_BITDELAY:                       "); \
                        asm("movlw      0x04                        "); \
                        asm("sublw  "   ___mkstr(MTOUCH_RS232_FWBRG) ); \
                        asm("movwf      _mTouch_comm_delayCount     "); \
                    asm("MTOUCH_232_BITWAITLOOP:                    "); \
                        asm("nop                                    "); \
                    asm("MTOUCH_232_BITWAIT:                        "); \
                        asm("decfsz     _mTouch_comm_delayCount, F  "); \
                        asm("goto       MTOUCH_232_BITWAITLOOP      "); \
                        asm("goto       $+1                         "); \
                        asm("return                                 "); \
                } while(0)
                

        //  __  __ _     _                                  
        // |  \/  (_) __| |      _ __ __ _ _ __   __ _  ___ 
        // | |\/| | |/ _` |_____| '__/ _` | '_ \ / _` |/ _ \
        // | |  | | | (_| |_____| | | (_| | | | | (_| |  __/
        // |_|  |_|_|\__,_|     |_|  \__,_|_| |_|\__, |\___|
        //                                       |___/  
        #elif defined(_PIC14)

            #define MTOUCH_COMM_232_TXSET()         asm("bsf " ___mkstr(_INDF) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN))
            #define MTOUCH_COMM_232_TXCLEAR()       asm("bcf " ___mkstr(_INDF) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN))
            
            #define MTOUCH_COMM_232_INIT()                                                                          \
                do {                                                                                                \
                    asm("BANKSEL    "   ___mkstr(__paste(_,MTOUCH_SOFT_UART_TXPORT))                                            ); \
                    asm("bsf        "   ___mkstr(NOBANK(__paste(_,MTOUCH_SOFT_UART_TXPORT))) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN)  ); \
                    asm("BANKSEL    "   ___mkstr(__paste(_,MTOUCH_SOFT_UART_TXTRIS))                                            ); \
                    asm("bcf        "   ___mkstr(NOBANK(__paste(_,MTOUCH_SOFT_UART_TXTRIS))) ", " ___mkstr(MTOUCH_SOFT_UART_TXPIN)  ); \
                } while(0)
                
            #define MTOUCH_COMM_232_TRANSMIT()                                          \
                do {                                                                    \
                    mTouch_comm_shiftReg = output;                                      \
                    asm("movlw LOW  "   ___mkstr(NOBANK(__paste(_,MTOUCH_SOFT_UART_TXPORT)))        ); \
                    asm("movwf      "   ___mkstr(_FSR)                               ); \
                    asm("BANKSEL    _mTouch_comm_shiftReg                           "); \
                    asm("movlw      8                                               "); \
                    asm("movwf      _mTouch_comm_bitCount                           "); \
                    MTOUCH_COMM_232_TXCLEAR();                                          \
                    asm("call       MTOUCH_232_BITDELAY                             "); \
                    asm("goto       $+1                                             "); \
                    asm("nop                                                        "); \
                                                                                        \
                    asm("MTOUCH_232_TXNEXTBIT:                          "); \
                        asm("rrf    _mTouch_comm_shiftReg, F            "); \
                        asm("btfss  "   ___mkstr(NOBANK(_STATUS))   ", 0"); \
                        asm("goto       MTOUCH_232_TXZERO               "); \
                    asm("MTOUCH_232_TXONE:                              "); \
                        asm("nop                                        "); \
                        MTOUCH_COMM_232_TXSET();                            \
                        asm("call       MTOUCH_232_BITDELAY             "); \
                        asm("decfsz     _mTouch_comm_bitCount, F        "); \
                        asm("goto       MTOUCH_232_TXNEXTBIT            "); \
                        asm("goto       MTOUCH_232_TXSTOP               "); \
                    asm("MTOUCH_232_TXZERO:                             "); \
                        MTOUCH_COMM_232_TXCLEAR();                          \
                        asm("call       MTOUCH_232_BITDELAY             "); \
                        asm("decfsz     _mTouch_comm_bitCount, F        "); \
                        asm("goto       MTOUCH_232_TXNEXTBIT            "); \
                        asm("goto       MTOUCH_232_TXSTOP               "); \
                    asm("MTOUCH_232_TXSTOP:                             "); \
                        asm("goto       $+1                             "); \
                        asm("nop                                        "); \
                        MTOUCH_COMM_232_TXSET();                            \
                        asm("call       MTOUCH_232_BITDELAY             "); \
                        asm("return                                     "); \
                } while(0)
                
            #define MTOUCH_COMM_232_BITDELAY()                          \
                do {                                                    \
                    asm("MTOUCH_232_BITDELAY:                       "); \
                        asm("movlw      4                           "); \
                        asm("sublw  "   ___mkstr(MTOUCH_RS232_FWBRG) ); \
                        asm("movwf      _mTouch_comm_delayCount     "); \
                    asm("MTOUCH_232_BITWAITLOOP:                    "); \
                        asm("nop                                    "); \
                    asm("MTOUCH_232_BITWAIT:                        "); \
                        asm("decfsz     _mTouch_comm_delayCount, F  "); \
                        asm("goto       MTOUCH_232_BITWAITLOOP      "); \
                        asm("goto       $+1                         "); \
                        asm("return                                 "); \
                } while(0)

        #endif // end: if defined(_PIC14)
    #endif
#endif  // end: if  defined(MTOUCH_V1_COMMS)
#endif  // end: if !defined(MTOUCH_COMM_H)