/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_Config.h
 *  Dependencies:    
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework Configuration File
 *                   - See documentation for better explanations of all
 *                     configuration options.
 *************************************************************************/
/***********************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
 /****************************************************************************
 * Revision History
 * Date		Rev	 Author  Comments
 * 09.20.10 1.00         Public Release
 ****************************************************************************/
/** @file   mTouch_Config.h
*   @brief  Framework Configuration! Main configuration file for the mTouch 
*           Framework.
*/
#ifndef __MTOUCH_CONFIG_H
#define __MTOUCH_CONFIG_H

#include "mTouch_config_slider.h"
#include "mTouch_config_sleep.h"
#include "mTouch_config_cvdAdvanced.h"
#include "mTouch_config_proximity.h"
#include "mTouch_optionsForConfig.h"

//****************************************************************************************************
//  ____            _                   ____       _               
// / ___| _   _ ___| |_ ___ _ __ ___   / ___|  ___| |_ _   _ _ __  
// \___ \| | | / __| __/ _ \ '_ ` _ \  \___ \ / _ \ __| | | | '_ \ 
//  ___) | |_| \__ \ ||  __/ | | | | |  ___) |  __/ |_| |_| | |_) |
// |____/ \__, |___/\__\___|_| |_| |_| |____/ \___|\__|\__,_| .__/ 
//        |___/                                             |_|        
//
//****************************************************************************************************
    #define _XTAL_FREQ                      32000000 // <-- Oscillator frequency of your MCU in Hz   

    // Framework-Application Integration Type
    //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    //  Option Label                    :: Option Behavior
    //.................................................................................................
    //  MTOUCH_CONTROLS_ISR             :: The framework defines and controls 100% of the ISR.
    //  MTOUCH_CALLED_FROM_ISR          :: The application's ISR function is responsible for calling mTouch_Scan().
    //  MTOUCH_CALLED_FROM_MAINLOOP     :: The mainloop mTouch API will automatically call the scan function.    
    //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    
    #define MTOUCH_INTEGRATION_TYPE         MTOUCH_CALLED_FROM_ISR

    
        // If MTOUCH_INTEGRATION_TYPE is defined as either MTOUCH_CONTROLS_ISR or MTOUCH_CALLED_FROM_ISR,
        // this option will determine which timer is used to clock the mTouch scan routine.
        #define MTOUCH_ISR_TIMER            2       // NOTE: You are responsible for initializing
                                                    //       the timer with any desired pre- or
                                                    //       post-scaling options including any
                                                    //       period registers (PR2/4/6). 
                                                    // 8-bit Timers Only. Not compatible with TMR1/3/5.
    
    
    // mTouch Scan Method Functionality
    //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    //  Option Label                    :: Option Behavior
    //.................................................................................................
    //  MTOUCH_SCANS_ONE_SENSOR         :: One mTouch function call will result in one sensor being scanned once.
    //  MTOUCH_SCANS_ALL_SENSORS        :: One mTouch function call will result in all sensors being scanned once.
    //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

    #define MTOUCH_SCAN_FUNCTIONALITY       MTOUCH_SCANS_ONE_SENSOR

    
    
    // mTouch Scan Method Functionality
    //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    //  Option Label                    :: Option Behavior
    //.................................................................................................
    //  MTOUCH_ALWAYS_CONTROLS_ADC      :: ADC is considered under the control of the framework at all times.
    //  MTOUCH_RELEASES_ADC_AFTER_SCAN  :: ADC is released and available for use between scans.
    //:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
    
    #define MTOUCH_ADC_CONTROL              MTOUCH_ALWAYS_CONTROLS_ADC
                                                    

                                                    //:::::::::::::::::::::::::::::::::::::::::::::::::
                                                    // System Version Control
                                                    //
                                                    //  These values are used only when communicating
                                                    //  with the mTouch PC software to relay
                                                    //  information about the current version of
                                                    //  firmware being used.
                                                    //
    #define MTOUCH_BOARD_ID                 0x00    //  Which board is this? 
                                                    //      Used in 'V2' mTouch GUI to identify the board.
                                                    //      No effect on framework behavior.
    #define MTOUCH_BOARD_REVISION           0x01    //  Which hardware revision?
                                                    //      Displayed in 'V2' mTouch GUI. 
                                                    //      No effect on framework behavior.
    #define MTOUCH_FRAME_REVISION           0x20    //  Which version of the mTouch framework is this?
                                                    //      Displayed in 'V2' mTouch GUI. 
                                                    //      No effect on framework behavior.
    #define MTOUCH_SOFTWARE_REVISION        0x01    //  Which version of user-application code is this?
                                                    //      Displayed in 'V2' mTouch GUI. 
                                                    //      No effect on framework behavior.
                                                    //:::::::::::::::::::::::::::::::::::::::::::::::::

//****************************************************************************************************
//  ____                              ____       _               
// / ___|  ___ _ __  ___  ___  _ __  / ___|  ___| |_ _   _ _ __  
// \___ \ / _ \ '_ \/ __|/ _ \| '__| \___ \ / _ \ __| | | | '_ \ 
//  ___) |  __/ | | \__ \ (_) | |     ___) |  __/ |_| |_| | |_) |
// |____/ \___|_| |_|___/\___/|_|    |____/ \___|\__|\__,_| .__/ 
//                                                        |_|    
//
//****************************************************************************************************
                                                    
    #define MTOUCH_NUMBER_SENSORS       5           // <-- Number of mTouch sensors to scan in active
                                                    //     mode.
                                                    // 
                                                    //     NOTE: If you want the sensors to be scanned
                                                    //           in sleep mode, define them in the 
                                                    //           'mTouch_config_sleep.h' file.
                                                    //

    // Analog Pin Configuration
    #define MTOUCH_SENSOR0             AN9 //up
    #define MTOUCH_SENSOR1             AN12 //down
    #define MTOUCH_SENSOR2             AN10 //left
    #define MTOUCH_SENSOR3             AN11 //right
    #define MTOUCH_SENSOR4             AN8 //center
//    #define MTOUCH_SENSOR5              AN5
//    #define MTOUCH_SENSOR6              AN6
//    #define MTOUCH_SENSOR7              AN7
//    #define MTOUCH_SENSOR8              AN11
//    #define MTOUCH_SENSOR9              AN12
//    #define MTOUCH_SENSOR10             AN13
//    #define MTOUCH_SENSOR11             AN11
//    #define MTOUCH_SENSOR12             AN12
//    #define MTOUCH_SENSOR13             AN13
//    #define MTOUCH_SENSOR14             AN14
//    #define MTOUCH_SENSOR15             AN15
//    #define MTOUCH_SENSOR16             AN16
//    #define MTOUCH_SENSOR17             AN17
//    #define MTOUCH_SENSOR18             AN18
//    #define MTOUCH_SENSOR19             AN19
//    #define MTOUCH_SENSOR20             AN20
//    #define MTOUCH_SENSOR21             AN21
//    #define MTOUCH_SENSOR22             AN22
//    #define MTOUCH_SENSOR23             AN23
//    #define MTOUCH_SENSOR24             AN24        // TIP:
//    #define MTOUCH_SENSOR25             AN25        // Extra sensor definitions can be ignored - they will
//    #define MTOUCH_SENSOR26             AN26        // only be implemented if MTOUCH_NUMBER_SENSORS defines that
//    #define MTOUCH_SENSOR27             AN27        // they are active.
//    #define MTOUCH_SENSOR28             AN28
//    #define MTOUCH_SENSOR29             AN29
    /// @endcond


   
   
   
    // Press Threshold Configuration
    #define THRESHOLD_PRESS_SENSOR0     200
    #define THRESHOLD_PRESS_SENSOR1     200         /// @cond
    #define THRESHOLD_PRESS_SENSOR2     200         // <-- Required difference between the sensor's average
    #define THRESHOLD_PRESS_SENSOR3     200        //     value and the current reading to trigger a press
    #define THRESHOLD_PRESS_SENSOR4     200        //     Allowable Range :: [ 1 to 65535 ]
    #define THRESHOLD_PRESS_SENSOR5     200        
    #define THRESHOLD_PRESS_SENSOR6     200  
    #define THRESHOLD_PRESS_SENSOR7     200  
    #define THRESHOLD_PRESS_SENSOR8     200  
    #define THRESHOLD_PRESS_SENSOR9     200  
    #define THRESHOLD_PRESS_SENSOR10    200
    #define THRESHOLD_PRESS_SENSOR11    200
    #define THRESHOLD_PRESS_SENSOR12    200
    #define THRESHOLD_PRESS_SENSOR13    200
    #define THRESHOLD_PRESS_SENSOR14    200
    #define THRESHOLD_PRESS_SENSOR15    200
    #define THRESHOLD_PRESS_SENSOR16    200
    #define THRESHOLD_PRESS_SENSOR17    200
    #define THRESHOLD_PRESS_SENSOR18    200
    #define THRESHOLD_PRESS_SENSOR19    200
    #define THRESHOLD_PRESS_SENSOR20    200
    #define THRESHOLD_PRESS_SENSOR21    200      
    #define THRESHOLD_PRESS_SENSOR22    200         // TIP:
    #define THRESHOLD_PRESS_SENSOR23    200         // Extra threshold definitions for sensors that are not
    #define THRESHOLD_PRESS_SENSOR24    200         // implemented can be ignored.
    #define THRESHOLD_PRESS_SENSOR25    200  
    #define THRESHOLD_PRESS_SENSOR26    200  
    #define THRESHOLD_PRESS_SENSOR27    200  
    #define THRESHOLD_PRESS_SENSOR28    200  
    #define THRESHOLD_PRESS_SENSOR29    200         
    /// @endcond



    #define MTOUCH_RELEASE_THRESH_FACTOR    0.5     // <-- Defines the hysteresis between press and release
                                                    //     In other words, '0.5' would result in a release
                                                    //     threshold equal to half of the press threshold.
                                                    //     Allowable Range :: [ 0.30 to 1.00 ]
 

                                            

        
    
//****************************************************************************************************
//  _   _   _    ____ _____     ____   ____    ____                              
// | | | | / \  |  _ \_   _|   |  _ \ / ___|  / ___|___  _ __ ___  _ __ ___  ___ 
// | | | |/ _ \ | |_) || |_____| |_) | |     | |   / _ \| '_ ` _ \| '_ ` _ \/ __|
// | |_| / ___ \|  _ < | |_____|  __/| |___  | |__| (_) | | | | | | | | | | \__ \
//  \___/_/   \_\_| \_\|_|     |_|    \____|  \____\___/|_| |_| |_|_| |_| |_|___/
//
//****************************************************************************************************

//--------------------------------------------------------------------------------------
// #define APFCON_INITIALIZED       // For processors with an APFCON register(s), this 
//                                  // #define can be uncommented to stop the mTouch 
//                                  // Framework from producing a "remember to set 
//                                  // APFCON" warning.
//                                  //
//                                  // RULE OF PROGRAMMING #4: Register bits initialize, 
//                                  //      by law, to the value you don't want. Always 
//                                  //      explicitly initialize.
//--------------------------------------------------------------------------------------



    // Please define only v1 OR v2 communications.

   // #define MTOUCH_V1_COMMS                             // <-- If defined, outputs CVD data through RS232
                                                
        #define MTOUCH_V1_COMM_SPEED            115200  // <-- For software-implemented RS232 configurations,
                                                        //     this option determines the output baud rate.
                                                        //     Options: 115200 or 9600
                                                    
        #define MTOUCH_V1_COMM_OUTPUT_RAW               // <-- Define to output raw sensor values
        
        #define MTOUCH_V1_COMM_OUTPUT_AVG               // <-- Define to output average sensor values
        
        //#define MTOUCH_V1_COMM_OUTPUT_SLIDERS           // <-- Define to output slider data, if enabled
                                                    
        #define MTOUCH_V1_COMM_DELIMITER        ';'     // <-- Character used to split data in the ASCII stream.
        
        #define MTOUCH_PIC18_TX2_PIN            RPOR2   // <-- If using a PIC18 device, this is needed to 
                                                        //     initialize the UART module to use the correct
                                                        //     output pin for the TX data. Please make this 
                                                        //     #define equal to the name of the register that
                                                        //     the framework should load the 'TX' value into.
        
                                                    
        //#define MTOUCH_V1_COMM_BITBANGED              // <-- If defined - forces the mTouch framework to implement
                                                        //     is communications through software.

            
            #define MTOUCH_SOFT_UART_TXPORT     PORTA   // <-- Used only if MTOUCH_V1_COMM_BITBANGED is defined.
            #define MTOUCH_SOFT_UART_TXTRIS     TRISA   // <-- Used only if MTOUCH_V1_COMM_BITBANGED is defined.
            #define MTOUCH_SOFT_UART_TXPIN      5       // <-- Used only if MTOUCH_V1_COMM_BITBANGED is defined.

        
    //#define MTOUCH_V2_COMMS                           // <-- If enabled, the 'V2' communications will be
                                                        //     implemented. This uses a two-way UART protocol
                                                        //     to talk to a PC GUI and allow real-time edits
                                                        //     of configuration options. An EEPROM on the
                                                        //     device is required to implement this feature.
        
        #define MTOUCH_EEPROM_START_ADR     0x00        // <-- If V2 communications are enabled, this will
                                                        //     determine at which address the mTouch EEPROM 
                                                        //     data should start its data storage. 


//****************************************************************************************************
//  _____ _ _ _            _             
// |  ___(_) | |_ ___ _ __(_)_ __   __ _ 
// | |_  | | | __/ _ \ '__| | '_ \ / _` |
// |  _| | | | ||  __/ |  | | | | | (_| |
// |_|   |_|_|\__\___|_|  |_|_| |_|\__, |
//                                 |___/ 
//
//****************************************************************************************************

    #define MTOUCH_SAMPLES_PER_SCAN     30   //30       // <-- Number of times each sensor will be scanned
                                                    //     before asserting the mTouch_state.dataReady 
                                                    //     bit. See the mTouch Framework's documentation 
                                                    //     for help achieving a specific response time.
                                                    //     Allowable Range :: [ 1 to 65535 ]
                                                
    #define MTOUCH_AVG_UPDATE           3//3           // <-- Determines the weight given to the reading variable
                                                    //     in the calculation of a new average. The reading 
                                                    //     will have a weight of 1/2^MTOUCH_AVG_UPDATE in the average 
                                                    //     calculation. 
                                                    //     Ex: If it is 4, the average calculation would be 
                                                    //         average = (1/16)*reading + (15/16)*average
                                                    //     Allowed Values :: [ 1 to 4 ]

    #define MTOUCH_AVG_RATE             32//32           // <-- Determines how often the average will be updated.
                                                    //     A new calculation will occur every Nth mTouch
                                                    //     decode service call, where N is MTOUCH_AVG_RATE.
                                                    //     (Fastest = 1)
                                                    //     Allowable Range :: [ 1 to 65535 ]

    #define MTOUCH_DECIMATION_MAX_STEP  5           // <-- Maximum change allowed during one CVD sample
    
    #define MTOUCH_JITTER_BITS          6           // <-- Determines the number of bits to use to implement
                                                    //     a randomized delay between mTouch scans.
                                                    //     Allowable Range :: [ 0 to 8 ], 0 = off.

    #define MTOUCH_SCALING              1           // <-- Defines the degree of post-scaling that is 
                                                    //     implemented on the acquisition's accumulation
                                                    //     register before being stored as the result.
                                                    //     The scaling factor should be set so that it is
                                                    //     not possible for the result register to overflow 
                                                    //     (Max result output value is 65535.)
                                                    //     Set this equal to the number of times to right-shift.
                                                    //     Allowable Range :: [ 0 to 8 ]
                                                    //       0 - No scaling is performed.
                                                    //           1:1 ratio between accumulator and result
                                                    //
                                                    //       4 - Accumulator is divided by 16.
                                                    //           16:1 ratio between accumulator and result
                                                    //
                                                    //       8 - Accumulator is divided by 256.
                                                    //           256:1 ratio between accumulator and result
                                                    //
                                                
//****************************************************************************************************
//  ____                     _ _             
// |  _ \  ___  ___ ___   __| (_)_ __   __ _ 
// | | | |/ _ \/ __/ _ \ / _` | | '_ \ / _` |
// | |_| |  __/ (_| (_) | (_| | | | | | (_| |
// |____/ \___|\___\___/ \__,_|_|_| |_|\__, |
//                                     |___/ 
//****************************************************************************************************
    

    #define MTOUCH_POWER_UP_SAMPLES     30 //30     // <-- Determines the number of initialization samples
                                                //     taken at power up. Each sensor will be decoded
                                                //     this number of times.
                                                //     Allowable Range :: [ 1 to 65535 ]

    #define	MTOUCH_BUTTON_TIMEOUT       1000     // <-- Number of consecutive 'pressed' decode decisions
                                                //     before the sensor's state is reset. 
                                                //     Allowable Range :: [ 0 to 65535 ]
                                                
    #define	MTOUCH_DEBOUNCE_RELEASE     0       // <-- Number of consecutive scans a button must be seen
                                                //     as released before a 'released state' is declared
                                                //     Allowable Range :: [ 0 to 65535 ]
 
    #define	MTOUCH_DEBOUNCE_PRESS    	0       // <-- Number of consecutive scans a button must be seen 
                                                //     as pressed before a 'pressed state' is declared
                                                //     Allowable Range :: [ 0 to 65535 ]
                                                
    #define MTOUCH_NEGATIVE_CAPACITANCE 1       // <-- Determines the method used to handle a negative 
                                                //     capacitance shift. In other words, how the system 
                                                //     will respond to a reading value shift in the 
                                                //     opposite direction of a normal shift caused by a 
                                                //     finger. These shifts occur for a variety of reasons 
                                                //     and you may want to adjust this behavior based on 
                                                //     your system.
                                                //
                                                //     Options:
                                                //      0 - Normal Average Behavior
                                                //          (DEFAULT) No special behavior. The average 
                                                //          will update at the normal rate, as always.
                                                //          For Metal-over-Capacitive systems, a negative
                                                //          shift usually means a neighboring sensor is
                                                //          being pressed, so this is the best option to
                                                //          keep the current state of the sensor and
                                                //          allow it to be pressed quickly after a
                                                //          neighbor has been pressed.
                                                //      1 - Increase Average Speed
                                                //          The average will ignore the MTOUCH_AVG_RATE counter
                                                //          and update itself after each new reading
                                                //          value. This has the effect of speeding up
                                                //          the average until it catches up with the
                                                //          negative shift. 
                                                //      2 - Average Directly Follows Reading
                                                //          The average will instantly follow a negative
                                                //          shift. This behavior is best for a system
                                                //          that needs a very fast response time. It
                                                //          will help prevent quick, repeated presses 
                                                //          from being missed due to the average not
                                                //          updating quickly enough.
                                            
    //#define MTOUCH_ACTIVE_MOST_PRESSED_ONLY   // <-- If enabled, only one sensor (the one with the highest
                                                //     shift value) will be able to be pressed at any one time.
                                                
    //#define MTOUCH_TOGGLE_ENABLED             // <-- If enabled, a toggle latch bit will be implemented for
                                                //     each active sensor. You can access the current toggle
                                                //     state using mTouch_GetToggleState(i).
                                                
                                                
//****************************************************************************************************
//  ____  _ _     _                                 _  __        ___               _     
// / ___|| (_) __| | ___ _ __ ___    __ _ _ __   __| | \ \      / / |__   ___  ___| |___ 
// \___ \| | |/ _` |/ _ \ '__/ __|  / _` | '_ \ / _` |  \ \ /\ / /| '_ \ / _ \/ _ \ / __|
//  ___) | | | (_| |  __/ |  \__ \ | (_| | | | | (_| |   \ V  V / | | | |  __/  __/ \__ \
// |____/|_|_|\__,_|\___|_|  |___/  \__,_|_| |_|\__,_|    \_/\_/  |_| |_|\___|\___|_|___/
//                                  
//****************************************************************************************************

// See 'mTouch_config_slider.h'



//****************************************************************************************************
//  ____  _                   __  __           _      
// / ___|| | ___  ___ _ __   |  \/  | ___   __| | ___ 
// \___ \| |/ _ \/ _ \ '_ \  | |\/| |/ _ \ / _` |/ _ \
//  ___) | |  __/  __/ |_) | | |  | | (_) | (_| |  __/
// |____/|_|\___|\___| .__/  |_|  |_|\___/ \__,_|\___|
//                   |_|  
//****************************************************************************************************

// See 'mTouch_config_sleep.h'



//****************************************************************************************************
//     _                   _     _ _   _             
//    / \   ___ __ _ _   _(_)___(_) |_(_) ___  _ __    _
//   / _ \ / __/ _` | | | | / __| | __| |/ _ \| '_ \  (_)
//  / ___ \ (_| (_| | |_| | \__ \ | |_| | (_) | | | |  _
// /_/   \_\___\__, |\__,_|_|___/_|\__|_|\___/|_| |_| (_)
//                |_|  
//   ______     ______    ____                     ____            _             _     
//  / ___\ \   / /  _ \  / ___|  ___ __ _ _ __    / ___|___  _ __ | |_ _ __ ___ | |___ 
// | |    \ \ / /| | | | \___ \ / __/ _` | '_ \  | |   / _ \| '_ \| __| '__/ _ \| / __|
// | |___  \ V / | |_| |  ___) | (_| (_| | | | | | |__| (_) | | | | |_| | | (_) | \__ \
//  \____|  \_/  |____/  |____/ \___\__,_|_| |_|  \____\___/|_| |_|\__|_|  \___/|_|___/
//
//****************************************************************************************************

// See 'mTouch_config_cvdAdvanced.h'
    
       
#endif
