/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_config_cvdAdvanced.h
 *  Dependencies:    
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch CVD Framework Configuration File
 *                   - See documentation for better explanations of all
 *                     configuration options.
 *************************************************************************/
/***********************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/

/** @file   mTouch_config_cvdAdvanced.h
*   @brief  Framework Configuration! Advanced CVD Scanning Options
*/


//****************************************************************************************************
// __        __               __                        _____ _           _             
// \ \      / /_ ___   _____ / _| ___  _ __ _ __ ___   |_   _(_)_ __ ___ (_)_ __   __ _ 
//  \ \ /\ / / _` \ \ / / _ \ |_ / _ \| '__| '_ ` _ \    | | | | '_ ` _ \| | '_ \ / _` |
//   \ V  V / (_| |\ V /  __/  _| (_) | |  | | | | | |   | | | | | | | | | | | | | (_| |
//    \_/\_/ \__,_| \_/ \___|_|  \___/|_|  |_| |_| |_|   |_| |_|_| |_| |_|_|_| |_|\__, |
//                                                                                |___/ 
//****************************************************************************************************
                                            
    #define CVD_SETTLING_DELAY          5       // <-- Adjusts the number of additional instruction cycles
                                                //     to add to the Chold&Csensor settling time.
                                                //     The larger this value is, the more susceptible to
                                                //     noise the application will be. However, there is a
                                                //     minimum amount of time required to get full 
                                                //     sensitivity. 
                                                //     For information on how to tune this parameter,
                                                //     please refer to the mTouch Framework documentation.

    #define CVD_CHOLD_CHARGE_DELAY      5      // <-- Adjusts the number of additional instruction cycles
                                                //     to add to the Chold-charge stage of the CVD scan.
                                                //     For information on how to tune this parameter,
                                                //     please refer to the mTouch Framework documentation.
                                                
    //#define CVD_FORCE_REF_DAC                 // <-- If defined, the framework will use the DAC as the
                                                //     reference source option.

    //#define CVD_SLEEP_FORCE_REF_DAC           // <-- Same as above, but for sleep mode.

    
    
//****************************************************************************************************
//   ____                     _   ____  _           
//  / ___|_   _  __ _ _ __ __| | |  _ \(_)_ __  ___ 
// | |  _| | | |/ _` | '__/ _` | | |_) | | '_ \/ __|
// | |_| | |_| | (_| | | | (_| | |  __/| | | | \__ \
//  \____|\__,_|\__,_|_|  \__,_| |_|   |_|_| |_|___/
//
//****************************************************************************************************

    //#define CVD_GUARD_IO_ENABLED                                  // <-- If defined, the framework will implement a 
                                                                    //     guard sensor using an IO port.
                                                                    //     For information about guard sensors, please refer 
                                                                    //     to the mTouch Framework documentation.
                                                                
        #define CVD_GUARD_PORT              LATA                    // <-- The guard IO's LAT register, prefixed with an underscore.
        #define CVD_GUARD_PIN               2                       // <-- The guard IO's pin number

        
        
    //#define CVD_GUARD_DACOUT_ENABLED                              // <-- If defined, the framework will implement a
                                                                    //     guard sensor using the DAC's DACOUT pin.
                                                                    //     If available, this can produce better results 
                                                                    //     than the regular IO guard.
                                                                    
        #define CVD_GUARD_DACCON0_A     CVD_DACOUT_DACCON0_HALF_VDD // <-- This is the value that will be loaded into the DACCON0
                                                                    //     register prior to ScanA of the differential CVD scan.                                                                    
        #define CVD_GUARD_DACCON1_A     CVD_DACOUT_DACCON1_HALF_VDD // <-- This is the value that will be loaded into the DACCON1
                                                                    //     register prior to ScanA of the differential CVD scan.   
        #define CVD_GUARD_DACCON0_B     CVD_DACOUT_DACCON0_HALF_VDD // <-- This is the value that will be loaded into the DACCON0  
                                                                    //     register prior to ScanB of the differential CVD scan.                  
        #define CVD_GUARD_DACCON1_B     CVD_DACOUT_DACCON1_HALF_VDD // <-- This is the value that will be loaded into the DACCON1  
                                                                    //     register prior to ScanB of the differential CVD scan.  
                                                                    //     Valid Values:
                                                                    //          - Custom hex value for DACCON register (ex: 0xA0)
                                                                    //          - One of the following predefined values:
                                                                    //              - CVD_DACOUT_DACCON0_VSS        /   CVD_DACOUT_DACCON1_VSS
                                                                    //              - CVD_DACOUT_DACCON0_1_3RD_VDD  /   CVD_DACOUT_DACCON1_1_3RD_VDD
                                                                    //              - CVD_DACOUT_DACCON0_HALF_VDD   /   CVD_DACOUT_DACCON1_HALF_VDD
                                                                    //              - CVD_DACOUT_DACCON0_2_3RD_VDD  /   CVD_DACOUT_DACCON1_2_3RD_VDD
                                                                    //              - CVD_DACOUT_DACCON0_VDD        /   CVD_DACOUT_DACCON1_VDD
    
    
    //#define CVD_SLEEP_GUARD_IO_ENABLED                            // Same as above, but defines what happens in sleep mode
        #define CVD_SLEEP_GUARD_PORT        LATD
        #define CVD_SLEEP_GUARD_PIN         7

    //#define CVD_SLEEP_GUARD_DACOUT_ENABLED
        #define CVD_SLEEP_GUARD_DACCON0_A   CVD_DACOUT_DACCON0_VDD          
        #define CVD_SLEEP_GUARD_DACCON1_A   CVD_DACOUT_DACCON0_VDD          
        #define CVD_SLEEP_GUARD_DACCON0_B   CVD_DACOUT_DACCON0_VSS          
        #define CVD_SLEEP_GUARD_DACCON1_B   CVD_DACOUT_DACCON0_VSS          


//****************************************************************************************************
//  __  __       _               _   ____                                
// |  \/  |_   _| |_ _   _  __ _| | / ___|  ___ _ __  ___  ___  _ __ ___ 
// | |\/| | | | | __| | | |/ _` | | \___ \ / _ \ '_ \/ __|/ _ \| '__/ __|
// | |  | | |_| | |_| |_| | (_| | |  ___) |  __/ | | \__ \ (_) | |  \__ \
// |_|  |_|\__,_|\__|\__,_|\__,_|_| |____/ \___|_| |_|___/\___/|_|  |___/
//  
//****************************************************************************************************


    //#define CVD_MUTUAL_ENABLED                      // <-- If defined, the framework will integrate a mutual
                                                    //     sensor into the differential CVD scan.
    //#define CVD_MUTUAL_EXCLUSIVE_ENABLED          // <-- If defined, the framework will perform an
                                                    //     exclusively-mutual scan. There will be no charge
                                                    //     averaging between the external and internal caps.
                                                
        #define CVD_MUTUAL_LAT              LATA3   // <-- The mutual sensor's LAT pin label
        #define CVD_MUTUAL_TRIS             TRISA3  // <-- The mutual sensor's TRIS pin label
                                                
    //#define CVD_SLEEP_MUTUAL_ENABLED              // Same as above, but for sleep mode.
    //#define CVD_SLEEP_MUTUAL_EXCLUSIVE_ENABLED
    
        #define CVD_SLEEP_MUTUAL_LAT        LATA2
        #define CVD_SLEEP_MUTUAL_TRIS       TRISA2

