//
// NOTE: You may delete this file. It is used to generate the mTouch documentation 
//       located in the /Microchip/Help folder of the Microchip Application Libraries.
//



/** @file   mTouch_documentation.h
*   @brief  Internal framework header file used to generate this documentation. Safe to delete.
*/
/**
* @mainpage
*
* @section Intro Introduction
* Thank you for downloading the mTouch Framework.
*
* The mTouch Framework is a software package enabling designers to easily integrate touch technologies
* to their application. It combines high sensitivity with conducted and radiated noise immunity.
*
* Simple configuration file allows for easy setup. Microchip also offers full source code access for more
* advanced touch designers.
*
* <br><center>
* <table border="0">
* <tr valign="top">
* <td>@subpage GettingStarted
*     @li Quick Start Guides
*     @li Creating a New MPLAB Project
*     @li Detailed Implementation Notes
*     @li Customization Guides</td>
* <td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
* <td>@subpage FrameworkFeatures "mTouch Framework Features"
*     @li Overview
*     @li Slider Configuration Guide
*     @li Sleep-mode Configuration Guide
*     @li Guard Ring Configuration Guide
*     @li Mutual Sensor Configuration Guide</td>
* </tr>
* <tr valign="top">
* <td>@subpage Troubleshoot
*     @li Increasing Sensitivity
*     @li Faster Response Times
*     @li Tuning the CVD Waveform
*     @li Establishing PC Communications</td>
* <td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
* <td>@subpage ResourceRequirements "Resource Requirements"
*     @li Supported PIC&reg; microcontrollers
*     @li Hardware module usage
*     @li ROM, RAM, and processing requirements</td>
* </tr>
* <tr valign="top">
* <td>@subpage Changelog "Version Change Log"</td>
* <td>&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;</td>
* <td>@subpage SoftwareLicense</td>
* </tr>
* </table></center>
*
* <br><br>
*
* @image html filetree.jpg "mTouch Framework File Tree"
*
* <ul>
* <li> <b>/Microchip</b><br>
* Contains the library, read-only source files for the mTouch Framework.
*   <ul>
*   <li> <b>/Help</b><br>
*       Contains documentation for all the Microchip Application Libraries.
*       <ul>
*       <li> <b>/mTouch Framework Help Content</b><br>
*       Contains the documentation for the mTouch Framework.
*       </ul>
*   <li> <b>/mTouchCap</b>
*       <ul>
*       <li> <b>/PIC16F CVD FW Library</b><br>
*       Contains the C source (.c) files for the mTouch Framework.<br>
*       IMPORTANT: The files in this directory should never be directly editted.
*           <ul>
*           <li> mTouch.c implements the initialization, filtering, and decoding functions for the framework.
*           <li> mTouch_Acquistion.c implements a custom mTouch Acquisition method specific to the PIC and configuration.
*           <li> mTouch_Comm.c performs the required actions to communicate with the mTouch GUI.
*           <li> main_example.c contains an example application implementation.<br><br>
*           <li> <b>/includes</b><br>
*           Contains the C header (.h) files for the mTouch Framework. <br>
*           IMPORTANT: The files in this directory should never be directly editted.<br><br>
*           This header files should be copied to the local application source directory and placed in a folder named 'includes'.
*               <ul>
*               <li> mTouch_config.h stores the configuration options for the framework.
*               <li> mTouch_config_sleep.h stores the configuration options for sleep-mode sensors.
*               <li> mTouch_config_slider.h stores the configuration options for sliders and wheels.
*               <li> mTouch_config_cvdAdvanced.h stores the advanced CVD acquisition options.
*               </ul><br>
*           These header files should only be copied to the local directory if application-specific modifications are being made. The compiler will prioritize including local files over library files if the names match.
*               <ul>
*               <li> mTouch.h the master header file for the mTouch Framework that defines global variables and includes other files based on the current configuration. This is the only file you need to #include to use the API in your project.<br>
*               <li> mTouch_HardwareProfile_12F61x.h is one of many PIC-family-specific header files that contains information such available hardware modules, analog channel-to-port mapping, and default register configurations.
*               <li> mTouchCVD_macroLibrary_PIC16F1.h is one of several PIC-core-specific header files that contains macros for generating the acquisition module of the mTouch Framework.
*               <li> mTouch_processConfiguration.h performs much of the pre-compiling work to confirm a valid configuration and then set up the flags and variables to aid the compiler in generating a custom mTouch application.
*               </ul>
*           </ul>
*       </ul>
*   </ul>
* <li> <b>/mTouchCapDemos</b>
*   <ul>
*   <li> <b>/PIC16F_CVD_Demos</b><br>
*       Provides some example, pre-configured projects for various PIC microcontrollers and setups.
*       <ul>
*       <li> <b>/Cap Touch CSM-CVD Eval Board 04-02091 Rev D1</b><br>
*           Example project implementing CVD on the mTouch evaluation board.
*           <ul>
*           <li> <b>/code - no cover</b><br>
*               The thresholds in this project have been raised due to the higher level of sensitivity.
*           <li> <b>/code - with cover</b><br>
*               The thresholds in this project are lower than in the other due to the lower level of sensitivity.
*               <ul>
*               <li> <b>/MPLAB.8</b><br>
*                   Pre-built MPLAB 8 project file with the correct configuration
*               <li> <b>/MPLAB.X</b><br>
*                   Pre-built MPLAB X project file with the correct configuration
*               </ul>
*           </ul>
*       <li> <b>/Cap Touch CVD Eval Board 233-04-2028 Rev B</b><br>
*           Example project implementing CVD on the mTouch evaluation board.
*       <li> <b>/PIC12F1822 Example - 4 sensors</b><br>
*       <li> <b>/PIC16F617 Example - 4 sensors</b><br>
*       <li> <b>/PIC16F1936 Example - 4 sensors</b><br>
*       <li> <b>/Utilities</b><br>
*           Contains useful tools and reference materials that are not strictly related to the mTouch Framework.
*           <ul>
*           <li> <b>/mTouch GUI</b><br>
*               Allows the designer to see live data from the mTouch sensors
*           <li> <b>/Pickit Serial Loader</b><br>
*               Easy reflashing tool for the PKSA to have it behave as a UART-to-USB converter.
*           </ul>
*       </ul>
*   </ul>
* </ul>
*
*/
/**
* @page Changelog ChangeLog
*
* @section cl0200 Version 2.0
* @li Added support for a large number of processors. Refer to this documentations 'Resource Requirements' page for a full listing.
* @li Structures have been implemented for a large number of mTouch variables to maximize the efficiency of RAM resources. Several previously supported processors are now able to implement more sensors due to this change.
* @li New feature: Sleep-mode sensors can be used to send the framework into a low power state after it detects no change on the active-mode after a configurable amount of time. For more information about this feature, refer to the mTouch Framework flow chart.
* @li New feature: Sliders and wheels can be implemented by editing the mTouch_config_slider.h options. Supports up to 10 sliders with 8 sensors per slider.
* @li New feature: A most-pressed algorithm can be implemented by uncommenting #MTOUCH_ACTIVE_MOST_PRESSED_ONLY. (mTouch_config.h)
* @li New feature: Toggle mode is now available by uncommenting #MTOUCH_TOGGLE_ENABLED. (mTouch_config.h) It will toggle the output's value only when it detects a new press - not when it sees a release. Example API available in main.c. 
* @li New feature: Guard sensors can be implemented using any general IO pin or through the DACOUT pin. (mTouch_config_cvdAdvanced.h)
* @li New feature: Mutual sensors can be implemented to detect increased coupling between two interconnected sensors. (mTouch_config_cvdAdvanced.h)
* @li New feature: "V2" communications allow users to update configuration options without reprogramming the processor. Not yet compatible with sleep mode. Configuration options and thresholds are stored in EEPROM when this is implemented.
* @li New configuration option #MTOUCH_INTEGRATION_TYPE adjusts the way the framework integrates with the application (mTouch_config.h)
* @li New configuration option #MTOUCH_ISR_TIMER changes which 8-bit timer is used for the mTouch interrupt (mTouch_config.h)
* @li New configuration option #MTOUCH_SCAN_FUNCTIONALITY alternates between scanning one sensor per interrupt or scanning all sensors once per interrupt (mTouch_config.h)
* @li New configuration option #MTOUCH_ADC_CONTROL determines whether the framework releases the ADC after performing its scan (mTouch_config.h)
* @li New configuration option #APFCON_INITIALIZED can be used to disable "make sure you initialize" warnings on parts with APFCON registers (mTouch_config.h)
* @li New configuration option #MTOUCH_DECIMATION_MAX_STEP adjusts the maximum step size for sensors while going through the decimation filter (mTouch_config.h)
* @li New configuration option #MTOUCH_JITTER_BITS adjusts the number of bits used to implement the random seed value for jittering (mTouch_config.h)
* @li New configuration option #CVD_FORCE_REF_DAC can be defined to force the CVD acquisition to use the DAC as the reference for the ADC's hold capacitor (mTouch_config_cvdAdvanced.h)
* @li Configuration option #CVD_JITTER_ENABLE has been removed. To disable jittering, set the #MTOUCH_JITTER_BITS value to 0.
* @li #MTOUCH_SCALING has been adjusted to allow a larger number of scaling options. (mTouch_config.h)
* @li New configuration option #MTOUCH_V1_COMM_BITBANGED can be enabled to force the communications module to perform the UART communications through software only. The bit-banged code has been changed to assembly for more precise timing.
*
* @section cl0101 Version 1.1
* @li Non-enhanced Core Device Support
* @li HI-TECH 9.81 Lite and Standard Compiler Support
* @li Acquisition variable array #sensor_data is now initialized to 0x47F to reduce the required initialization time
* @li New configuration option #CVD_SETTLING_DELAY adjusts the amount of time the CVD acquisition will wait for the voltages on the external sensor and internal ADC hold capacitor to settle. See @ref ts-Waveform "this guide" for more information.
* @li New configuration option #CVD_CHOLD_CHARGE_DELAY adjusts the amount of time the CVD acquisition will wait for the voltage on the internal ADC hold capacitor to reach steady state (VDD or VSS) before connecting the two capacitors and allowing their voltages to settle. See @ref ts-Waveform "this guide" for more information.
* @li The scan order of the CVD acquisition has been reversed to ensure maximum sensitivity for larger external sensor capacitances. This means the waveform will change when viewed on a scope. See @ref ts-Waveform "this guide" for more information.
* @li #GENERATE_STORAGE_FUNCTION() and #GENERATE_JUMP_TABLE() macros have been redesigned with a more elegant implementation.
* @li The mTouch.h file has been split-in-two in order to accomodate the Microchip Application Libraries' directory structure. A local copy of mTouch.h should be included in your application's includes directory. mTouch_processConfiguration.h, the other half of the original mTouch.h file, only needs to be available in the library folders and does not require a local copy.
* @li Sensor 0's reference is now defined in the process configuration header file to help de-clutter the acquisition.c file.
* @li Many documentation updates including:
*   <ul>
*   <li> New 'Troubleshooting' sections designed to make it easier to solve common problems:
*       <ul>
*       <li> @ref ts-Sensitivity    "Increasing Sensitivity"
*       <li> @ref ts-ResponseTime   "Adjusting the Response Time"
*       <li> @ref ts-Waveform       "Tuning the CVD Waveform"
*       <li> @ref ts-Comms          "Establishing PC Communications"
*       </ul>
*   <li> Redesigned main page to help new users find information
*   <li> Guides to creating new projects in @ref GettingStartedNewP8 "MPLAB 8" and @ref GettingStartedNewPX "MPLAB X"
*   </ul>
*
* @section cl0100 Version 1.0 - First Public Release
* @li 64-Pin Device Support
* @li HI-TECH 9.81 PRO Compiler Support
* @li Comm/HardwareProfile update to support devices with >1 UART modules
* @li New MTOUCH_V1_COMM_OUTPUT_RAW and MTOUCH_V1_COMM_OUTPUT_AVG configuration options to quickly adjust comm output
*/
/**
* @page ResourceRequirements Supported PIC&reg; MCUs and Requirements
*
* @section rrCompiler Required Compiler
* @li <b>HI-TECH C Compiler for PIC10/12/16 MCUs v9.81 or later</b><br>
* If a non-PRO license is used to compile the framework, the maximum number of supported sensors will decrease. Noise 
* testing was performed using the PRO license.<br>
* Using a third-party compiler may result in unknown behavior due to the heavy use of compile-time code formatting.<br>
*
* @section rrModules Module Requirements
* @li <b>8-bit Hardware Timer Module for Interrupt</b><br>
* This module is used to schedule regular ADC scans on the sensors. 8-bit timers normally interrupt after 256 counts; however, the mTouch
* framework pre-loads the timer's register with a pseudo-random seed value to slightly jitter the sampling rate. This means that the
* average number of counts before it interrupts is 224.<br>
* You control the TMR0 prescaler. The framework does not touch the OPTION register.<br>
* NOTE: No timer is required if #MTOUCH_INTEGRATION_TYPE is defined as MTOUCH_CALLED_FROM_MAINLOOP.
*
* @li <b>Analog/Digital Converter</b><br>
* The ADC is the core module used by the mTouch framework to acquire a sensor reading. If you wish to perform an ADC reading in 
* between mTouch scans you will need to set the #MTOUCH_ADC_CONTROL configuration option to MTOUCH_RELEASES_ADC_AFTER_SCAN.
* The ADC is initialized for the framework by the mTouch_Init() function.<br>
*
* @li <b>Universal Synchronous/Asynchronous Receiver/Transmitter</b> <i>(optional)</i><br>
* If #MTOUCH_V1_COMMUNICATIONS or #MTOUCH_V2_COMMUNICATIONS is enabled, the mTouch Framework will automatically configure and begin
* outputting sensor data through the UART module. If no such module exists for the selected part, a software-based implementation
* can be included for the V1 communication functions.<br>
*
* @section rrMemory Memory/Processing Requirements
* Roughly, the amount of time that will be spent in the mTouch ISR is based on the TMR0 prescaler value.
* 
* <center>
* <table>
* <tr><td><b>TMR0 Prescaler</b></td><td><b>mTouch ISR Processing Duty Cycle</b></td></tr>
* <tr><td align="center">1</td><td align="center"><i>Invalid</i></td></tr>
* <tr><td align="center">2</td><td align="center">40%</td></tr>
* <tr><td align="center">4</td><td align="center">20%</td></tr>
* <tr><td align="center">8</td><td align="center">10%</td></tr>
* </table>
* </center>
*
* The program memory and RAM requirements are determined mainly by the number of sensors.
* <center>
* <table border="0">
* <tr valign="top">
* <td><center><b>Code Size / Program Memory (words)</b></center></td>
* <td><center><b>Data Space / RAM (bytes)</b></center></td>
* </tr>
* <tr valign="top">
* <td>
*     <table>
*         <tr><td>&nbsp;</td><td colspan="2" align="center"><b>Standard Midrange</b></td><td colspan="2" align="center"><b>Enhanced Midrange</b></td></tr>
*         <tr><td># Sensors</td><td align="center"><b>PRO</b></td><td align="center"><b>LITE</b></td><td align="center"><b>PRO</b></td><td align="center"><b>LITE</b></td></tr>
*         <tr><td align="center">1</td><td align="center">643</td><td align="center">865</td><td align="center">624</td><td align="center">849</td></tr>
*         <tr><td align="center">2</td><td align="center">924</td><td align="center">1171</td><td align="center">782</td><td align="center">1141</td></tr>
*         <tr><td align="center">3</td><td align="center">1030</td><td align="center">1322</td><td align="center">875</td><td align="center">1301</td></tr>
*         <tr><td align="center">4</td><td align="center">1130</td><td align="center">1471</td><td align="center">968</td><td align="center">1459</td></tr>
*         <tr><td align="center">5</td><td align="center">1229</td><td align="center">1837</td><td align="center">1061</td><td align="center">1616</td></tr>
*         <tr><td align="center">6</td><td align="center">1343</td><td align="center">1945</td><td align="center">1157</td><td align="center">1773</td></tr>
*         <tr><td align="center"><b>+1</b></td><td align="center"><b>+100</b></td><td align="center"><b>+150</b></td><td align="center"><b>+110</b></td><td align="center"><b>+150</b></td></tr>
*     </table>
* </td>
* <td>
*     <table>
*         <tr><td>&nbsp;</td><td colspan="2" align="center"><b>Standard Midrange</b></td><td colspan="2" align="center"><b>Enhanced Midrange</b></td></tr>
*         <tr><td># Sensors</td><td align="center"><b>PRO</b></td><td align="center"><b>LITE</b></td><td align="center"><b>PRO</b></td><td align="center"><b>LITE</b></td></tr>
*         <tr><td align="center">1</td><td align="center">45</td><td align="center">51</td><td align="center">36</td><td align="center">44</td></tr>
*         <tr><td align="center">2</td><td align="center">58</td><td align="center">66</td><td align="center">48</td><td align="center">56</td></tr>
*         <tr><td align="center">3</td><td align="center">68</td><td align="center">76</td><td align="center">58</td><td align="center">66</td></tr>
*         <tr><td align="center">4</td><td align="center">78</td><td align="center">86</td><td align="center">68</td><td align="center">76</td></tr>
*         <tr><td align="center">5</td><td align="center">88</td><td align="center">96</td><td align="center">78</td><td align="center">86</td></tr>
*         <tr><td align="center">6</td><td align="center">98</td><td align="center">106</td><td align="center">88</td><td align="center">96</td></tr>
*         <tr><td align="center"><b>+1</b></td><td align="center"><b>+10</b></td><td align="center"><b>+10</b></td><td align="center"><b>+10</b></td><td align="center"><b>+10</b></td></tr>
*     </table>
* </td>
* </tr>
* </table>
* </center>
*
*
* @section PICSupport Supported PIC&reg; Microcontrollers
* @li <b>PIC12/16F61x Family</b>
* <ul>
* <li>PIC12F/HV615      :: 2 sensor limit (Lite: 1 sensor limit)
* <li>PIC12F617         :: All 4 sensors supported
* <li>PIC16F/HV616      :: All 8 sensors supported
* </ul>
* @li <b>PIC12/16F68x Family</b>
* <ul>
* <li>PIC12F683         :: All 4 sensors supported
* <li>PIC16F677         :: 8 sensor limit
* <li>PIC16F684         :: All 8 sensors supported
* <li>PIC16F685         :: All 12 sensors supported
* <li>PIC16F687         :: 8 sensor limit
* <li>PIC16F688         :: All 8 sensors supported
* <li>PIC16F689         :: All 12 sensors supported
* <li>PIC16F690         :: All 12 sensors supported
* </ul>
* @li <b>PIC16F72x Family</b>
* <ul>
* <li>PIC16F/LF722      :: 7 sensor limit
* <li>PIC16F/LF722A     :: 8 sensor limit
* <li>PIC16F/LF723      :: All 11 sensors supported
* <li>PIC16F/LF723A     :: All 11 sensors supported
* <li>PIC16F/LF724      :: All 11 sensors supported
* <li>PIC16F/LF726      :: All 11 sensors supported
* <li>PIC16F/LF727      :: All 11 sensors supported
* </ul>
* @li <b>PIC16F78x Family</b>
* <ul>
* <li>PIC16F/HV785      :: 9 sensor limit
* </ul>
* @li <b>PIC16F8xx Family</b>
* <ul>
* <li>(Under development)
* </ul>
* @li <b>PIC16F9xx Family</b>
* <ul>
* <li>(Under development)
* </ul>
* @li <b>PIC16F150x Family</b>
* <ul>
* <li>PIC16F/LF1507     :: 12 sensors supported
* </ul>
* @li <b>PIC16F151x Family</b>
* <ul>
* <li>PIC16F/LF1516     :: All 17 sensors supported
* <li>PIC16F/LF1518     :: All 17 sensors supported
* <li>PIC16F/LF1517     :: 22 sensor limit
* <li>PIC16F/LF1519     :: 22 sensor limit
* </ul>
* @li <b>PIC16F152x Family</b>
* <ul>
* <li>PIC16F/LF1526     :: 22 sensor limit
* <li>PIC16F/LF1527     :: 22 sensor limit
* <li><i>This family does not support single-sensor applications.</i>
* </ul>
* @li <b>PIC16F182x Family</b>
* <ul>
* <li>PIC12F/LF1822     :: All 4 sensors supported
* <li>PIC16F/LF1823     :: All 8 sensors supported
* <li>PIC16F/LF1824     :: All 8 sensors supported
* <li>PIC16F/LF1825     :: All 8 sensors supported
* <li>PIC16F/LF1826     :: All 12 sensors supported
* <li>PIC16F/LF1827     :: All 12 sensors supported
* <li>PIC16F/LF1828     :: All 12 sensors supported
* <li>PIC16F/LF1829     :: All 12 sensors supported
* </ul>
* @li <b>PIC16F184x Family</b>
* <ul>
* <li>PIC12F/LF1840     :: All 4 sensors supported
* <li>PIC16F/LF1847     :: All 12 sensors supported
* </ul>
* @li <b>PIC16LF190x Family</b>
* <ul>
* <li>PIC16LF1902       :: 8 sensor limit
* <li>PIC16LF1903       :: All 11 sensors supported
* <li>PIC16LF1906       :: All 11 sensors supported
* <li>PIC16LF1904       :: All 14 sensors supported
* <li>PIC16LF1907       :: All 14 sensors supported
* </ul>
* @li <b>PIC16F193x Family</b>
* <ul>
* <li>PIC16F/LF1933     :: All 11 sensors supported
* <li>PIC16F/LF1936     :: All 11 sensors supported
* <li>PIC16F/LF1938     :: All 11 sensors supported
* <li>PIC16F/LF1934     :: All 14 sensors supported
* <li>PIC16F/LF1937     :: All 14 sensors supported
* <li>PIC16F/LF1939     :: All 14 sensors supported
* </ul>
* @li <b>PIC16F194x Family</b>
* <ul>
* <li>PIC16F/LF1946     :: All 17 sensors supported
* <li>PIC16F/LF1947     :: All 17 sensors supported
* </ul>
* @li <b>PIC18FxxJ50 Family</b>
* <ul>
* <li>(Under development)
* </ul>
*
*/
/**
* @page FrameworkFeatures mTouch Framework Features
*
* @section Features mTouch Framework Implementation Features
* We focused on creating a system that exhibits high signal sensitivity and noise immunity. To achieve these goals, we implemented the following features:
* @li @ref Configuration "Simple configuration file" allows for easy customization of the application 
* @li <b>Detailed warning and error messages</b> guide the user in case of an incorrect configuration value
* @subsection AcquisitionFeatures - Acquisition Module 
* @li @ref digAcq "Differential CVD scanning" method maximizes signal and reduces common mode noise concerns
* @li @ref ISR() "Sample rate jittering" algorithm eliminates issues with resonant noise frequencies
* @li Generated assembly implementation <b>maximizes noise immunity</b> and <b>minimizes execution time</b>
* @li Decimate-filtered result register seeding an accumulator array provides <b>impulse noise rejection</b> and an increase in signal through <b>oversampling</b>
* @li @ref CVD_READ_SENSOR() "Single sensor scanning" uses a unique scanning method so there are <b>no extra reference pin requirements</b>
* @li Previous sensor scanning removes the need to have a dedicated reference sensor - freeing up a pin!
* @subsection FilteringFeatures - Filtering Module 
* @li @ref mTouch_UpdateAverage() "Baseline average" tracks environmental changes to maximize response time
* @li Enhanced filtering options allow for <b>customizable filter behavior</b> and update speed based on the application's requirements
* @subsection DecodingFeatures - Decoding Module 
* @li @ref Decoding "Individual thresholds" provide the ability to adjust for different sensor sizes and types
* @li <b>Threshold hysteresis</b> reduces sensor flickering in noisy conditions as well as increasing the reliability of sensor state transitions
* @li Configurable @ref Decoding "press timer" to implement a <b>maximum press duration</b> for the application
* @li <b>Debouncing counters</b> provided to further increase reliability, if desired.
* @subsection CommunicationFeatures - Communication 
* @li @ref digCom "Preconfigured UART implementation" to report data to a PC
* @li Compile-time generated communication protocol allows for <b>compatibility with any version of the CVD Framework GUI</b>
*
*
* @section feat-Slider Slider and Wheel Configuration
* The slider module has been designed to implement the sliders as efficiently as possible, given the number of enabled sliders. For example, if one slider is enabled, the decoding more efficient than for three sliders.
*
* The framework can support up to 10 sliders and up to 8 sensors per slider. The sensors used for the slider must be defined as normal active-mode sensors. They will be scanned as a normal sensor and then later decoded as part of the slider. (This means you can have a sensor act both as a regular button and as part of a slider at the same time.) The framework cannot support 80 individual sensors, so many of the sensors would need to be reused in multiple sliders for the full "10 sliders w/ 8 sensors each" limit to be reached.
*
* To enable a single slider, open mTouch_config_slider.h:
* @li Set #MTOUCH_NUMBER_OF_SLIDERS to 1
* @li Set #MTOUCH_NUMBER_SLIDER0_SENSORS to the number of sensors that will be used to implement the slider.
* @li Set #MTOUCH_SLIDER0_SENSOR0 to the index value of the active-mode sensor which will be considered 'Sensor0' for this slider. The index is determined by the options in mTouch_config.h.
* @li Make sure there is a definition for each sensor in the slider. ex: MTOUCH_SLIDER0_SENSOR1 and MTOUCH_SLIDER0_SENSOR2 if Slider 0 has 3 sensors.
* @li If this slider is implemented as a wheel on the PCB (the start and end points are connected), define MTOUCH_SLIDER0_WHEEL_ENABLE.
* @li The slider's output value will be stored in the mTouch_slider[] array. Example API for handling the slider output can be found in the main.c file.
*
*
* @section feat-Sleep Sleep-mode Configuration
* @note Sleep-mode is not currently compatible with the V2 communications and GUI. We are working to add this functionality soon.
*
* Sleep-mode allows the framework to scan a select number of sensors (8 or fewer), make a decoding decision on the new data, and either wake up to perform active-scanning, or go to sleep to save power. For more information on how this works, please refer to the framework flow chart.
*
* Sleep-mode is implemented when MTOUCH_NUMBER_OF_SLEEP_SENSORS is 1 or greater. An example API is available in the main.c file.
*
* On power-up, the system will first initialize all active-mode sensors, then initializes sleep-mode sensors, and executes a sleep command once finished. After this point, the intialization is complete and the system runs as shown in the flow chart. 
*
*
* @section feat-Guard Guard Ring Configuration
* Guard rings can be used to minimize the parasitic capacitance affecting your capacitive sensors. The theory is that we want to decrease the voltage differential between the capacitive sensor and its environment. We can do this by driving a plane on the board that surrounds the sensors to mimic the voltage of the sensor when it is connected to the internal ADC hold capacitor.
*
* @note One guard ring can be shared among all sensors.
*
* There are two ways to implement the guard ring in the framework:
*
* @subsection feat-Guard-DACOUT - Using the DACOUT pin:
* This is the easiest and most fine-tunable of the two options. #CVD_GUARD_DACOUT_ENABLED should be defined to implement this feature. The mTouch_config_cvdAdvanced.h file also contains options to change the values that are loaded into the DACCON registers each time a scan is performed. These values will determine where the guard ring will settle. The closer this comes to the actual voltage of the sensor during the settling phase the scan, the more effective the guard will be in limiting parasitics. Also, notice how there are two sets of DACCON values - one is labeled 'A' and the other is labeled 'B'. 
*
* 'A' refers to the first half of the CVD scan when the external sensor is held low, the internal hold capacitor is charged high, and then they are connected. 'B' refers to the second half where the reverse is true. The settling voltage will be different for each of these, so different DACCON value are required.
*
* Some predefined values have been provided to make the configuration process more simple:
* @li #CVD_DACOUT_DACCON0_HALF_VDD and #CVD_DACOUT_DACCON1_HALF_VDD can be used for both scan A and scan B if the external and internal capacitors are equivalent.
* @li #CVD_DACOUT_DACCON0_1_3RD_VDD and #CVD_DACOUT_DACCON1_1_3RD_VDD for scan 'A', and #CVD_DACOUT_DACCON0_2_3RD_VDD and #CVD_DACOUT_DACCON1_2_3RD_VDD for scan 'B' are also good starting points.
*
* You may also decide to not use the pre-defined options and provide your own DACCON values. Simply put the desired register value in place of the labels.
*
* @subsection feat-Guard-IO - Using an I/O pin:
* Alternatively, you can use any I/O pin by defining the #CVD_GUARD_IO_ENABLED option. In this case, #CVD_GUARD_PORT and #CVD_GUARD_PIN determine which I/O is used for this purpose.
*
* Since we're trying to mimic the voltage of the sensor, series resistance will need to be added to the guard pin line to increase the fall and rise time. This resistance value should be tuned to provide the closest possible voltage to the sensor during the charge settling phase of the CVD scan. It will likely not be able to mimic the sensor's voltage as well as the DACOUT pin, however - but the closer you can get it, the better.
*
*
* @section feat-Mutual Mutual Sensor Configuration
* @note This feature is still under development, but it is functional and ready for use.
*
* In some cases, you may wish to measure the amount of coupling between a driven pin and a capacitive sensor. Detecting ungrounded metal, for instance, is not always easily done with a normal CVD scan. To enable a mutual sensor, define the #CVD_MUTUAL_ENABLED option in mTouch_config_cvdAdvanced.h. This will cause the defined mutual pin to be driven so that increased coupling will cause a shift in the same direction as adding capacitance to a normal capacitive sensor. Both the CVD scan and the mutual drive will be occuring at the same time. This means you are getting the benefit of both scans in one. As the distance between the object-to-be-detected and the sensor gets smaller, the impact of the mutual scan will increase. As the distance gets larger, the normal CVD scan will provide a majority of the sensitivity.
*
* If you wish to use only a mutual scan and stop performing the normal CVD acquisition, you can define the #CVD_MUTUAL_EXCLUSIVE_ENABLED instead. 
*/

/**
* @page GettingStarted Getting Started
*
* @section Expect What to Expect from the mTouch Framework
* When designing with the mTouch Framework, it's important to understand the system's overall behavior. Borrowing
* from the <a href="http://en.wikipedia.org/wiki/Software_framework">Wikipedia article</a> on software frameworks, here
* are the distinguishing features of a framework over a library or normal application:<br>
* @li <b>Inversion of Control</b><br><i>In a framework, unlike in libraries or normal user applications, the overall program's
*                                   flow of control is not dictated by the caller, but by the framework.</i><br>
*                                   The mTouch Framework requires the TMR0 interrupt, ADC, and a significant portion
*                                   of the overall processing time. Developers do, however, have full control over the
*                                   main loop of their application.<br>
*                                   (<b>See also:</b> @ref ResourceRequirements "Resource Requirements")<br><br>
* @li <b>Default Behavior</b><br><i>A framework has default behavior. This default behavior must actually be some useful behavior and not a series of no-ops.</i><br>
*                                   The mTouch Framework's default behavior is to perform a differential CVD scan on the sensors, perform digital
*                                   filtering on the acquired signal, and decode the signal's behavior into a Yes/No press state.<br><br>
* @li <b>Extensibility</b><br><i>A framework can be extended by the user usually by selective overriding or specialized by user code providing specific functionality.</i><br>
*                                   The mTouch Framework comes with a predefined mTouch_Service() function which will perform all of the decoding
*                                   necessary for normal mTouch capacitive buttons. For enhanced capabilities such as sliders, wheels, proximity sensors,
*                                   level sensors, and other capacitive touch uses a custom service function may be required. This simple process is explained
*                                   in detail in the quick start guide.<br>
*                                   (<b>See also:</b> @ref cusService)<br><br>
* @li <b>Non-Modifiable Framework Code</b><br><i>The framework code, in general, is not allowed to be modified. Users can extend the framework, but not modify its code.</i><br>
*                                   The mTouch Framework is provided as an open source solution, so it is possible to edit the core acquisition code of the framework. 
*                                   Doing this, however, is <b>highly discouraged</b> and could easily impact the noise immunity of your final solution. For this reason,
*                                   we consider the mTouch ISR and all @c include/ files as essentially non-modifiable.<br>
*
* @section GSGuides Developer Resource Guides
*
* <b>Quick Start Guides</b>
*
* Use these quick start guides to get you up and running with the mTouch Framework.
* @li @subpage GettingStartedEval   "mTouch CVD Evaluation Board"
* @li @subpage GettingStartedCustom "Custom Hardware"
* @li @subpage GettingStartedNewP8  "Creating a New MPLAB 8 Framework Project"
* @li @subpage GettingStartedNewPX  "Creating a New MPLAB X Framework Project"
* @li @subpage PKSARS232            "Using the PICKit Serial Analyzer as an RS232-to-USB converter"
*
* @subpage digging
*
* These files will describe the implementation of the framework's three main modules.
* @li @ref digAcq               "Acquisition Method and Implementation"
* @li @ref digFil               "Filtering and Decoding Implementation"
* @li @ref digCom               "Communication Implementation"
* @li @ref RequiredIncludes     "Required Include Files" (and which ones you can delete)
*
* @subpage custom
*
* These files describe some common customization challenges and solutions.
* @li @ref cusService           "Replacing Default Filtering/Decode Logic with a Custom Service() Function"
*/
/**
* @page digging Digging into the Framework
* @li @subpage digAcq
* @li @subpage digFil
* @li @subpage digCom
* @li @subpage RequiredIncludes
*/
/**
* @page custom Customizing Framework Behavior
* @li @subpage cusService
*/
/**
* @page Troubleshoot Troubleshooting the Framework
* @li @subpage ts-Sensitivity   "Increasing Sensitivity"
* @li @subpage ts-ResponseTime  "Adjusting the Response Time"
* @li @subpage ts-Waveform      "Tuning the CVD Waveform"
* @li @subpage ts-Comms         "Establishing PC Communications"
*/
/**
* @page ts-Sensitivity Increasing Sensitivity - Troubleshooting the Framework
*
* @li <b>1. Make sure your application's CVD waveform is correct.</b><br><br>
*        See the @ref ts-Waveform "Waveform Troubleshooting guide" for help with this process. 
* 
* <br>
*
* @li <b>2. Use #MTOUCH_SAMPLES_PER_SCAN to increase the amount of oversampling.</b><br><br>
*        All applications will want to oversample at least 8 times to ensure random noise cannot
*        significantly affect the sensor's reading.<br>
*        <br>
*        Most applications oversample between 30-50 times.<br>
*        <br>
*        The diminishing returns on sensitivity when increasing this value occur at an exponential 
*        rate. This means there is only a small difference at 70 and 80 samples, while there is a 
*        very large difference between 10 and 20 samples. 
* 
* <br>
*
* @li <b>3. Change #MTOUCH_SCALING to 1 to eliminate resolution loss.</b><br><br>
*        When the acquisition module has oversampled #MTOUCH_SAMPLES_PER_SCAN number of times, the final
*        sum must be moved from a 24-bit array to a 16-bit array. The reading value cannot be allowed
*        to exceed the 16-bit limit of 65535. To prevent this from occuring, the framework will use
*        #MTOUCH_SCALING to determine what to divide the accumulation register by. This results in a
*        slight loss in resolution. If this is a concern for your application, setting #MTOUCH_SCALING value to
*        1 will eliminate this step.<br>
*        <br>
*        It is your responsibility to ensure the CVD's reading cannot exceed 65535.
* 
* <br>
*
* @li <b>4. Check that the average is not becoming corrupted.</b><br><br>
*        If your average is updating too quickly, you may be losing some sensitivity because the average
*        is no longer equal to the sensor's unpressed value. To check this, uncomment #MTOUCH_V1_COMM_OUTPUT_AVG
*        in the configuration file and view the average in real time with the reading.<br>
*        <br>
*        If the average is updating too quickly, you can slow it down by increasing the value of 
*        #MTOUCH_AVG_UPDATE and #MTOUCH_AVG_RATE.<br>
*        <br>
*        If the average is becoming corrupted due to fast tapping on the sensor, use the 
*        #MTOUCH_NEGATIVE_CAPACITANCE configuration option to adjust how the average behaves when the reading
*        has fallen below the average. For fast-tapping systems, setting this value to 1 is recommended.
*        Using the '2' value for this option is only recommended for systems that are not going to have
*        any noise spikes.
* 
* <br>
*
* @li <b>5. Decrease the amount of parasitic capacitance affecting the sensor and its trace.</b><br><br>
*        The amount of parasitic capacitance on the sensor will directly impact its maximum level of
*        sensitivity. Eliminate ground planes near your sensor and make sure its trace path is not 
*        running near any high-current or digital lines. This is particularly important for proximity
*        sensors and normal touch sensors with long traces.
*/
/**
* @page ts-ResponseTime Faster Response Time - Troubleshooting the Framework
*
* The response time of your system is going to be determined by the equation:
* @code
* Response Time = mTouch ISR Interrupt Rate * MTOUCH_NUMBER_SENSORS * MTOUCH_SAMPLES_PER_SCAN * MTOUCH_DEBOUNCE_PRESS
* @endcode
*
* <br>
*
* <b>Suggestions for Making the Response Time Faster:</b><br>
*
* <br>
*
* @li <b>1. Decrease #MTOUCH_SAMPLES_PER_SCAN to reduce the amount of oversampling.</b><br><br>
*        This configuration option should never drop below 10 in normal applications. The higher 
*        this value is, the more stable the sensor's readings will be coming out of the acquisition
*        ISR. However - the diminishing returns on this value follow an exponential curve. In other
*        words, the increase in signal fidelity when changing from 10 to 20 samples is about the
*        same as changing from 40 to 80.
*
* <br>
*
* @li <b>2. Reduce the TMR0 Prescaler value.</b><br><br>
*        This will increase the rate at which the timer's counter will increment. Do not set this
*        value to 1:1 or the interrupt will occur too quickly to allow processing in the main loop.
*        This value is determined by the application when it initializes the OPTION register. The
*        framework does not adjust this register.
*
* <br>
*
* @li <b>3. Increase the PIC's oscillator frequency.</b><br><br>
*        This will increase the rate at which the timer's counter will increment, but will also 
*        increase the amount of power needed by the PIC.
*
* <br>
*
* @li <b>4. Decrease #MTOUCH_NUMBER_SENSORS.</b><br><br>
*        The mTouch Framework scans all sensors completely before returning the result to be processed
*        by the main loop application. Reducing the number of sensors being scanned will reduce the
*        number of times the ISR must be called before new data is available.
*
* <br>
*
* @li <b>5. Reduce the amount of debouncing required to change sensor states.</b><br><br>
*        There are two debounce values available:<br>
*        #MTOUCH_DEBOUNCE_PRESS is for the RELEASE-to-PRESS transition.<br>
*        #MTOUCH_DEBOUNCE_RELEASE is for the PRESS-to-RELEASE transition.
*/
/**
* @page ts-Waveform Tuning the CVD Waveform - Troubleshooting the Framework
*
*
* Below is a picture of what the CVD waveform should look like on a scope. Note that sensor lines are
* used both for scanning themselves and the other sensors. You may notice the sensor being pulled high
* and low during other sensor's scans.
*
* @image html cvdWaveform.jpg
*
* The mTouch framework provides the ability to tune the timing of CVD waveform for specific hardware
* designs. The amount of parasitic capacitance, the size of the sensor, the size of the sensor's series
* resistor, and VDD all play a role in determining the amount of time required to charge/discharge
* during each step of the scan.
*
* @image html cvdWaveformDelay.jpg "CVD Waveform Delay Configuration Options"
*
* <br>
*
* @li <b>CVD_CHOLD_CHARGE_DELAY</b><br><br>
* This value determines the amount of time provided for charging the internal hold capacitor of the
* ADC. <br><br>
* If this value is <b>too small</b>: There will be noticable crosstalk between closely-indexed sensors
* that is not related to the hardware layout of the application.<br>
* If this value is <b>too large</b>: The mTouch ISR will take an unnecessarily long amount of time to 
* execute.<br><br>
* <b>To correctly set this value:</b>
* <ol>
* <li>Set this value to 0, compile and program.
* <li>Look at the raw values on the mTouch GUI or through a terminal program.
* <li>Produce the largest shift possible on the sensor with the highest amount of capacitance by pressing
* on it. ("Highest Capacitance" usually corresponds to the sensor with the highest unpressed raw value.)
* <ul>
* <li>Do you notice a strange crosstalk behavior on a different sensor? If no, leave this value to 0. If yes...
* </ul>
* <li>Increase the delay until the crosstalk behavior is eliminated. 
* </ol>
* <br>
* Once you've solved the crosstalk for the sensor with the highest amount of capacitance, the others will 
* be fine as well and you have correctly tuned the Chold delay time.
*
* <br>
*
* @li <b>CVD_SETTLING_DELAY</b><br><br>
* This value determines the amount of time provided for the external sensor and internal hold capacitor
* to charge-average their voltages.<br><br>
* If this value is <b>too small</b>: The sensors will not be as sensitive as they could be and your sensor's readings will be VDD dependant.<br>
* If this value is <b>too large</b>: The noise immunity of the system will not be as robust as it could be.<br><br>
* <b>To correctly set this value:</b>
* <ol>
* <li>Set this value to 0, compile and program.
* <li>Look at the raw values on the mTouch GUI or through a terminal program while powering the system at the desired VDD level.
* <li>Adjust VDD by plus and minus 0.5V 
* <ul>
* <li>Do you notice any change in the unpressed value as VDD is changing? If no, leave this value alone. If yes...
* </ul>
* <li>Increase the delay until the sensor's reading no longer changes as VDD changes.
* </ol>
* <br>
* Once this has occurred, the settling time has been correctly tuned to provide the maximum amount of sensitivity
* while minimizing the framework's susceptibility to noise.
*
*/
/**
* @page ts-Comms PC Communications - Troubleshooting the Framework
*
* <b>Step-by-Step Guide to Troubleshooting the Framework's PC Communications</b><br>
*
* <br>
*
* @li <b>1. Make sure #MTOUCH_V1_COMMS is defined in the mTouch_config.h file.</b>
* @code
* #define MTOUCH_V1_COMMS
* @endcode
*
* <br>
*
* @li <b>2. Program the board normally (not in debug mode) and connect the PKSA to both the PC and the board.</b><br>
*
* <br>
*
* @li <b>3. Make sure both the PIC and the PKSA are being supplied with power.</b><br><br>
*       Either:
*       <ul>
*       <li>Supply power to the PIC from an external source that is also supplying power to the PKSA's PWR and GND pins.
*       <li>Supply power to the PIC from the PKSA by pressing and holding the black button for 3 seconds until the green power LED turns on.
*       <li>Supply power to the PIC from an external source that is NOT connected to the PWR/GND pins of the PKSA and then supply power to the PKSA by pressing on its black button for 3 seconds until the green power LED turns on.
*       </ul>
*       Be careful not to supply power to the PIC from two different sources as you may destroy the board.
*
* <br>
*
* @li <b>4. Is the PKSA's red LED on?</b><br><br>
*       If no -
*       <ul>
*       <li> Use a scope to verify the TX pin of the PKSA is receiving data from the PIC.
*       <li> If it is, the PKSA's firmware has not been updated to perform as a UART-to-USB converter. 
*            See @ref PKSARS232 "this guide" on using the PKSA loader utility to reflash the PICKit 
*            Serial Analyzer's firmware.
*       </ul>
*       If yes -
*
* <br>
*
* @li <b>5. Determine which COM port has been assigned to the PKSA.</b><br><br>
*       In Windows - 
*       <ul>
*       <li> Right click 'My Computer' --> Properties --> Hardware --> Device Manager
*       <li> You should find it listed under "Ports (COM)".
*       </ul>
*       If it's not there, you need to reinstall the Windows driver for the UART-to-USB behavior. The
*       driver can be found in the PKSA Loader Utility's folder. Correct installation is based on the
*       version of Windows you are using, but can be accomplished by right clicking the file and selecting
*       'Install' in most cases.
*
* <br>
*
* @li <b>5. Open the mTouch GUI and make sure the communication settings are set to:</b><br><br>
*       <ul>
*       <li> 115.2 kbps
*       <li> COM Port dependant on current assignment in Device Manager.
*       </ul>
*/
/**
* @page digAcq Acquisition Method and Implementation
*
* The acquisition routine of the mTouch Framework is implemented using a library of macro functions
* that implement the process in assembly. This ensures we achieve an efficient, time-specific algorithm 
* that can be both customizable and robust to external noise. This code is not intended to be touched by
* the end user. Any edits to this code could result in the loss of noise immunity.
*
* As shown below, the acquisition routine performs two ADC scans. The difference between these two scans
* is then used to update a decimate-filtered result register. This register is only allowed to update
* by one count in either the up or down direction. So, for example, you must update this register 100
* times to move its result 100 counts. This is done to remove high-frequency impulse noise from the
* signal.
*
* The decimate-filtered result is then thrown into an accumulator so the shifts can be oversampled. Once
* it has oversampled #MTOUCH_SAMPLES_PER_SCAN number of times, the value of the accumulator is scaled so it
* will fit in a 16-bit integer and then stored into one final, latched result array that can be accessed
* using the mTouch_GetSensor(i) macro.
*
* @warning You should never try to access the arrays in the mTouch_Acquisition.c file. If you do,
* however, you will find the index values are shifted from what you would expect. Sensor0's data is
* stored in Sensor1's array locations. SensorN is stored in Sensor0's array location. This index shifting
* is corrected in the storage macro and so it is invisible to anything outside of the mTouch ISR.
*
* For more information about the acquistion module, check out these help sections:
* @li ISR() : Follows the code and clarifies each step of the mTouch ISR
* @li READ_SENSOR_EXPANDED() : Step-by-step explanation of the CVD scanning routine
*
* @image html acquisitionBlock.jpg
*/
/**
* @page digFil Filtering and Decoding Implementation
*
* The filtering and decoding routine of the mTouch Framework executes all of the detection algorithms.
* Using the latched output from the acquisition module, the mTouch_Service() function is called each
* time a new value is available for each sensor (known by checking the mTouch_isDataReady() function).
* It will then loop through all of the active sensors and determine if they have changed state. If either
* the #MTOUCH_V1_COMMS or #MTOUCH_V2_COMMS configuration option is defined, the algorithm will also 
* transmit the latest set of data. 
* 
* The decode routine implements a baseline average that is only updated while the sensor is in the 
* #MTOUCH_RELEASED state. The difference between the sensor's average and its reading is stored in a variable
* called @c delta and is compared against the #THRESHOLD_PRESS_SENSOR0 values (or the equivalent release
* threshold values) to determine if a change in state is necessary.
*
* While pressed, the system can also begin a counter to automatically release the sensor after a given
* period of time. This can be implemented using the #MTOUCH_BUTTON_TIMEOUT configuration option.
*
* In some systems when the user releases a sensor other sensors can see a slight negative shift in value.
* This is the opposite direction of the 'press' shift, so we have included some special options to define
* how the average updates in these situations. There are also options to change the normal averaging update
* rate to customize the feel of your sensors. For more information, see the mTouch_UpdateAverage()
* function.
*
* For more information about the filtering and decoding module, check out these help sections:
* @li mTouch_Decode() : Follows the code and clarifies each step of the decoding routine
* @li @ref cusService "Implementing a Custom Service() Function" : Guide to replacing the default mTouch 
*                    service routine with your own
*/
/**
* @page digCom Communication Implementation
*
* The communication module of the mTouch Framework has been designed to talk to the included mTouch
* GUI by default but can easily be adapted to meet your needs. The framework handles all initialization
* requirements once it knows the PIC's oscillator speed and desired baud rate. 
*
* The data packets being sent to the GUI are as shown:
* @code
* 16bitMaskofSensorStates;Sensor0Reading;Sensor1Reading;...;SensorNReading(CR)(LF)
* @endcode
* which looks like this when populated:
* @code
* 65525;02301;02354;02332;02296;02318
* @endcode
* @note A @c 1 in the bit mask means the sensor is not pressed (either initializing or released). A @c 0 
* means that it is pressed.
*
* The data is sent in decimal format with a maximum of 5 digits by default.
*/
/**
* @page cusService Implementing a Custom Service() Function
*
* Some users may wish to create their own service function to interpret the sensor's readings.
* The easiest way to handle this is to copy the mTouch.c file from the library to your local 
* project directory. Change the mTouch.c file location in your project and make all edits
* to the local version.
*
* @section cusService1 * Replacing the Default mTouch Service Call
*
* The mTouch_Decode() function is called once each sensor has been scanned. New reading values
* for each sensor can be accessed with the mTouch_GetSensor(i) macro, where i is the Sensor# as defined
* in the configuration file.
*
* @section cusService2 * Replacing the default mTouch Initialization Function
*
* The majority of the default mTouch initialization function is dedicated to preparing
* the decode and filtering module. Since you are creating your own service function,
* this initialization function should be replaced with one that initializes your own
* variables.<br> 
* To initalize the mTouch framework, make sure to include the following code blocks.
* function:
* @code
* for (unsigned char i = 0; i < MTOUCH_NUMBER_SENSORS; i++)
* {
*   sensor_data[i] = 0x47F; // Initializes the acquisition module's sensor_data array to minimize initalization time
* }
* @endcode
* @code
* CVD_SET_ADC_CLK();        // Automatically configures the ADC's clock based on your configuration file's _XTAL_FREQ value
*                           // Alternatively, you can initialize the ADC registers yourself
* @endcode
* @code
* #if defined(MTOUCH_V1_COMMS)
* mTouch_Comm_Init();    // OPTIONAL - Call this if you want to use the default mTouch comm implementation
* #endif
* @endcode
* @code
* CVD_SET_TMR0IE();         // REQUIRED
* CVD_SET_GIE();            // REQUIRED
* @endcode
*/
/**
* @page PKSARS232 Using the PICKit Serial Analyzer as an RS232-to-USB converter
*
* @section Intro Introduction
*   The PICKit Serial Analyzer (PKSA) is an easy way to connect the UART of your PIC&reg; microcontroller to your PC. 
*   All of the required steps are handled by the PICKit Serial Loader Utility provided with the framework.
*
*   <b>Maximum Speed</b>: 115.2 kbps<br>
*   <b>Voltage Level</b>: Depends on the voltage applied to pin #2. Valid range is from 3-5V.<br>
*   Optional 5V (<20mA) power supply from the PICKit Serial Analyzer.<br>
*
* @section Convert Changing Your PICKit Serial Analyzer into an RS-232 Converter
*   <ol>
*   <li>Press the button on the PICKit Serial Analyzer when connecting to PC's USB to enter @c Bootloader @c Mode.<br>
*     (The 'Target' and 'Busy' LEDs should be flashing quickly.)
*   <li>Open the PICKit Serial Loader Utility provided by the framework
*   <li>Follow the on-screen instructions
*   </ol>
*
* @section Use Using your RS-232 Converter
*   <ol>
*   <li>Unplug and replug the PKSA. Press and hold the black PKSA button to toggle the on-board 5V power supply.
*       <ul><li>Warning: There is no protection provided on this power supply. Maximum current is 20mA.</ul>
*   <li>LED Status Legend:
*       <ul>
*       <li>Target LED On: Transmission in progress
*       <li>Busy LED On: Reception was successful
*       </ul>
*   </ol>
*
* @image html pksaExtVdd.JPG "Example schematic connecting the PKSA with an external power supply"
* @image html pksaVdd.JPG "Example schematic of PKSA supplying power to the board"
* @image html pksaExt5Vdd.JPG "Example schematic of connecting PKSA with an external 5V power supply. Notice - no V<sub>DD</sub> connection required."
*
* @section Pinout PICKit Serial Analyzer Pin Configuration
*   <ol>
*   <li>The PC's TX line. The PIC's RX line. (Marked with an arrow)
*   <li>V<sub>DD</sub> : 3-5V
*   <li>V<sub>SS</sub>
*   <li>Aux1 : not used in RS232 configuration
*   <li>Aux2 : not used in RS232 configuration
*   <li>The PC's RX line. The PIC's TX line.
*   </ol>
* @image html pksaPins.JPG
*/
/**
* @page GettingStartedEval mTouch CVD Evaluation Kit Quick Start Guide 
*
* @note This guide assumes you are using the mTouch Evaluation kit as your hardware. For 
* details on where to begin when implementing the mTouch framework on a custom hardware
* design, use @ref GettingStartedCustom "this guide".
*
* Congratulations for the purchase of your new mTouch CVD Evaluation Kit.
*
* Although the kit will run right out of the box, this document will walk you through
* the setup process to quickly configure the kit. Should you need any custom configuration,
* this will get you up and running.
*
* The kit comes pre-programmed and set to operate in the configuration listed below. If
* you need to alter any of the parameters, then a new hex file will need to be generated
* and the main board will need to be reprogrammed with the new firmware. MPLAB IDE will
* then be required as well as a C compiler.
*
* @section GSE_DSC Default Software Configuration:
* @li Target MCU: <b>PIC16F1937</b>. Possible Values: @ref PICSupport "See here" for supported PIC microcontrollers.
* @li #MTOUCH_NUMBER_SENSORS: <b>7</b>. Possible Values: 1-15
* @li Sensor/Pin Assignment:
* @code
*   #define MTOUCH_SENSOR0             AN12  
*   #define MTOUCH_SENSOR1             AN10    
*   #define MTOUCH_SENSOR2             AN8     
*   #define MTOUCH_SENSOR3             AN9     
*   #define MTOUCH_SENSOR4             AN11     
*   #define MTOUCH_SENSOR5             AN13     
*   #define MTOUCH_SENSOR6             AN4
* @endcode
* @li #MTOUCH_SAMPLES_PER_SCAN: <b>50</b>. Valid range: 1-65535
* @li Press Threshold Values:
* @code
*   #define THRESHOLD_PRESS_SENSOR0         1245
*   #define THRESHOLD_PRESS_SENSOR1         1566
*   #define THRESHOLD_PRESS_SENSOR2         2021
*   #define THRESHOLD_PRESS_SENSOR3         1688
*   #define THRESHOLD_PRESS_SENSOR4         2154
*   #define THRESHOLD_PRESS_SENSOR5         1466
*   #define THRESHOLD_PRESS_SENSOR6         1783
* @endcode
* 
* @section GSE_CNS Changing the Number of Sensors:
* If you need the stack to manage more or less than the default number, all you need to 
* do is update the constant #MTOUCH_NUMBER_SENSORS to any other value from 1 to 15. This
* constant can be found in the mTouch_config.h file.
*
* @section GSE_ASI Assigning Sensor Indexes to the Correct A/D Channels
* Depending on how the daughter board is connected to the main board, the sensors may
* be connected to different A/D inputs on the PIC microcontroller. In the mTouch
* framework, sensors can be mapped to a specific index in the application. Go to
* mTouch_config.h and look for the section with the definitions shown below, and
* map the sensor indexes on the left side to the correct A/D channels for your setup.
* @code
*   #define MTOUCH_SENSOR0             AN0  
*   #define MTOUCH_SENSOR1             AN1    
*   #define MTOUCH_SENSOR2             AN2     
*   #define MTOUCH_SENSOR3             AN3     
*   #define MTOUCH_SENSOR4             AN4     
*   #define MTOUCH_SENSOR5             AN5     
*   #define MTOUCH_SENSOR6             AN6     
*   #define MTOUCH_SENSOR7             AN7     
*   #define MTOUCH_SENSOR8             AN8     
*   #define MTOUCH_SENSOR9             AN9     
*   #define MTOUCH_SENSOR10            AN10    
*   #define MTOUCH_SENSOR11            AN11    
*   #define MTOUCH_SENSOR12            AN12    
*   #define MTOUCH_SENSOR13            AN13
* @endcode
*
* @section GSE_CAC Changing the Number of Samples per Scan
* The number of samples taken in an application will be based on a trade-off between
* sensitivity and response time. The more samples taken, the better the signal-to-noise
* ratio of the system, but the slower its response to a finger. Typical response times
* are less than 100ms to eliminate delay that is visible to the human eye. If a specific
* response time is required, example equations are provided in the configuration file
* to show how to calculate the rate that will be chosen based on this value.
*
* @section GSE_AST Adjusting the Sensor Threshold Values
* Your mTouch evaluation kit comes already preconfigured with the sensor threshold
* values for the 8-button daughter board. If different sensitivities are needed, go
* to the mTouch_config.h file, locate the section with the definitions shown
* below, and set the new values.
* A general rule of thumb to finding and setting correct values is to determine the
* range of shift for the sensor (min and max shift) and set the threshold to about
* 75% of the range. The following formula can be used: 
* @f[ Threshold_{Press} = min + 0.75 (max - min) @f]
* @code
*   #define THRESHOLD_PRESS_SENSOR0         65         
*   #define THRESHOLD_PRESS_SENSOR1         65
*   #define THRESHOLD_PRESS_SENSOR2         65
*   #define THRESHOLD_PRESS_SENSOR3         65
*   #define THRESHOLD_PRESS_SENSOR4         65
*   #define THRESHOLD_PRESS_SENSOR5         65
*   #define THRESHOLD_PRESS_SENSOR6         65
* @endcode
*
*/
/**
* @page GettingStartedCustom Custom Hardware Quick Start Guide
*
* @note This guide assumes you are using custom hardware. For details on where to begin 
* when implementing the mTouch framework on the mTouch CVD Evaluation Kit, use 
* @ref GettingStartedEval "this guide".
*
* @section GS_SysConfig System Configuration
* The mTouch_Init() function will automatically configure the ADC, communications module (if enabled),
* and all required interrupts. There are only a few things the user is required to define on power-up:
* <ol>
* <li>Configure the oscillator to provide a fast clock speed.
* @code
* OSCCON = 0b01110000;       // 32 MHz Fosc w/ PLLEN_ON config word on PIC16F1937
* @endcode
* <br>
* <li>  Set the prescaler for the timer you are using for the mTouch scan interrupt. (Weak pull-up 
*       resistors should be disabled.)<br>
*       For more information about the implications of this setting, see @ref ResourceRequirements.
* @code
* OPTION_REG = 0b10000000;   // TMR0 Prescaler = 1:2 (~40% processor usage)
* @endcode
* <br>
* <li>Sensor pins should be initialized as digital output low.
* @code
* ANSELA = 0b00000000;       // Digital
* TRISA  = 0b00000000;       // Output
* PORTA  = 0b00000000;       // Low   
* @endcode
* </ol>
* <br>
* @section GS_InitConfig Framework Configuration
* Open mTouch_config.h and make the following edits:
* <ol>
* <li>Set the #_XTAL_FREQ value with the PIC&reg; Microcontroller's Fosc speed in Hz
* @code
* #define _XTAL_FREQ         32000000
* @endcode
* <br>
* <li>Set the #MTOUCH_ISR_TIMER value to the chosen timer.
* @code
* #define MTOUCH_ISR_TIMER   0
* @endcode
* <br>
* <li>Set the #MTOUCH_NUMBER_SENSORS value to the number of capacitive sensors in your design
* @code
* #define MTOUCH_NUMBER_SENSORS 5
* @endcode
* <br>
* <li>Assign an analog channel to each sensor using the #MTOUCH_SENSOR0 definitions.
* @code
* #define MTOUCH_SENSOR0        AN0 
* #define MTOUCH_SENSOR1        AN1    
* #define MTOUCH_SENSOR2        AN2    
* #define MTOUCH_SENSOR3        AN3     
* #define MTOUCH_SENSOR4        AN4   
* @endcode
* (Extra definitions - such as @c MTOUCH_SENSOR5, in this case - will simply be ignored.)
* </ol>
* <br>
* @section GS_API Application Main Loop API Hooks
* In your application's main loop:
* <ol>
* <li>Use the mTouch_isDataReady() function to determine when to service new mTouch readings.
* <li>Once it's set, call mTouch_Service(). The #mTouch_dataReady state is reset automatically.
* @code
* while(1)
* {
*   if (mTouch_isDataReady())   // Is new information ready?
*   {
*      mTouch_Service();        // Decode the newly captured data and transmit new data updates.
*   }
* }
* @endcode
* <br>
* <li>Use mTouch_GetButtonState(i) to perform actions based on the sensor's state. (<b>See also</b>: #mTouch_ButtonState)
* @code
* #define LED0      LATA0
*
* #define LED_ON    0
* #define LED_OFF   1
*
* if (mTouch_GetButtonState(0) < MTOUCH_PRESSED)
* { 
*   LED0 = LED_OFF;                // Sensor is either initializing or released.
* } else { 
*   LED0 = LED_ON;                 // Sensor is pressed.
* } 
* @endcode
* </ol>
* Your application should now compile and function. If you have enabled communications, you can see
* the real-time sensor readings using the mTouch Framework GUI. For more information about adjusting
* the thresholds on sensors and further customizations, refer to the other @ref GettingStarted guides.
*/
/**
* @page GettingStartedNewP8 Creating a new MPLAB 8 mTouch Framework Project
*
* <ol>
* <li>  Copy the mTouch_config.h file out of the framework's /includes directory and place it in a folder named 
*       'includes' in the local application's directory.<br><br>
* <li>  Open MPLAB 8 and close any currently active projects.<br><br>
* <li>  Navigate to: Project --> Project Wizard... --> Next <br><br>
* <li>  Select the PIC your application will be using and click 'Next'.<br><br>
* <li>  Select the appropriate HI-TECH C compiler and click 'Next'.<br><br>
* <li>  Select the desired location for your project and click 'Next'.<br>
*       HINT: Place your project in a sub-folder to keep the compiler-generated files from cluttering the project 
*       source directory.<br><br>
* <li>  In the left window, navigate to the mTouch library's source directory and add all the '.c' files to your 
*       project.<br><br>
* <li>  In the left window, navigate to your local application directory and add all source files to your project. 
*       Click 'next'.<br><br>
* <li>  Click 'Finish', wait for the project to be created, then navigate to Project --> Build Options... --> 
*       Project <br><br>
* <li>  Under the 'Directories' tab, select 'Include Search Path' in the drop down box.<br><br>
* <li>  Click 'New' and then the '...' button that appears in the list window.<br>
*       Navigate to and select the framework's root source directory 'PIC16F CVD FW Library'.<br><br>
* <li>  Click 'New' and type '..' as the directory name.<br>
*       NOTE: Make sure this entry is located above the framework directory. If not, use the 'up' button to 
*       reorder them.<br>
* <li>  Use the following line of code to include the mTouch framework in your project:<br>
* @code
* #include "mTouch.h"
* @endcode
* </ol>
*
* Now check out the main.c file located in the framework's library folder or one of the main.c files in
* the mTouchCapDemos folder for suggestions on using the mTouch framework's API.
*/
/**
* @page GettingStartedNewPX Creating a new MPLAB X mTouch Framework Project
*
* <ol>
* <li>  Copy the mTouch_config.h file out of the framework's /includes directory and place it in a folder named 
*       'includes' in the local application's directory.<br><br>
* <li>  Open MPLAB X and close any currently active projects.<br><br>
* <li>  Navigate to: File --> New Project... --> Microchip Embedded --> Standalone Project --> Next <br><br>
* <li>  Select the PIC your application will be using and click 'Next'.<br><br>
* <li>  Select the tool you will be using to program or debug this application and click 'Next'.<br><br>
* <li>  Select the appropriate HI-TECH C compiler and click 'Next'.<br><br>
* <li>  Select the desired name and location for your project and click 'Finish'.<br><br>
* <li>  In the Projects window, right click your project's Source Files folder and select 'New Logical Folder' 
*       and name it 'mTouch Framework Source'<br>
*       Right click the new folder, select 'Add Existing Item...', navigate to the main framework source 
*       directory and select the files:
*       <ul>
*       <li> mTouch.c
*       <li> mTouch_Acquistion.c
*       <li> mTouch_Comm.c
*       </ul><br>
* <li>  In the Projects window, right click your project's Source Files folder, select 'Add Existing Item...', 
*       navigate to your local application directory and add all source files to your project.<br><br>
* <li>  In the Projects window, right click your project's Header Files folder, select 'Add Existing Item...', 
*       navigate to your local application directory and add all header files to your project - including the 
*       two mTouch Framework header files in the /includes directory of your application.<br><br>
* <li>  Navigate to File --> Project Properties --> Compiler<br>
*       Select 'Preprocessor' from the Option Categories drop down box.<br>
*       Click the '...' button next to "Include Directories".<br><br>
* <li>  Click 'Create' and then 'Browse...'.<br>
*       Navigate to and select the framework's root source directory 'PIC16F CVD FW Library'.<br><br>
* <li>  Click 'Create' and type '..' as the directory name.<br>
*       NOTE: Make sure this entry is located above the framework directory. If not, use the 'up' button to 
*       reorder them.<br>
* <li>  Use the following line of code to include the mTouch framework in your project:<br>
* @code
* #include "mTouch.h"
* @endcode
* </ol>
*
* Now check out the main_example.c file located in the framework's library folder or one of the main.c files in
* the mTouchCapDemos folder for suggestions on using the mTouch framework's API.
*/
/**
* @page RequiredIncludes Required Hardware Include Files
*
* The mTouch Framework will automatically choose the correct header files for your chosen device.
* You only need to add the following files to your project's main folder:
*   @li C:/YourApplicationSourceDirectory/includes/mTouch.h
*   @li C:/YourApplicationSourceDirectory/includes/mTouch_config.h
*   @li C:/YourApplicationSourceDirectory/includes/mTouch_config_slider.h
*   @li C:/YourApplicationSourceDirectory/includes/mTouch_config_sleep.h
*   @li C:/YourApplicationSourceDirectory/includes/mTouch_config_cvdAdvanced.h
*
* To set up your project to look for the header files in the right location, your compiler's 
* <b>include search path</b> should contain two entries:
*   @li C:/YourApplicationSourceDirectory/  :: (The '/includes' directory should reside inside this one.)
*   @li C:/mTouchFrameworkSourceDirectory/  :: (The '/includes' directory should reside inside this one.)
*
* @section RI_HardwareProfile Hardware Profile
* Each framework-supported PIC microcontroller has an associated HardwareProfile header file
* to define the relationship between the AN channel and its port/pin assignment, how to initialize
* the communications module, and which communications methods are available. This file will
* also perform some error checking on the current configuration to determine if your microcontroller
* will be able to correctly implement the requested framework features.
*
* <i>Copy the hardware profile that is associated with your microcontroller's family.</i>
*
* Hardware Profile Options:
* @li includes/mTouch_hardwareProfile_12F61x.h  :: PIC12F615, PIC12F617, PIC12HV615
* @li includes/mTouch_hardwareProfile_16F151x.h :: PIC16F1516, PIC16F1517, PIC16F1518, PIC16F1519
* @li includes/mTouch_hardwareProfile_16F152x.h :: PIC16F1526, PIC16F1527
* @li includes/mTouch_hardwareProfile_16F182x.h :: PIC12F1822, PIC16F1823, PIC16F1824, PIC16F1825, PIC16F1826, PIC16F1827, PIC16F1828, PIC16F1829
* @li includes/mTouch_hardwareProfile_16F193x.h :: PIC16F1933, PIC16F1934, PIC16F1936, PIC16F1937, PIC16F1938, PIC16F1939
* @li includes/mTouch_hardwareProfile_16F194x.h :: PIC16F1946, PIC16F1947
* @li (and others... see Files list or Resource Requirements for full details)
*
* @section RI_MacroLibrary Macro Library
* The mTouch acquisition function is implemented using a combination of C and assembly. To support 
* multiple cores with different assembly instruction sets, separate macro definitions are required.
* These macros perform the mTouch scanning routine and initial signal processing on the results. Due
* to noise concerns, we do not recommend editting these macros.
*
* <i>Copy the macro library that is associated with your microcontroller's instruction set + mTouch_macroLibrary_common.h</i>
* 
* Macro Library Options:
* @li mTouchCVD_macroLibrary_PIC16F.h :: PIC12/16 Non-Enhanced Core, Midrange Devices
* @li mTouchCVD_macroLibrary_PIC16F1.h :: PIC16F1 Enhanced Core, Midrange Devices
* @li mTouchCVD_macroLibrary_PIC18F.h :: PIC18 Devices
*
* @section RI_processConfig Internal Framework Helper Files
* These files are used in all cases to prepare the framework for compiling. If you want to make edits
* to them, you should copy these file as well.
*
* @li mTouch_processConfiguration.h
* @li mTouch_optionsForConfig.h
* @li mTouch_comm.h
* @li mTouch_eeprom.h
* @li mTouch_slider.h
*/
/**
* @page SoftwareLicense Software License Agreement
*
* IMPORTANT - READ CAREFULLY.  This  End User License Agreement ("EULA") is a legal agreement between you (either an individual or a single entity)
* and Microchip Technology Incorporated and its subsidiary Microchip Technology (Barbados) Incorporated (in the aggregate referred to as "Microchip")
* for Microchip's mTouchTM Software distributed in connection with this EULA, including but not limited to any source code, object code, and HEX
* files included in the media (hereinafter the "Software") and accompanying documentation (hereinafter the "Documentation").  The Software and
* Documentation provided herewith are intended, and supplied to you, for use solely and exclusively in a system that incorporates Microchip
* microcontroller or Microchip digital signal controller products and for no other use or purpose whatsoever.  Microchip will license the Software and
* Documentation to you only if you first accept the terms of this agreement.  By indicating acceptance below, or by otherwise installing, copying or
* using the Software or Documentation, you are agreeing to be bound by the terms of this agreement.  If you do not agree to the terms of this agreement,
* so indicate below and promptly destroy any copies of the Software on your system and return the unused Software for a full refund of moneys paid for
* this Software, if any, if you received it by physical media.
*
* THE SOFTWARE AND DOCUMENTATION MAY BE ACCESSED ONLY IF YOU ACCEPT THE TERMS AND CONDITIONS OF THIS LICENSE AGREEMENT.  PLEASE INDICATE YOUR ACCEPTANCE
* OR NON-ACCEPTANCE BY SELECTING "I ACCEPT" OR "I DO NOT ACCEPT" AS INDICATED BELOW.
*
* The Software and Documentation are protected by U.S. copyright laws and international copyright treaties, as well as other intellectual property laws
* and treaties.  
*
* <ol>
* <li><b>Ownership and Title.</b><br><BR>
* All right, title and interest, including intellectual property rights, in and to the Software and the Documentation shall remain in Microchip. The
* Software and Documentation are licensed pursuant to this agreement, not sold.  Microchip owns all proprietary rights to the mTouchTM sensing solution
* technology, including, but not limited to, sensing, decision making and responsive action as implemented in mTouchTM technology including all hardware
* and software implementations of the technology.  Microchip reserves all rights not expressly granted to you.<br><br>
*
* <li><b>Grant of Limited License and Restrictions.</b><br>
*
* Subject to all of the terms and conditions of this agreement, Microchip grants to you a worldwide, fully paid-up, non-exclusive license to reproduce,
* have reproduced, market, sell, distribute and use the object code or HEX files in the Software, but not any source code files contained in the Software,
* for the sole and limited purpose of integrating mTouchTM technology into your own "conforming systems" that you will distribute.  Conforming systems are
* those systems, which incorporate Microchip microcontroller or Microchip digital signal controller products in the same system.   No license is hereby
* granted to the Software for any purpose other than integrating the mTouchTM technology into a conforming system and no license is granted to distribute
* the software in a form other than as part of a conforming system.<br>
*
* Subject to all of the terms and conditions of this agreement, Microchip further hereby grants to you a world-wide, fully paid-up, non-exclusive license to
* use the Documentation provided with the Software internally only. The Documentation may not be reproduced, displayed, distributed, or disclosed to any third party.<br>
*
* You may enter into agreements or arrangements with individuals or entities other than Microchip for the reproduction of the Software for the sole purpose
* of incorporating the mTouchTM technology into your conforming system; provided, however, that you, prior to any negotiations or discussion with individuals
* other than Microchip with respect to the reproduction of Software, first obtain an executed non- disclosure/confidentiality agreement with those individuals
* or entities in a form and content approved in advance by Microchip that requires all reproductions be incorporated into your conforming system. All copies of
* the Software created by you or for you must include at least the Microchip's copyright, trademark and other proprietary notices.<br>
*
* You may not remove or alter any Microchip copyright, trademark, or other proprietary rights notice contained in any portion of the Software or Documentation
* that bear such a notice or in any reproductions thereof.  You may not remove or alter any identifying screen that is produced by the Software.<br>
*
* You may modify, adapt and prepare derivative works of the Software only as necessary to incorporate the mTouchTM technology into your conforming system, and
* may distribute any such derivative works as part of your conforming system without additional license or fees in object code or HEX files only, subject to all
* of the terms and conditions of this agreement.  You may not prepare derivative works of the Documentation.<br>
*
* This license is not transferable or sub-licensable.  Neither this agreement nor any rights, licenses or obligations hereunder, may be assigned by you without
* the prior written approval of Microchip.<br>
*
* This license does not allow any sublicense, distribution, display, or disclosure of the Software source code or Documentation or any other proprietary rights
* to any third party in any circumstance or manner, and you agree that you will not engage in any such sublicensing, disclosure or distribution, without the prior
* written consent of Microchip.<br>
*
* You may use the Software only with respect to the manufacture and sale of your own conforming system.  You have no right to sell, assign, lend, rent, lease or
* otherwise transfer all or any portion of the Software or the proprietary rights for the Software except as expressly set forth herein, and this agreement grants
* no additional manufacturing, sales or marketing rights.<br>
*
* You may not reverse engineer (by disassembly, decompilation or otherwise) the Software and may not copy or reproduce all or any portion of the Software, except
* and only to the extent that such activity is specifically allowed by this agreement or expressly permitted by applicable law notwithstanding the foregoing limitations.
* You may not copy or reproduce all or any portion of the Documentation without the express written consent of Microchip.<br><br>
*
* <li><b>Confidentiality.</b><br>
*
* You agree not to disclose the Software or Documentation, in whole or in part, or any proprietary information, including but not limited to any idea, concept, know-how,
* or technology, contained in or embodied by the Software or Documentation (collectively, "Confidential Information") to any third party.  Confidential Information shall
* not include any information which is in or becomes part of the public domain or which is independently developed or which is independently obtained free from any obligation
* of confidentiality.<br><br>
*
* <li><b>Termination of Agreement.</b><br>
*
* This EULA is effective as of the date of your acceptance and shall remain in effect unless terminated by Microchip pursuant to the provisions of this section.  Without
* prejudice to any other rights, Microchip may terminate this agreement if you fail to comply with the terms and conditions of this agreement.  Upon termination, you will
* immediately destroy all copies of the Software and will return all Documentation.  You will have a reasonable period after the termination of this Agreement to sell the
* existing inventory of your conforming systems.<br><br>
*
* <li><b>Indemnity.</b><br>
*
* You will indemnify and hold Microchip, its related companies and its suppliers harmless for, from and against, any claims or liabilities, including without limitation product
* liability claims, arising out of the use, reproduction or distribution of your products that incorporate the Software, or any unauthorized use or sale by you of your product
* or any unauthorized use or disclosure of any information regarding the Software, the Documentation or any other proprietary rights of Microchip.<br><br> 
*
* <li><b>Limited Warranty and Limited Liability.</b><br>
*
* The only warranty made by Microchip is that the original media in which the Software is embodied and which are distributed by Microchip shall be free of defects in materials
* and workmanship for a period of 90 days after delivery to you.  Microchip's and its suppliers' entire liability and your exclusive remedy shall be limited to the replacement
* of the original media, if defective, within a reasonable period of time.<br> 
*
* EXCEPT AS SPECIFICALLY PROVIDED IN THE PREVIOUS TWO SENTENCES, THE SOFTWARE AND DOCUMENTATION IS PROVIDED "AS IS" WITHOUT  WARRANTY OF ANY KIND, EITHER EXPRESS OR  IMPLIED,
* INCLUDING WITHOUT LIMITATION, ANY  WARRANTY OF MERCHANTABILITY, WARRANTY OF FITNESS FOR A PARTICULAR PURPOSE, AND WARRANTY OF NON-INFRINGEMENT OF ANY INTELLECTUAL PROPERTY
* RIGHT OF A THIRD PARTY.<br>
*
* THE ENTIRE RISK ARISING OUT OF USE OR PERFORMANCE OF THE SOFTWARE AND OTHER DOCUMENTATION PROVIDED UNDER THIS AGREEMENT IS ASSUMED BY YOU, AND MICROCHIP DISCLAIMS ALL
* RESPONSIBILITY FOR THE ACCURACY OR APPLICATION OR OF ERRORS OR OMISSIONS IN THE SOFTWARE AND DOES NOT WARRANT THE FUNCTIONS CONTAINED IN THE SOFTWARE WILL MEET YOUR REQUIREMENTS
* OR THAT THE OPERATION OF THE SOFTWARE WILL BE UNINTERRUPTED OR ERROR-FREE, OR THAT ANY DEFECTS IN THE SOFTWARE WILL BE CORRECTED.  IN NO EVENT SHALL MICROCHIP BE LIABLE FOR ANY
* DIRECT, INCIDENTAL, SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF SUBSTITUTE GOODS, TECHNOLOGY OR SERVICES, ANY CLAIMS BY THIRD
* PARTIES (INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION, OR OTHER SIMILAR COSTS.<br><br>
*
* <b><li>Patent and Copyright Infringement.<br>
*
* YOU ACKNOWLEDGE THAT<br>
* <ul>
* <li>(a) mTouchTM SOFTWARE, EMBEDDED TECHNOLOGY, OR SERVICES RELATED THERETO ( HEREAFTER "MTOUCH") PERTAIN TO CAPACITIVE SENSING FUNCTIONS,<br>
*
* <li>(b) THE DESIGN OF ALL ASPECTS OF ANY PRODUCT(S) OR SYSTEM(S) EMPLOYING MTOUCH IS SOLELY YOUR RESPONSIBILITY, AND<br>
*
* <li>(c) IT IS SOLELY YOUR RESPONSIBILITY TO DETERMINE WHETHER YOUR PRODUCTS OR SYSTEMS EMPLOYING MTOUCH INFRINGE A PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS OF THIRD PARTIES.
*      YOU AGREE THAT MICROCHIP SHALL HAVE NO OBLIGATION TO INDEMNIFY OR DEFEND YOU IN THE EVENT THAT A THIRD PARTY ASSERTS OR PROVES A CLAIM THAT ONE OR MORE OF YOUR SYSTEMS OR
*      PRODUCTS EMPLOYING MTOUCH INFRINGE A THIRD PARTY'S PATENTS, COPYRIGHTS, OR OTHER PROPRIETARY RIGHTS (HEREINAFTER A "CLAIM").  YOU HEREBY WAIVE ANY RIGHT TO CAUSE MICROCHIP
*      TO DEFEND OR INDEMNIFY YOU OR ANY OF YOUR CUSTOMERS IN CONNECTION WITH A CLAIM, INCLUDING BUT NOT LIMITED TO ANY SUCH RIGHT AS MAY BE IMPOSED OR IMPLIED BY LAW, STATUTE,
*      OR COMMON LAW.<br>
*
*  IF A SUIT OR PROCEEDING IS BROUGHT AGAINST MICROCHIP BASED ON A CLAIM THAT MTOUCH SUPPLIED BY MICROCHIP TO YOU CONSTITUTES INFRINGEMENT OF A PATENT, COPYRIGHT, DATA BASE RIGHT,
*  TRADEMARK OR OTHER INTELLECTUAL PROPERTY RIGHT WHERE THE ALLEGED INFRINGEMENT IS BASED ON<br>
*
*  <li>(a) MICROCHIP'S COMPLIANCE WITH YOUR DESIGNS OR SPECIFICATIONS, OR<br>
*
*  <li>(b) YOUR USE OF MTOUCH IN COMBINATION WITH ANY OTHER PRODUCT OR IN A PROCESS (WHETHER DIRECT OR CONTRIBUTORY  INFRINGEMENT), OR<br>
*
*  <li>(c) YOUR OR ANYONE ELSE'S MODIFICATION OF MTOUCH AFTER MICROCHIP DELIVERS IT TO YOU, OR<br>
*
*  <li>(d) YOUR USE OF MTOUCH IN A MANNER FOR WHICH IT WAS NOT DESIGNED, YOU SHALL DEFEND MICROCHIP AGAINST SUCH CLAIM AND SHALL INDEMNIFY MICROCHIP AND HOLD MICROCHIP HARMLESS FOR,
*          FROM AND AGAINST ALL DAMAGES AND COSTS AWARDED AGAINST MICROCHIP.<br>
*
*  THE FOREGOING STATES THE SOLE AND EXCLUSIVE REMEDY AND LIABILITY OF THE PARTIES FOR INTELLECTUAL PROPERTY INFRINGEMENT.  IN NO EVENT SHALL MICROCHIP BE LIABLE TO YOU OR YOUR
*  CUSTOMERS FOR ANY SPECIAL, INCIDENTAL, PUNITIVE OR CONSEQUENTIAL DAMAGES FOR INTELLECTUAL PROPERTY INFRINGEMENT, INCLUDING BUT NOT LIMITED TO ANY CLAIMS FOR INDEMNITY OR CONTRIBUTION.<br><br>
* </ul>
* <li>General.</b><br>
*
* Nothing in this agreement constitutes a waiver of Microchip's rights under U.S. copyright laws or any other federal, state, local or foreign law.  You are responsible for installation,
* management and operation of the Software.  This agreement shall be construed, interpreted and governed by the laws of the State of Arizona, without regard to its conflicts of laws
* provisions.  If any provision of this agreement is found void or unenforceable, the remainder of the agreement will remain valid and enforceable according to its terms.  If any remedy
* provided is determined to have failed of its essential purpose, all limitations of liability and exclusion of damages set forth in the limited warranty shall remain in effect.<br>
* 
* This EULA shall apply to any updates, fixes, patches, or subsequent releases of the Software or Documentation supplied by Microchip, if any.<br>
* 
* By selecting "I ACCEPT" below, you represent and warrant that you are at least 18 years old.  If you are entering into this EULA on behalf of an entity, you further represent and warrant
* that you have the authority to bind that entity to the terms of this EULA.<br>
* 
* You agree to comply with all export laws, restrictions and regulations of the U.S. Department of Commerce or other United States or foreign agency or authority.<br>
* 
* If you have any questions concerning this agreement, please write Microchip Technology Inc. 2355 W. Chandler Blvd. Chandler, AZ  85224 USA, ATTN: Legal Department<br><br>
* 
* Copyright (c) 2010, Microchip Technology Incorporated, 2355 West Chandler Blvd, Chandler, AZ 85224-6199, U.S.A.  All rights reserved.  mTouch is a trademark of Microchip Technology
* Incorporated.  Microchip is a registered trademark of Microchip Technology Incorporated.<br><br>
* </ol> 
*/







//=================================================================================
//   ____                           
//  / ___|_ __ ___  _   _ _ __  ___ 
// | |  _| '__/ _ \| | | | '_ \/ __|
// | |_| | | | (_) | |_| | |_) \__ \
//  \____|_|  \___/ \__,_| .__/|___/
//                       |_|   
//=================================================================================

/**
* @defgroup Configuration Configuration Options
*
* The mTouch Configuration Options provide an easy way to manipulate the behavior of the
* framework while staying focused on the end application. Detailed explanations of each
* option are provided below.
*/
/**
* @defgroup Acquisition Acquisition
* @brief Generates and executes a configured version of the CVD acquisition routine
*
* The mTouch Acquisition module generates an assembly implementation of the interrupt
* service routine that is designed to maximize noise immunity and minimize execution
* time. It will automatically add and remove macros as the number of sensors in the
* application changes. By default, it will use the previously scanned sensor as the
* reference for the current scan. If the application only has a single sensor, a special
* acquisition method which does not require a reference sensor is used. This specialized
* method should not be used in systems with more than one sensor due to crosstalk
* concerns.
*/
/**
* @defgroup Decoding Filtering and Decoding
*
* The mTouch Filtering and Decoding module executes the detection algorithms for the
* framework. Using the latched output from the acquisition module, the service function
* is called each time a new value is available for each sensor. It will then loop
* through all of the active sensors and determine if they have changed state.
*
* This is achieved by implementing a baseline average of the sensor's reading. The
* average is only updated while the sensor is in the #MTOUCH_RELEASED state. The difference
* between the average and the current reading is compared against the #THRESHOLD_PRESS_SENSOR0
* (or equivalent) value to determine if a change in state is necessary.
*
* If the system sees that the reading has dipped below the average - the opposite direction
* of a finger press - the average will update itself based on the behavior chosen by
* configuration option #MTOUCH_NEGATIVE_CAPACITANCE. 
*/


//=================================================================================
//   ____             __ _          ___        _   _                 
//  / ___|___  _ __  / _(_) __ _   / _ \ _ __ | |_(_) ___  _ __  ___ 
// | |   / _ \| '_ \| |_| |/ _` | | | | | '_ \| __| |/ _ \| '_ \/ __|
// | |__| (_) | | | |  _| | (_| | | |_| | |_) | |_| | (_) | | | \__ \
//  \____\___/|_| |_|_| |_|\__, |  \___/| .__/ \__|_|\___/|_| |_|___/
//                         |___/        |_|    
//=================================================================================
/** @name System Setup (mTouch_config.h)
*
*   These configuration options must be uniquely set for each application. They
*   define the manner in which the framework will be implemented in the application.
*/
//@{
    /**
    * @def      _XTAL_FREQ
    * @ingroup  Configuration
    * @brief    The oscillator frequency of your system in Hz
    * @hideinitializer
    */
    
    /**
    * @def      MTOUCH_INTEGRATION_TYPE
    * @ingroup  Configuration
    * @brief    Defines the manner in which the framework will integrate with the 
    *           application.
    *
    *   Valid options:
    *   @li     #MTOUCH_CONTROLS_ISR :: The framework defines and controls the entire ISR
    *   @li     #MTOUCH_CALLED_FROM_ISR :: Your application's ISR function is responsible 
    *           for calling mTouch_Scan(). Example code is provided in main.c
    *   @li     #MTOUCH_CALLED_FROM_MAINLOOP :: The mainloop mTouch API will automatically 
    *           call the scan function.    
    *
    * @hideinitializer
    */
    
    /**
    * @def      MTOUCH_ISR_TIMER
    * @ingroup  Configuration
    * @brief    If the mTouch scanning routine is called from the ISR, this defines which 
    *           timer's interrupt flag is used.
    *
    *   8-bit Timers Only. Not compatible with TMR1/3/5. 
    *
    *   @note   You are responsible for initializing the timer with any desired pre- or 
    *           post-scaling options including any period registers (PR2/4/6). 
    *
    * @hideinitializer
    */
    
    /**
    * @def      MTOUCH_SCAN_FUNCTIONALITY
    * @ingroup  Configuration
    * @brief    Determines how many sensors are scanned in a single 'scan' function call
    *
    *   Valid options:
    *   @li     #MTOUCH_SCANS_ONE_SENSOR :: One mTouch function call will result in one 
    *           sensor being scanned once.
    *   @li     #MTOUCH_SCANS_ALL_SENSORS :: One mTouch function call will result in all 
    *           sensors being scanned once.
    *
    * @hideinitializer
    */
    
    /**
    * @def      MTOUCH_ADC_CONTROL
    * @ingroup  Configuration
    * @brief    Determines whether or not the framework controls the ADC at all times
    *
    *   Valid options:
    *   @li     #MTOUCH_ALWAYS_CONTROLS_ADC :: ADC is considered under the control of the 
    *           framework at all times. This is the more efficient method of implementation.
    *   @li     #MTOUCH_RELEASES_ADC_AFTER_SCAN :: ADC is released and available for use 
    *           between scans. This is less efficient, but provides greater flexibility.
    *
    * @hideinitializer
    */
//@}



/** @name Sensor Setup (mTouch_config.h)
*
*   These configuration options initialize the sensors in the framework by defining
*   which analog pins are to be scanned and how much sensitivity is expected from
*   them (thresholds).
*/
//@{
    /**
    * @def      MTOUCH_NUMBER_SENSORS
    * @ingroup  Configuration
    * @brief    Number of active-mode mTouch sensor inputs to scan
    * @hideinitializer
    */
    
    /**
    * @def      MTOUCH_SENSOR0
    * @ingroup  Configuration
    * @brief    Analog port assignment for the active-mode sensor with index 0
    * @hideinitializer
    */
    
    /**
    * @def      THRESHOLD_PRESS_SENSOR0
    * @ingroup  Configuration
    * @brief    Defines the press threshold of the active-mode sensor with index 0
    *
    *   This is how far the reading must deviate from the average/baseline before it 
    *   exits the #MTOUCH_RELEASED state and enters the #MTOUCH_PRESSED state. The 
    *   default values are only placeholders and should be edited to fit your design.
    *
    *   The release threshold is automatically calculated off this value and the value
    *   of #MTOUCH_RELEASE_THRESH_FACTOR. This allows the user to focus on where to 
    *   place a single threshold and the other value will follow it automatically.
    *
    * @attention To determine this value, it is highly recommended to look at the 
    *            communications output and see how your sensors are behaving. This is
    *            the only way to safely choose a threshold on any new design.
    * 
    * @hideinitializer
    */
    
    /**
    * @def      MTOUCH_RELEASE_THRESH_FACTOR
    * @ingroup  Configuration
    * @brief    Defines the hysteresis relationship between the press and release 
    *           thresholds
    *
    *   This value will be multiplied by each of the THRESHOLD_PRESS_SENSOR#
    *   values to calculate individual release thresholds for each sensor.
    *
    *   If your system is not releasing quickly enough, make this number slightly 
    *   larger to raise the threshold. If your system is releasing too quickly, make 
    *   this number slightly smaller.
    *
    *   For example:
    *
    *       RELEASE_THRESHOLD_FACTOR is 0.5<br>
    *       THRESHOLD_PRESS_SENSOR0 is 100.<br>
    *       THRESHOLD_PRESS_SENSOR1 is 150.
    *
    *       @code
    *       THRESHOLD_RELEASE_SENSOR0 = THRESHOLD_PRESS_SENSOR0 * RELEASE_THRESHOLD_FACTOR
    *       @endcode
    *
    *   This means the reading on the active-mode sensor with index '0' will need to 
    *   shift up, during a press, more than 100 counts above the average in order to 
    *   trigger a press. Once there, the average will stop updating and the sensor 
    *   will remain in the pressed state until the reading returns to within 50 counts 
    *   of the average <b>or</b> until the press timeout counter has completed. 
    *   Likewise, the active-mode sensor with index '1' will need to shift up by 150 
    *   and then down to below 75 in order to enter and exit the pressed state.
    *
    * @hideinitializer
    */
//@}


/** @name UART-PC Communications Setup (mTouch_config.h)
*
*   These configuration options define if and how the mTouch framework is to 
*   communicate with a PC.
*
*   @attention  It is highly recommended that you look at the raw data output 
*               to determine how the system is behaving and where to place the 
*               thresholds.
*/
//@{
    /**
    * @def      MTOUCH_V1_COMMS
    * @ingroup  Configuration
    * @brief    If enabled, a one-way ASCII stream is output using the UART protocol.
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_V1_COMM_SPEED
    * @ingroup  Configuration
    * @brief    If V1 comms are enabled, defines the output baud rate.
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_V1_COMM_OUTPUT_RAW
    * @ingroup  Configuration
    * @brief    If V1 comms are enabled, enables output of raw sensor values.
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_V1_COMM_OUTPUT_AVG
    * @ingroup  Configuration
    * @brief    If V1 comms are enabled, enables output of average sensor values.
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_V1_COMM_OUTPUT_SLIDERS
    * @ingroup  Configuration
    * @brief    If V1 comms are enabled, enables output of slider decoding values.
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_V1_COMM_DELIMITER
    * @ingroup  Configuration
    * @brief    If V1 comms are enabled, defines which character is used as the 
    *           delimiter of bytes of data.
    */
    /**
    * @def      MTOUCH_PIC18_TX2_PIN
    * @ingroup  Configuration
    * @brief    If V1 comms are enabled and using a PIC18 device, defines the name 
    *           of the register that the framework should load the 'TX' value into.
    */
    /**
    * @def      MTOUCH_V1_COMM_BITBANGED
    * @ingroup  Configuration
    * @brief    If V1 comms are enabled, this forces the framework to implement the
    *           UART communications using a software routine.
    */
    /**
    * @def      MTOUCH_SOFT_UART_TXPORT
    * @ingroup  Configuration
    * @brief    If bit-banged V1 comms are enabled, defines which port to use for
    *           output.
    */
    /**
    * @def      MTOUCH_SOFT_UART_TXTRIS
    * @ingroup  Configuration
    * @brief    If bit-banged V1 comms are enabled, defines which tris to use for
    *           output.
    */
    /**
    * @def      MTOUCH_SOFT_UART_TXPIN
    * @ingroup  Configuration
    * @brief    If bit-banged V1 comms are enabled, defines which pin to use for 
    *           output.
    */
    /**
    * @def      MTOUCH_V2_COMMS
    * @ingroup  Configuration
    * @brief    If enabled, a two-way non-ASCII stream and challenge/response system 
    *           is implemented using the UART protocol.
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_EEPROM_START_ADR
    * @ingroup  Configuration
    * @brief    If V2 comms are enabled, defines the starting address for the
    *           mTouch EEPROM data storage.
    * @hideinitializer
    */
//@}


/** @name Filtering Setup (mTouch_config.h)
*
*   These configuration options define if and how the mTouch framework is to 
*   communicate with a PC.
*/
//@{
    /**
    * @def      MTOUCH_SAMPLES_PER_SCAN
    * @ingroup  Configuration
    * @brief    Determines the degree of oversampling and the sample rate of your system
    *
    *   This is the number of times each sensor will be scanned before asserting the 
    *   mTouch_state.dataReady bit. If the application requires a specific response 
    *   time, there are several variables that  will be important to the calculation 
    *   of this parameter.
    *
    *   @li _XTAL_FREQ is the PICs oscillation frequency and will determine how quickly 
    *       the TMR0 counter will increment.
    *
    *   @li The interrupt timer's pre/postscaler also determines how quickly the counter 
    *       will increment. This value is not set by the framework.
    *
    *   @li MTOUCH_NUMBER_SENSORS defines how many times the ISR will interrupt between 
    *       each decrement of the sample counter. The sample counter is initialized
    *       with MTOUCH_SAMPLES_PER_SCAN as its value.
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_AVG_UPDATE
    * @ingroup  Configuration
    * @brief    Determines the weight given to the raw value when updating the average.
    *
    *   When the average updates itself using a new reading, this value determines what 
    *   weight is given to the reading variable in the  calculation of the new average.
    *
    *   The reading variable will have a weight of 1/MTOUCH_AVG_UPDATE in the average 
    *   calculation. 
    *
    *   Example:
    *
    *   If MTOUCH_AVG_UPDATE is 4, the average calculation would be:
    *   @code
    *   average = (1/4)*reading + (3/4)*average
    *   @endcode
    *   <b>Allowed Values :: 1 - 4</b>
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_AVG_RATE
    * @ingroup  Configuration
    * @brief    Sets the update rate of the average when unpressed.
    *
    *   This value is used to seed a counter variable that will cause the average to be 
    *   updated only once every Nth new reading value, where N is MTOUCH_AVG_RATE.
    *
    *   Example:
    *
    *   If MTOUCH_AVG_RATE is 2, the average will use every other new reading to update 
    *   itself.
    *
    *   <b>Allowed Values :: 1 - 65535</b>
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_JITTER_BITS
    * @ingroup  Configuration
    * @brief    Defines the number of bits to use to implement a random delay between
    *           mTouch scans.
    *
    *   A random value is loaded into the interrupt timer to vary the interrupt's rate.
    *   The maximum number of bits of the random value is determined by this option.
    *   It is set to a value of '6' by default. More bits = more randomness.
    *
    *   <b>Allowed Values :: 0 - 8</b>
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_DECIMATION_MAX_STEP
    * @ingroup  Configuration
    * @brief    Maximum allowed change of the framework's latched mTouch result
    *           variable from a single mTouch sample. 
    *
    *   To achieve maximum noise immunity, this value should be set to '1'. If the
    *   reading is having trouble keeping up with the level of sensitivity in your
    *   application, however, this value can be used to increase the maximum speed
    *   of the sensor's reading.
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SCALING
    * @ingroup  Configuration
    * @brief    mTouch post-scaler of reading value
    *
    *   This option will determine the amount of post-scaling that is implemented on the 
    *   acquisition's accumulator register before being stored in the results register.
    *   The scaling factor should be set so that it is not possible for the result 
    *   register to overflow (Max output value is 65535.)
    *
    *   Set this option to the number of times the framework should right shift the 
    *   accumulator value to achieve the reading. When in doubt, '4' is usually a good 
    *   option.
    *
    * @hideinitializer
    */
//@}



/** @name Decoding Setup (mTouch_config.h)
*
*   These configuration options will adjust the decoding behavior of the mTouch Framework.
*   Editting these values is optional but allows for a more highly customized final
*   result.
*/
//@{
    /**
    * @def      MTOUCH_POWER_UP_SAMPLES
    * @ingroup  Configuration
    * @brief    Number of times each sensor should be scanned and decoded before exitting
    *           the initialization state and entering the released state.
    *
    *   Allowable Range: 1-65535
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_BUTTON_TIMEOUT
    * @ingroup  Configuration
    * @brief    Number of mTouch decode function calls where the sensor is consecutively
    *           pressed before resetting the sensor to its released state
    *
    *   This feature will reset the sensor's state if it remains in the 'pressed' state
    *   for too long. 
    *
    *   For example: 
    *
    *   Let's assume the dataReady flag is set once every 100ms due to the 
    *   MTOUCH_SAMPLES_PER_SCAN option. If we want a timeout of 10sec, the 
    *   MTOUCH_BUTTON_TIMEOUT value should be set to 100. If we want a timeout of 7.4 
    *   seconds, then BUTTON_TIMEOUT should be set to 74.
    *
    *   @code
    *   Timeout Duration = dataReady Toggle Rate * BUTTON_TIMEOUT 
    *   @endcode
    *
    *   Setting to '0' or commenting the #define will disable the timeout feature.
    *
    *   <b>Allowable Range :: [ 0, 2 - 255 ]</b>
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_DEBOUNCE_RELEASE
    * @ingroup  Configuration
    * @brief    Number of consecutive scans a sensor must be decoded as released before 
    *           changing states
    *
    *   Increasing this value will significantly slow response time, so make sure the 
    *   scan rate is fast enough to continue to provide the desired response rate.
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_DEBOUNCE_PRESS
    * @ingroup  Configuration
    * @brief    Number of consecutive scans a sensor must be decoded as pressed before 
    *           changing states
    *
    *   Increasing this value will significantly slow response time, so make sure the 
    *   scan rate is fast enough to continue to provide the desired response rate.
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_NEGATIVE_CAPACITANCE
    * @ingroup  Configuration
    * @brief    Determines the method used to handle a negative capacitance shift on the 
    *           sensor's readings
    *
    *   This option determines the method used to handle a negative capacitance shift. 
    *   In other words, how the system will respond to a reading value shift in the 
    *   opposite direction of a normal shift caused by a finger.
    *
    *   These shifts occur for a variety of reasons and you may want to adjust this 
    *   behavior based on your system.
    *
    *   <b>Options:</b>
    *   @li <b>0</b> - Normal Average Behavior<br>
    *       (DEFAULT) No special behavior. The average will update at the normal rate, as 
    *       always. For Metal-over-Capacitive systems, a negative shift usually means a 
    *       neighboring sensor is being pressed, so this is the best option to keep the 
    *       current state of the sensor and allow it to be pressed quickly after a neighbor 
    *       has been pressed.
    *   @li <b>1</b> - Increase Average Speed<br>
    *       The average will ignore the AVG_RATE counter and update itself after each new 
    *       reading value. This has the effect of speeding up the average until it catches
    *       up with the negative shift. 
    *   @li <b>2</b> - Average Directly Follows Reading<br>
    *       The average will instantly follow a negative shift. This behavior is best for 
    *       a system that needs a very fast response time. It will help prevent quick, 
    *       repeated presses  from being missed due to the average not updating quickly 
    *       enough.
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_ACTIVE_MOST_PRESSED_ONLY
    * @ingroup  Configuration
    * @brief    If enabled, only one active-mode sensor will be able to be pressed at
    *           any one time.
    *
    *   The sensor with the largest shift is allowed to remain pressed. All others are
    *   reset to the released state.
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_TOGGLE_ENABLED
    * @ingroup  Configuration
    * @brief    If enabled, a toggle latch bit will be implemented for each active sensor.
    *
    *   The toggle bit is changed on-press only. So when the user presses the first time,
    *   the toggle bit is set high. When the user releases, nothing happens. When the user
    *   presses again, the toggle bit is set low. When the user releases, nothing happens.
    *   Etc.
    *
    * @hideinitializer
    */
//@}

/** @name Slider and Wheel Setup (mTouch_config_slider.h)
*
*   These options will configure if there are any sliders and wheels in your application
*   as well as how they are decoded. The framework is able to support up to 10 sliders
*   and 8 sensors per slider. (The framework does not support 80 unique sensors - if an
*   application actually used all 10 sliders with 8 sensors per slider, it would need to
*   re-use some active-mode sensors multiple times.)
*
*/
//@{
    /**
    * @def      MTOUCH_NUMBER_OF_SLIDERS
    * @ingroup  Configuration
    * @brief    Defines the number of sliders to be implemented
    *
    *   Any non-zero value will enable this module.
    *
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_NUMBER_SLIDER0_SENSORS
    * @ingroup  Configuration
    * @brief    Required for each slider. Defines the number of sensors that make up slider 0. 
    *
    *   @attention  When more than one slider is enabled, each slider needs its own
    *               version of this configuration option. Ex:
    *               MTOUCH_NUMBER_SLIDER1_SENSORS, MTOUCH_NUMBER_SLIDER2_SENSORS, etc.
    *               
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLIDER0_SENSOR0
    * @ingroup  Configuration
    * @brief    Required for each sensor of each slider. Defines the active-mode sensor index of Slider 0's first sensor.
    *
    *   @attention  This is the index value of the active-mode sensor - NOT the analog
    *               channel number. 
    *               
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLIDER_STEP_SIZE
    * @ingroup  Configuration
    * @brief    Defines the maximum value that the latched slider output value is allowed
    *           to change based on one new reading. Similar to MTOUCH_DECIMATION_MAX_STEP.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLIDER0_WHEEL_ENABLE
    * @ingroup  Configuration
    * @brief    If defined, tells the framework to decode the slider as a wheel.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLIDER_SCALING_ENABLED
    * @ingroup  Configuration
    * @brief    If defined, slider output values will always fall between 0 and 255,
    *           regardless of the number of sensors. If not, the maximum value will be
    *           based on the number of sensors in the slider.
    * 
    * @hideinitializer
    */
//@}


/** @name Sleep-Mode Setup (mTouch_config_sleep.h)
*
*   Provides the ability to add 'sleep mode' sensors to your design. Once at least one
*   sleep-mode sensor is enabled, the framework will adjust its behavior to alternate
*   between active and sleep mode.
*
*/
//@{
    /**
    * @def      MTOUCH_NUMBER_OF_SLEEP_SENSORS
    * @ingroup  Configuration
    * @brief    Defines the number of sleep-mode sensors to implement in the framework.
    *           Sleep-mode is automatically enabled when this value is larger than 0.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_SENSOR0
    * @ingroup  Configuration
    * @brief    Defines the analog channel of the first sensor to scan in sleep mode.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_SAMPLES_PER_SCAN
    * @ingroup  Configuration
    * @brief    Sleep mode equivalent to MTOUCH_SAMPLES_PER_SCAN.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_INACTIVE_RESET
    * @ingroup  Configuration
    * @brief    Defines how many "all sensors released" active-mode consecutive decodes
    *           the application must see before entering sleep-mode.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_INIT_SCANS
    * @ingroup  Configuration
    * @brief    Defines how many times each sleep sensor will be scanned before leaving
    *           the initialization state the first time.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_REINIT_SCANS
    * @ingroup  Configuration
    * @brief    Defines how many times each sleep sensor will be scanned before leaving
    *           the initialization state after each active-to-sleep mode transition.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_MAX_PRESSTIME
    * @ingroup  Configuration
    * @brief    Sleep mode equivalent to MTOUCH_BUTTON_TIMEOUT. Only implemented if no
    *           active mode sensors are enabled.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_PTHRESH_SENSOR0
    * @ingroup  Configuration
    * @brief    Sleep mode equivalent to PRESS_THRESHOLD_SENSOR0.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_RTHRESH_FACTOR
    * @ingroup  Configuration
    * @brief    Sleep mode equivalent to MTOUCH_RELEASE_THRESH_FACTOR.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_AVG_RATE
    * @ingroup  Configuration
    * @brief    Sleep mode equivalent to MTOUCH_AVG_RATE.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_AVG_UPDATE
    * @ingroup  Configuration
    * @brief    Sleep mode equivalent to MTOUCH_AVG_UPDATE.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_DECIMATION_STEP
    * @ingroup  Configuration
    * @brief    Sleep mode equivalent to MTOUCH_DECIMATION_MAX_STEP.
    * 
    * @hideinitializer
    */
    /**
    * @def      MTOUCH_SLEEP_SCAN_INTERRUPTABLE
    * @ingroup  Configuration
    * @brief    If defined, the sleep mode scan routine will implement a check to 
    *           determine if it was interrupted mid-scan. If it is, the data is trashed
    *           and the scan is performed again.
    * 
    * @hideinitializer
    */
//@}

/** @name Advanced CVD Scan Controls (mTouch_config_cvdAdvanced.h)
*
*   These controls provide additional, more advanced options for adjusting the behavior
*   of the CVD scan. Incorrect configuration of these options can result in very strange,
*   erratic behavior.
*
*/
//@{
    /**
    * @def      CVD_SETTLING_DELAY
    * @ingroup  Configuration
    * @brief    Determines the amount of time provided for the external sensor and 
    *           internal hold capacitor  to balance their voltages during the CVD 
    *           scanning process.
    *
    *   If this value is <b>too small</b>: The sensors will not be as sensitive as they 
    *   could be and your sensor's readings will be VDD dependant.
    *
    *   If this value is <b>too large</b>: The noise immunity of the system will not be 
    *   as robust as it could be.<br>
    *
    *   To correctly set this value:
    *   <ol>
    *    <li>Set this value to 0, compile and program.
    *    <li>Look at the raw values on the mTouch GUI or through a terminal program while
    *        powering the system at the desired VDD level.
    *    <li>Adjust VDD by plus and minus 0.5V 
    *       <ul>
    *       <li>Do you notice any change in the unpressed value as VDD is changing? If no, 
    *           leave this value alone. If yes...
    *       </ul>
    *    <li>Increase the delay until the sensor's reading no longer changes as VDD changes.
    *    </ol>
    *   Once this has occurred, the settling time has been correctly tuned to provide the 
    *   maximum amount of sensitivity while minimizing the framework's susceptibility to 
    *   noise.
    * 
    * @hideinitializer
    */
    /**
    * @def      CVD_CHOLD_CHARGE_DELAY
    * @ingroup  Configuration
    * @brief    Determines the amount of time provided for charging the internal 
    *           hold capacitor of the ADC during the CVD scanning process.
    *
    *   If this value is <b>too small</b>: There will be noticable crosstalk between 
    *   closely-indexed sensors that is not related to the hardware layout of the application.
    *
    *   If this value is <b>too large</b>: The mTouch ISR will take an unnecessarily long 
    *   amount of time to execute.<br>
    *
    *   To correctly set this value:
    *   <ol>
    *    <li>Set this value to 0, compile and program.
    *    <li>Look at the raw values on the mTouch GUI or through a terminal program.
    *    <li>Press as hard as you can on the sensor with the highest amount of capacitance. 
    *       (This usuallycorresponds to the sensor with the highest unpressed raw value.)
    *       <ul>
    *        <li>Do you notice a strange crosstalk behavior on a different sensor? If no, 
    *            leave this value to 0. If yes...
    *       </ul>
    *    <li>Increase the delay until the crosstalk behavior is eliminated. 
    *   </ol>
    * 
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_IO_ENABLED
    * @ingroup  Configuration
    * @brief    If defined, the framework will implement a guard sensor using an IO port.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_PORT
    * @ingroup  Configuration
    * @brief    If CVD_GUARD_IO_ENABLED is defined, this determines which port to use for 
    *           the guard.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_PIN
    * @ingroup  Configuration
    * @brief    If CVD_GUARD_IO_ENABLED is defined, this determines which pin to use for 
    *           the guard.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_DACOUT_ENABLED
    * @ingroup  Configuration
    * @brief    If defined, the framework will implement a guard sensor using the DAC's
    *           DACOUT pin. If available, this is recommended over the IO option as it
    *           provides better results.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_DACCON0_A
    * @ingroup  Configuration
    * @brief    If CVD_GUARD_DACOUT_ENABLED is defined, this determines what value is
    *           loaded into the DACCON0 register prior to the first ADC sample of the
    *           differential CVD scan.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_DACCON1_A
    * @ingroup  Configuration
    * @brief    If CVD_GUARD_DACOUT_ENABLED is defined, this determines what value is
    *           loaded into the DACCON1 register prior to the first ADC sample of the
    *           differential CVD scan.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_DACCON0_B
    * @ingroup  Configuration
    * @brief    If CVD_GUARD_DACOUT_ENABLED is defined, this determines what value is
    *           loaded into the DACCON0 register prior to the second ADC sample of the
    *           differential CVD scan.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_GUARD_DACCON1_B
    * @ingroup  Configuration
    * @brief    If CVD_GUARD_DACOUT_ENABLED is defined, this determines what value is
    *           loaded into the DACCON1 register prior to the second ADC sample of the
    *           differential CVD scan.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_MUTUAL_ENABLED
    * @ingroup  Configuration
    * @brief    If defined, the framework will integrate a mutual sensor in to the
    *           differential CVD scan.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_MUTUAL_EXCLUSIVE_ENABLED
    * @ingroup  Configuration
    * @brief    If defined, the framework will attempt to perform an exclusively-mutual
    *           scan. NOTE: Not yet tested!
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_MUTUAL_LAT
    * @ingroup  Configuration
    * @brief    If CVD_MUTUAL_ENABLED is defined, determines what LAT register is used
    *           for the mutual sensor.
    *
    * @hideinitializer
    */
    /**
    * @def      CVD_MUTUAL_TRIS
    * @ingroup  Configuration
    * @brief    If CVD_MUTUAL_ENABLED is defined, determines which pin is used for the
    *           mutual sensor.
    *
    * @hideinitializer
    */
//@}


/** @name Unimplemented
*
*   These options will be used in later versions of the V2 communications.
*/
//@{
/**
* @def      MTOUCH_BOARD_ID
* @brief    No effect on framework behavior.
* @hideinitializer
*/
/**
* @def      MTOUCH_FRAME_REVISION
* @brief    No effect on framework behavior.
* @hideinitializer
*/
/**
* @def      MTOUCH_BOARD_REVISION
* @brief    No effect on framework behavior.
* @hideinitializer
*/
/**
* @def      MTOUCH_SOFTWARE_REVISION
* @brief    No effect on framework behavior.
* @hideinitializer
*/
//@}

