/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    mTouch Framework v2.0
 *  FileName:        mTouch_eeprom.h
 *
 *  Processor:       See documentation for supported PIC microcontrollers 
 *  Compiler:        HI-TECH Ver. 9.81 or later
 *  IDE:             MPLAB IDE v8.50 (or later) or MPLAB X                        
 *  Hardware:         
 *  Company:         
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *  Description:     mTouch Framework Include File
 *                   - No application code should be implemented in this
 *                     or any other mTouch framework file. This will allow
 *                     for better customer support and easier upgrades to
 *                     later firmware versions. Use the main.c and user-
 *                     generated files to implement your application.
 *                   - See the documentation located in the docs/ folder
 *                     for a more information about how the framework is
 *                     implemented.
 *************************************************************************/
/**************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and 
 * any derivatives created by any person or entity by or on your behalf, 
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors 
 * retain all ownership and intellectual property rights in the 
 * accompanying software and in all derivatives hereto. 
 * 
 * This software and any accompanying information is for suggestion only. 
 * It does not modify Microchip's standard warranty for its products. You 
 * agree that you are solely responsible for testing the software and 
 * determining its suitability. Microchip has no obligation to modify, 
 * test, certify, or support the software. 
 * 
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER 
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED 
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A 
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH 
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY 
 * APPLICATION. 
 * 
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY, 
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT 
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT, 
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, 
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE, 
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY 
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW, 
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS 
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID 
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE. 
 * 
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF 
 * THESE TERMS. 
 *************************************************************************/
/** @file   mTouch_eeprom.h
*   @brief  Internal framework header file implementing EEPROM functionality
*/
#if defined(MTOUCH_EEPROM_ENABLED)
#ifndef __MTOUCH_EEPROM_H
#define __MTOUCH_EEPROM_H

    #if _EEPROMSIZE == 0
        #error The mTouch Framework's EEPROM implementation has been enabled for V2 communications, but the selected part does not contain an EEPROM! 'V2' communications are not possible on this device. Please enable 'V1' communications or turn off comms.
    #endif

    #if defined(_PIC18) | defined(_PIC14)
        #define MTOUCH_EEPROM_read(addr)            EEPROM_READ(addr)
        #define MTOUCH_EEPROM_write(addr, value)    EEPROM_WRITE(addr, value)
    #else
        #define MTOUCH_EEPROM_read(addr)            eeprom_read(addr)
        #define MTOUCH_EEPROM_write(addr, value)    eeprom_write(addr, value)
    #endif


    // BITS
    #define   EECON1_RD                 EECON1bits.RD       // Read start
    #define   EECON1_WR                 EECON1bits.WR       // Write start
    #define   EECON1_WREN               EECON1bits.WREN     // Write enable
    #define   EECON1_WRERR              EECON1bits.WRERR    // 1:write abort   0:write complete
    #define   EECON1_CFGS               EECON1bits.CFGS     // Flash Program/Data EEPROM or Configuration Select bit (1:configuration 0:Flash/EEPROM)
    #define   EECON1_EEPGD              EECON1bits.EEPGD    // Flash Program or Data EEPROM Memory Select bit (1:flash 0:eeprom)

    // Utility Macros
    #define   START_READ_EEPROM()       EECON1 |= EECON1bits.RD
    #define   START_WRITE_EEPROM()      EECON1 |= EECON1bits.WR
    #define   ENABLE_WRITE_EEPROM()     EECON1 |= EECON1bits.WREN
    #define   DISABLE_WRITE_EEPROM()    EECON1 &= ~EECON1bits.WREN
    #define   SELECT_EEPROM()           EECON1 &= ~(EECON1bits.EEPGD | EECON1bits.CFGS)

    // MAPPING

    // Block sizes
    #define   MTOUCH_EEPROM_SYS_SIZE    18
    #define   MTOUCH_EESLEEP_SYS_SIZE   16
    #define   MTOUCH_EEPROM_SENS_SIZE   4
    #define   MTOUCH_EESLEEP_SENS_SIZE  4

    #define   MTOUCH_EEPROM_INIT_VALUE  0xBD                // Used as an initialization signature

    // Addresses Calculations
    #define   MTOUCH_EEPROM_SYS_ADR     MTOUCH_EEPROM_START_ADR + 1
    #define   MTOUCH_EEPROM_SENS_ADR    MTOUCH_EEPROM_SYS_ADR   + MTOUCH_EEPROM_SYS_SIZE    
    
    #if defined(MTOUCH_SLEEP_ENABLED)
    #define   MTOUCH_EESLEEP_SYS_ADR    MTOUCH_EEPROM_SENS_ADR  + (MTOUCH_EEPROM_SENS_SIZE * MTOUCH_NUMBER_SENSORS)
    #define   MTOUCH_EESLEEP_SENS_ADR   MTOUCH_EESLEEP_SYS_ADR  + MTOUCH_EESLEEP_SYS_SIZE 
    #endif
    
    // Ending Address Calculations
    #if defined(MTOUCH_SLEEP_ENABLED)
    #define   MTOUCH_EEPROM_END_ADR     MTOUCH_EESLEEP_SENS_ADR  + (MTOUCH_EESLEEP_SENS_SIZE * MTOUCH_NUMBER_OF_SLEEP_SENSORS)
    #else
    #define   MTOUCH_EEPROM_END_ADR     MTOUCH_EEPROM_SENS_ADR  + (MTOUCH_EEPROM_SENS_SIZE * MTOUCH_NUMBER_SENSORS)
    #endif

    // System Block
    #define   MTOUCH_EEPROM_BOARDID_ADR     MTOUCH_EEPROM_SYS_ADR
    #define   MTOUCH_EEPROM_rTHRESHper_ADR  MTOUCH_EEPROM_SYS_ADR + 1
    #define   MTOUCH_EEPROM_pDEB_ADRL       MTOUCH_EEPROM_SYS_ADR + 2
    #define   MTOUCH_EEPROM_pDEB_ADRH       MTOUCH_EEPROM_SYS_ADR + 3
    #define   MTOUCH_EEPROM_rDEB_ADRL       MTOUCH_EEPROM_SYS_ADR + 4
    #define   MTOUCH_EEPROM_rDEB_ADRH       MTOUCH_EEPROM_SYS_ADR + 5
    #define   MTOUCH_EEPROM_SpS_ADRL        MTOUCH_EEPROM_SYS_ADR + 6
    #define   MTOUCH_EEPROM_SpS_ADRH        MTOUCH_EEPROM_SYS_ADR + 7
    #define   MTOUCH_EEPROM_SCALE_ADR       MTOUCH_EEPROM_SYS_ADR + 8
    #define   MTOUCH_EEPROM_pTIMEOUT_ADRL   MTOUCH_EEPROM_SYS_ADR + 9
    #define   MTOUCH_EEPROM_pTIMEOUT_ADRH   MTOUCH_EEPROM_SYS_ADR + 10
    #define   MTOUCH_EEPROM_AVGUPDATE_ADR   MTOUCH_EEPROM_SYS_ADR + 11
    #define   MTOUCH_EEPROM_AVGRATE_ADRL    MTOUCH_EEPROM_SYS_ADR + 12
    #define   MTOUCH_EEPROM_AVGRATE_ADRH    MTOUCH_EEPROM_SYS_ADR + 13
    #define   MTOUCH_EEPROM_JITTER_ADR      MTOUCH_EEPROM_SYS_ADR + 14
    #define   MTOUCH_EEPROM_NEGCAP_ADR      MTOUCH_EEPROM_SYS_ADR + 15
    #define   MTOUCH_EEPROM_DECSTEP_ADRL    MTOUCH_EEPROM_SYS_ADR + 16
    #define   MTOUCH_EEPROM_DECSTEP_ADRH    MTOUCH_EEPROM_SYS_ADR + 17
    
    #if defined(MTOUCH_SLEEP_ENABLED)
    #define   MTOUCH_EESLEEP_INACTIVE_ADRL  MTOUCH_EESLEEP_SYS_ADR
    #define   MTOUCH_EESLEEP_INACTIVE_ADRH  MTOUCH_EESLEEP_SYS_ADR + 1
    #define   MTOUCH_EESLEEP_rTHRESHper_ADR MTOUCH_EESLEEP_SYS_ADR + 2
    #define   MTOUCH_EESLEEP_SpS_ADRL       MTOUCH_EESLEEP_SYS_ADR + 3
    #define   MTOUCH_EESLEEP_SpS_ADRH       MTOUCH_EESLEEP_SYS_ADR + 4
    #define   MTOUCH_EESLEEP_INIT_ADRL      MTOUCH_EESLEEP_SYS_ADR + 5
    #define   MTOUCH_EESLEEP_INIT_ADRH      MTOUCH_EESLEEP_SYS_ADR + 6
    #define   MTOUCH_EESLEEP_REINIT_ADRL    MTOUCH_EESLEEP_SYS_ADR + 7
    #define   MTOUCH_EESLEEP_REINIT_ADRH    MTOUCH_EESLEEP_SYS_ADR + 8
    #define   MTOUCH_EESLEEP_pTIMEOUT_ADRL  MTOUCH_EESLEEP_SYS_ADR + 9
    #define   MTOUCH_EESLEEP_pTIMEOUT_ADRH  MTOUCH_EESLEEP_SYS_ADR + 10
    #define   MTOUCH_EESLEEP_AVGUPDATE_ADR  MTOUCH_EESLEEP_SYS_ADR + 11
    #define   MTOUCH_EESLEEP_AVGRATE_ADRL   MTOUCH_EESLEEP_SYS_ADR + 12
    #define   MTOUCH_EESLEEP_AVGRATE_ADRH   MTOUCH_EESLEEP_SYS_ADR + 13
    #define   MTOUCH_EESLEEP_DECSTEP_ADRL   MTOUCH_EESLEEP_SYS_ADR + 14
    #define   MTOUCH_EESLEEP_DECSTEP_ADRH   MTOUCH_EESLEEP_SYS_ADR + 15
    #endif

    // Sensor Block
    #define   MTOUCH_EEPROM_pTHRESH_ADRL    MTOUCH_EEPROM_SENS_ADR
    #define   MTOUCH_EEPROM_pTHRESH_ADRH    MTOUCH_EEPROM_SENS_ADR + 1
    #define   MTOUCH_EEPROM_rTHRESH_ADRL    MTOUCH_EEPROM_SENS_ADR + 2
    #define   MTOUCH_EEPROM_rTHRESH_ADRH    MTOUCH_EEPROM_SENS_ADR + 3
    
    #if defined(MTOUCH_SLEEP_ENABLED)
    #define   MTOUCH_EESLEEP_pTHRESH_ADRL   MTOUCH_EESLEEP_SENS_ADR
    #define   MTOUCH_EESLEEP_pTHRESH_ADRH   MTOUCH_EESLEEP_SENS_ADR + 1
    #define   MTOUCH_EESLEEP_rTHRESH_ADRL   MTOUCH_EESLEEP_SENS_ADR + 2
    #define   MTOUCH_EESLEEP_rTHRESH_ADRH   MTOUCH_EESLEEP_SENS_ADR + 3
    #endif

    // Initialization
    #define MTOUCH_EEPROM_INIT_THRESHOLD(index)                                                                                                                                             \
        do {                                                                                                                                                                                \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTHRESH_ADRL + (MTOUCH_EEPROM_SENS_SIZE*index), (unsigned char)(__paste(THRESHOLD_PRESS_SENSOR,index) & 0xFF));                                             \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTHRESH_ADRH + (MTOUCH_EEPROM_SENS_SIZE*index), (unsigned char)(__paste(THRESHOLD_PRESS_SENSOR,index) >> 8));                                               \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESH_ADRL + (MTOUCH_EEPROM_SENS_SIZE*index), (unsigned char)((unsigned int)(MTOUCH_RELEASE_THRESH_FACTOR * __paste(THRESHOLD_PRESS_SENSOR,index)) & 0xFF));  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESH_ADRH + (MTOUCH_EEPROM_SENS_SIZE*index), (unsigned char)((unsigned int)(MTOUCH_RELEASE_THRESH_FACTOR * __paste(THRESHOLD_PRESS_SENSOR,index)) >> 8));    \
        } while (0)
        
    #if defined(MTOUCH_SLEEP_ENABLED)
    #define MTOUCH_EESLEEP_INIT_THRESHOLD(index)                                                                                                                                             \
        do {                                                                                                                                                                                \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_pTHRESH_ADRL + (MTOUCH_EESLEEP_SENS_SIZE*index), (unsigned char)(__paste(MTOUCH_SLEEP_PTHRESH_SENSOR,index) & 0xFF));                                             \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_pTHRESH_ADRH + (MTOUCH_EESLEEP_SENS_SIZE*index), (unsigned char)(__paste(MTOUCH_SLEEP_PTHRESH_SENSOR,index) >> 8));                                               \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_rTHRESH_ADRL + (MTOUCH_EESLEEP_SENS_SIZE*index), (unsigned char)((unsigned int)(MTOUCH_SLEEP_RTHRESH_FACTOR * __paste(MTOUCH_SLEEP_PTHRESH_SENSOR,index)) & 0xFF));  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_rTHRESH_ADRH + (MTOUCH_EESLEEP_SENS_SIZE*index), (unsigned char)((unsigned int)(MTOUCH_SLEEP_RTHRESH_FACTOR * __paste(MTOUCH_SLEEP_PTHRESH_SENSOR,index)) >> 8));    \
        } while (0)
    #endif
        
    #define MTOUCH_EEPROM_pTHRESHOLD(index) (uint16_t)((uint16_t)(MTOUCH_EEPROM_read(MTOUCH_EEPROM_pTHRESH_ADRH + (MTOUCH_EEPROM_SENS_SIZE*index))) << 8) + MTOUCH_EEPROM_read(MTOUCH_EEPROM_pTHRESH_ADRL + (MTOUCH_EEPROM_SENS_SIZE*index))
    #define MTOUCH_EEPROM_rTHRESHOLD(index) (uint16_t)((uint16_t)(MTOUCH_EEPROM_read(MTOUCH_EEPROM_rTHRESH_ADRH + (MTOUCH_EEPROM_SENS_SIZE*index))) << 8) + MTOUCH_EEPROM_read(MTOUCH_EEPROM_rTHRESH_ADRL + (MTOUCH_EEPROM_SENS_SIZE*index))
    
    #if defined(MTOUCH_SLEEP_ENABLED)
    #define MTOUCH_EESLEEP_pTHRESHOLD(index) (uint16_t)((uint16_t)(MTOUCH_EEPROM_read(MTOUCH_EESLEEP_pTHRESH_ADRH + (MTOUCH_EESLEEP_SENS_SIZE*index))) << 8) + MTOUCH_EEPROM_read(MTOUCH_EESLEEP_pTHRESH_ADRL + (MTOUCH_EESLEEP_SENS_SIZE*index))
    #define MTOUCH_EESLEEP_rTHRESHOLD(index) (uint16_t)((uint16_t)(MTOUCH_EEPROM_read(MTOUCH_EESLEEP_rTHRESH_ADRH + (MTOUCH_EESLEEP_SENS_SIZE*index))) << 8) + MTOUCH_EEPROM_read(MTOUCH_EESLEEP_rTHRESH_ADRL + (MTOUCH_EESLEEP_SENS_SIZE*index))
    #endif
        
    #if defined(CVD_JITTER_ENABLE)
        #define MTOUCH_EEPROM_INIT_JITTER() MTOUCH_EEPROM_write(MTOUCH_EEPROM_JITTER_ADR, 1)
    #else
        #define MTOUCH_EEPROM_INIT_JITTER() MTOUCH_EEPROM_write(MTOUCH_EEPROM_JITTER_ADR, 0)
    #endif
        
    #define MTOUCH_EEACTIVE_INIT()                                                                                  \
        do {                                                                                                        \
            MTOUCH_EEPROM_INIT_THRESHOLDS();                                                                        \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_BOARDID_ADR,      (uint8_t)(MTOUCH_BOARD_ID)                      );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_rTHRESHper_ADR,   (uint8_t)(MTOUCH_RELEASE_THRESH_FACTOR * 100)   );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_pDEB_ADRL,        (uint8_t)(MTOUCH_DEBOUNCE_PRESS & 0xFF)         );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_pDEB_ADRH,        (uint8_t)(MTOUCH_DEBOUNCE_PRESS >> 8)           );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_rDEB_ADRL,        (uint8_t)(MTOUCH_DEBOUNCE_RELEASE & 0xFF)       );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_rDEB_ADRH,        (uint8_t)(MTOUCH_DEBOUNCE_RELEASE >> 8)         );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_SpS_ADRL,         (uint8_t)(MTOUCH_SAMPLES_PER_SCAN & 0xFF)       );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_SpS_ADRH,         (uint8_t)(MTOUCH_SAMPLES_PER_SCAN >> 8)         );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_SCALE_ADR,                  MTOUCH_SCALING                        );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTIMEOUT_ADRL,    (uint8_t)(MTOUCH_BUTTON_TIMEOUT & 0xFF)         );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_pTIMEOUT_ADRH,    (uint8_t)(MTOUCH_BUTTON_TIMEOUT >> 8)           );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_AVGUPDATE_ADR,              MTOUCH_AVG_UPDATE                     );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_AVGRATE_ADRL,     (uint8_t)(MTOUCH_AVG_RATE & 0xFF)               );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_AVGRATE_ADRH,     (uint8_t)(MTOUCH_AVG_RATE >> 8)                 );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_JITTER_ADR,                 mTouch_jitter_mask[MTOUCH_JITTER_BITS]);  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_NEGCAP_ADR,                 MTOUCH_NEGATIVE_CAPACITANCE           );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_DECSTEP_ADRL,     (uint8_t)(MTOUCH_DECIMATION_MAX_STEP & 0xFF)    );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_DECSTEP_ADRH,     (uint8_t)(MTOUCH_DECIMATION_MAX_STEP >> 8)      );  \
            MTOUCH_EEPROM_write(MTOUCH_EEPROM_START_ADR,                  MTOUCH_EEPROM_INIT_VALUE              );  \
        } while(0)
        
    #if defined(MTOUCH_SLEEP_ENABLED)                                                                                   
    #define MTOUCH_EESLEEP_INIT()                                                                                   \
        do {                                                                                                        \
            MTOUCH_EESLEEP_INIT_THRESHOLDS();                                                                       \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_INACTIVE_ADRL,   (uint8_t)(MTOUCH_SLEEP_INACTIVE_RESET & 0xFF)   );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_INACTIVE_ADRH,   (uint8_t)(MTOUCH_SLEEP_INACTIVE_RESET >> 8)     );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_rTHRESHper_ADR,  (uint8_t)(MTOUCH_SLEEP_RTHRESH_FACTOR * 100)    );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_SpS_ADRL,        (uint8_t)(MTOUCH_SLEEP_SAMPLES_PER_SCAN & 0xFF) );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_SpS_ADRH,        (uint8_t)(MTOUCH_SLEEP_SAMPLES_PER_SCAN >> 8)   );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_INIT_ADRL,       (uint8_t)(MTOUCH_SLEEP_INIT_SCANS & 0xFF)       );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_INIT_ADRH,       (uint8_t)(MTOUCH_SLEEP_INIT_SCANS >> 8)         );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_REINIT_ADRL,     (uint8_t)(MTOUCH_SLEEP_REINIT_SCANS & 0xFF)     );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_REINIT_ADRH,     (uint8_t)(MTOUCH_SLEEP_REINIT_SCANS >> 8)       );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_pTIMEOUT_ADRL,   (uint8_t)(MTOUCH_SLEEP_MAX_PRESSTIME & 0xFF)    );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_pTIMEOUT_ADRH,   (uint8_t)(MTOUCH_SLEEP_MAX_PRESSTIME >> 8)      );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_AVGUPDATE_ADR,             MTOUCH_SLEEP_AVG_UPDATE               );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_AVGRATE_ADRL,    (uint8_t)(MTOUCH_SLEEP_AVG_RATE & 0xFF)         );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_AVGRATE_ADRH,    (uint8_t)(MTOUCH_SLEEP_AVG_RATE >> 8)           );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_DECSTEP_ADRL,    (uint8_t)(MTOUCH_SLEEP_DECIMATION_STEP & 0xFF)  );  \
            MTOUCH_EEPROM_write(MTOUCH_EESLEEP_DECSTEP_ADRH,    (uint8_t)(MTOUCH_SLEEP_DECIMATION_STEP >> 8)    );  \
        } while(0)
    #endif
        
    #if defined(MTOUCH_SLEEP_ENABLED)
    #define MTOUCH_EEPROM_INIT()                                                                                        \
        do {                                                                                                            \
            if (MTOUCH_EEPROM_read(MTOUCH_EEPROM_START_ADR) != MTOUCH_EEPROM_INIT_VALUE)                                \
            {                                                                                                           \
                MTOUCH_EEACTIVE_INIT();                                                                                 \
                MTOUCH_EESLEEP_INIT();                                                                                  \
            }                                                                                                           \
        } while(0)
    #else
    #define MTOUCH_EEPROM_INIT()                                                                                        \
        do {                                                                                                            \
            if (MTOUCH_EEPROM_read(MTOUCH_EEPROM_START_ADR) != MTOUCH_EEPROM_INIT_VALUE)                                \
            {                                                                                                           \
                MTOUCH_EEACTIVE_INIT();                                                                                 \
            }                                                                                                           \
        } while(0)
    #endif

        
    #define MTOUCH_EEPROM_READandSEND(addr)     \
        do {                                    \
            uint8_t temp = EEDATL;              \
            EEADR   = addr;                     \
            SELECT_EEPROM();                    \
            START_READ_EEPROM();                \
            chksum ^= temp;                     \
            put_Char(temp);                     \
        } while(0)
    
    #define MTOUCH_EEPROM_WRITE(addr, value)    \
        do {                                    \
            EEADR   = addr;                     \
            EEDATL  = value;                    \
            SELECT_EEPROM();                    \
            ENABLE_WRITE_EEPROM();              \
            EECON2  = 0x55;                     \
            EECON2  = 0xAA;                     \
            START_WRITE_EEPROM();               \
            ENABLE_WRITE_EEPROM();              \
            while(!EEIF);                       \
            EEIF    = 0;                        \
        } while(0)
    
    #define MTOUCH_EEPROM_SEND_CHKSUM()         put_Char(chksum)
    
    #define MTOUCH_EEPROM_SEND_OPCODE(value, size)  \
        do {                                        \
            SENDB   = 1;                            \
            TXREG   = size;                         \
            while(SENDB);                           \
            put_Char(size);                         \
            put_Char(value);                        \
            chksum  = value;                        \
        } while(0)
        
    #define MTOUCH_EEPROM_SEND_BYTE(byte)   \
        do {                                \
            put_Char(byte);                 \
            chksum ^= byte;                 \
        } while (0)
    
    #define MTOUCH_EEPROM_INIT_0()     MTOUCH_EEPROM_INIT_THRESHOLD(0)
    #define MTOUCH_EEPROM_INIT_1()     MTOUCH_EEPROM_INIT_THRESHOLD(1)
    #define MTOUCH_EEPROM_INIT_2()     MTOUCH_EEPROM_INIT_THRESHOLD(2)
    #define MTOUCH_EEPROM_INIT_3()     MTOUCH_EEPROM_INIT_THRESHOLD(3)
    #define MTOUCH_EEPROM_INIT_4()     MTOUCH_EEPROM_INIT_THRESHOLD(4)
    #define MTOUCH_EEPROM_INIT_5()     MTOUCH_EEPROM_INIT_THRESHOLD(5)
    #define MTOUCH_EEPROM_INIT_6()     MTOUCH_EEPROM_INIT_THRESHOLD(6)
    #define MTOUCH_EEPROM_INIT_7()     MTOUCH_EEPROM_INIT_THRESHOLD(7)
    #define MTOUCH_EEPROM_INIT_8()     MTOUCH_EEPROM_INIT_THRESHOLD(8)
    #define MTOUCH_EEPROM_INIT_9()     MTOUCH_EEPROM_INIT_THRESHOLD(9)
    #define MTOUCH_EEPROM_INIT_10()    MTOUCH_EEPROM_INIT_THRESHOLD(10)
    #define MTOUCH_EEPROM_INIT_11()    MTOUCH_EEPROM_INIT_THRESHOLD(11)
    #define MTOUCH_EEPROM_INIT_12()    MTOUCH_EEPROM_INIT_THRESHOLD(12)
    #define MTOUCH_EEPROM_INIT_13()    MTOUCH_EEPROM_INIT_THRESHOLD(13)
    #define MTOUCH_EEPROM_INIT_14()    MTOUCH_EEPROM_INIT_THRESHOLD(14)
    #define MTOUCH_EEPROM_INIT_15()    MTOUCH_EEPROM_INIT_THRESHOLD(15)
    #define MTOUCH_EEPROM_INIT_16()    MTOUCH_EEPROM_INIT_THRESHOLD(16)
    #define MTOUCH_EEPROM_INIT_17()    MTOUCH_EEPROM_INIT_THRESHOLD(17)
    #define MTOUCH_EEPROM_INIT_18()    MTOUCH_EEPROM_INIT_THRESHOLD(18)
    #define MTOUCH_EEPROM_INIT_19()    MTOUCH_EEPROM_INIT_THRESHOLD(19)
    #define MTOUCH_EEPROM_INIT_20()    MTOUCH_EEPROM_INIT_THRESHOLD(20)
    #define MTOUCH_EEPROM_INIT_21()    MTOUCH_EEPROM_INIT_THRESHOLD(21)
    #define MTOUCH_EEPROM_INIT_22()    MTOUCH_EEPROM_INIT_THRESHOLD(22)
    #define MTOUCH_EEPROM_INIT_23()    MTOUCH_EEPROM_INIT_THRESHOLD(23)
    #define MTOUCH_EEPROM_INIT_24()    MTOUCH_EEPROM_INIT_THRESHOLD(24)
    #define MTOUCH_EEPROM_INIT_25()    MTOUCH_EEPROM_INIT_THRESHOLD(25)
    #define MTOUCH_EEPROM_INIT_26()    MTOUCH_EEPROM_INIT_THRESHOLD(26)
    #define MTOUCH_EEPROM_INIT_27()    MTOUCH_EEPROM_INIT_THRESHOLD(27)
    #define MTOUCH_EEPROM_INIT_28()    MTOUCH_EEPROM_INIT_THRESHOLD(28)
    #define MTOUCH_EEPROM_INIT_29()    MTOUCH_EEPROM_INIT_THRESHOLD(29)
    
    #define MTOUCH_EESLEEP_INIT_0()    MTOUCH_EESLEEP_INIT_THRESHOLD(0)
    #define MTOUCH_EESLEEP_INIT_1()    MTOUCH_EESLEEP_INIT_THRESHOLD(1)
    #define MTOUCH_EESLEEP_INIT_2()    MTOUCH_EESLEEP_INIT_THRESHOLD(2)
    #define MTOUCH_EESLEEP_INIT_3()    MTOUCH_EESLEEP_INIT_THRESHOLD(3)
    #define MTOUCH_EESLEEP_INIT_4()    MTOUCH_EESLEEP_INIT_THRESHOLD(4)
    #define MTOUCH_EESLEEP_INIT_5()    MTOUCH_EESLEEP_INIT_THRESHOLD(5)
    #define MTOUCH_EESLEEP_INIT_6()    MTOUCH_EESLEEP_INIT_THRESHOLD(6)
    #define MTOUCH_EESLEEP_INIT_7()    MTOUCH_EESLEEP_INIT_THRESHOLD(7)

    #if MTOUCH_NUMBER_SENSORS < 30
        #undef  MTOUCH_EEPROM_INIT_29()
        #define MTOUCH_EEPROM_INIT_29()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 29
        #undef  MTOUCH_EEPROM_INIT_28()
        #define MTOUCH_EEPROM_INIT_28()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 28
        #undef  MTOUCH_EEPROM_INIT_27()
        #define MTOUCH_EEPROM_INIT_27()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 27
        #undef  MTOUCH_EEPROM_INIT_26()
        #define MTOUCH_EEPROM_INIT_26()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 26
        #undef  MTOUCH_EEPROM_INIT_25()
        #define MTOUCH_EEPROM_INIT_25()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 25
        #undef  MTOUCH_EEPROM_INIT_24()
        #define MTOUCH_EEPROM_INIT_24()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 24
        #undef  MTOUCH_EEPROM_INIT_23()
        #define MTOUCH_EEPROM_INIT_23()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 23
        #undef  MTOUCH_EEPROM_INIT_22()
        #define MTOUCH_EEPROM_INIT_22()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 22
        #undef  MTOUCH_EEPROM_INIT_21()
        #define MTOUCH_EEPROM_INIT_21()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 21
        #undef  MTOUCH_EEPROM_INIT_20()
        #define MTOUCH_EEPROM_INIT_20()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 20
        #undef  MTOUCH_EEPROM_INIT_19()
        #define MTOUCH_EEPROM_INIT_19()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 19
        #undef  MTOUCH_EEPROM_INIT_18()
        #define MTOUCH_EEPROM_INIT_18()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 18
        #undef  MTOUCH_EEPROM_INIT_17()
        #define MTOUCH_EEPROM_INIT_17()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 17
        #undef  MTOUCH_EEPROM_INIT_16()
        #define MTOUCH_EEPROM_INIT_16()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 16
        #undef  MTOUCH_EEPROM_INIT_15()
        #define MTOUCH_EEPROM_INIT_15()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 15
        #undef  MTOUCH_EEPROM_INIT_14()
        #define MTOUCH_EEPROM_INIT_14()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 14
        #undef  MTOUCH_EEPROM_INIT_13()
        #define MTOUCH_EEPROM_INIT_13()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 13
        #undef  MTOUCH_EEPROM_INIT_12()
        #define MTOUCH_EEPROM_INIT_12()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 12
        #undef  MTOUCH_EEPROM_INIT_11()
        #define MTOUCH_EEPROM_INIT_11()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 11
        #undef  MTOUCH_EEPROM_INIT_10()
        #define MTOUCH_EEPROM_INIT_10()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 10
        #undef  MTOUCH_EEPROM_INIT_9()
        #define MTOUCH_EEPROM_INIT_9()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 9
        #undef  MTOUCH_EEPROM_INIT_8()
        #define MTOUCH_EEPROM_INIT_8()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 8
        #undef  MTOUCH_EEPROM_INIT_7()
        #define MTOUCH_EEPROM_INIT_7()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 7
        #undef  MTOUCH_EEPROM_INIT_6()
        #define MTOUCH_EEPROM_INIT_6()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 6
        #undef  MTOUCH_EEPROM_INIT_5()
        #define MTOUCH_EEPROM_INIT_5()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 5
        #undef  MTOUCH_EEPROM_INIT_4()
        #define MTOUCH_EEPROM_INIT_4()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 4
        #undef  MTOUCH_EEPROM_INIT_3()
        #define MTOUCH_EEPROM_INIT_3()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 3
        #undef  MTOUCH_EEPROM_INIT_2()
        #define MTOUCH_EEPROM_INIT_2()
    #endif
    #if MTOUCH_NUMBER_SENSORS < 2
        #undef  MTOUCH_EEPROM_INIT_1()
        #define MTOUCH_EEPROM_INIT_1()
    #endif
    
    #if defined(MTOUCH_SLEEP_ENABLED)
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS < 8
            #undef  MTOUCH_EESLEEP_INIT_7()
            #define MTOUCH_EESLEEP_INIT_7()
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS < 7
            #undef  MTOUCH_EESLEEP_INIT_6()
            #define MTOUCH_EESLEEP_INIT_6()
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS < 6
            #undef  MTOUCH_EESLEEP_INIT_5()
            #define MTOUCH_EESLEEP_INIT_5()
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS < 5
            #undef  MTOUCH_EESLEEP_INIT_4()
            #define MTOUCH_EESLEEP_INIT_4()
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS < 4
            #undef  MTOUCH_EESLEEP_INIT_3()
            #define MTOUCH_EESLEEP_INIT_3()
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS < 3
            #undef  MTOUCH_EESLEEP_INIT_2()
            #define MTOUCH_EESLEEP_INIT_2()
        #endif
        #if MTOUCH_NUMBER_OF_SLEEP_SENSORS < 2
            #undef  MTOUCH_EESLEEP_INIT_1()
            #define MTOUCH_EESLEEP_INIT_1()
        #endif
    #endif

#define MTOUCH_EEPROM_INIT_THRESHOLDS()     \
    do {                                    \
        MTOUCH_EEPROM_INIT_0();             \
        MTOUCH_EEPROM_INIT_1();             \
        MTOUCH_EEPROM_INIT_2();             \
        MTOUCH_EEPROM_INIT_3();             \
        MTOUCH_EEPROM_INIT_4();             \
        MTOUCH_EEPROM_INIT_5();             \
        MTOUCH_EEPROM_INIT_6();             \
        MTOUCH_EEPROM_INIT_7();             \
        MTOUCH_EEPROM_INIT_8();             \
        MTOUCH_EEPROM_INIT_9();             \
        MTOUCH_EEPROM_INIT_10();            \
        MTOUCH_EEPROM_INIT_11();            \
        MTOUCH_EEPROM_INIT_12();            \
        MTOUCH_EEPROM_INIT_13();            \
        MTOUCH_EEPROM_INIT_14();            \
        MTOUCH_EEPROM_INIT_15();            \
        MTOUCH_EEPROM_INIT_16();            \
        MTOUCH_EEPROM_INIT_17();            \
        MTOUCH_EEPROM_INIT_18();            \
        MTOUCH_EEPROM_INIT_19();            \
        MTOUCH_EEPROM_INIT_20();            \
        MTOUCH_EEPROM_INIT_21();            \
        MTOUCH_EEPROM_INIT_22();            \
        MTOUCH_EEPROM_INIT_23();            \
        MTOUCH_EEPROM_INIT_24();            \
        MTOUCH_EEPROM_INIT_25();            \
        MTOUCH_EEPROM_INIT_26();            \
        MTOUCH_EEPROM_INIT_27();            \
        MTOUCH_EEPROM_INIT_28();            \
        MTOUCH_EEPROM_INIT_29();            \
    } while(0)

#define MTOUCH_EESLEEP_INIT_THRESHOLDS()    \
    do {                                    \
        MTOUCH_EESLEEP_INIT_0();            \
        MTOUCH_EESLEEP_INIT_1();            \
        MTOUCH_EESLEEP_INIT_2();            \
        MTOUCH_EESLEEP_INIT_3();            \
        MTOUCH_EESLEEP_INIT_4();            \
        MTOUCH_EESLEEP_INIT_5();            \
        MTOUCH_EESLEEP_INIT_6();            \
        MTOUCH_EESLEEP_INIT_7();            \
    } while(0)    

#endif
#endif
/// @endcond
