/**
 * @brief enter here if using the 8bit demo, else use the base code ISR
 * @file OnePIC_8bit_demo_isr.c
 * 
 * @defgroup demo_isr ISR
 * @ingroup eight_bit_demo
 * @{
 *
 * @note there can only be 1 ISR function!
 */


#include "OnePIC_int.h"
#include "OnePIC_timer.h"
#include "mTouchCVD.h"
#include "bsp.h"

#include <htc.h>

#include "OnePIC_8bit_demo_isr.h"
#include "OnePIC_8bit_demo_irda.h" //custom setup for IrDA RX interrupt
#include "OnePIC_8bit_demo_main.h"

#ifdef WIRELESS_DEMO //exclude this if only using the base code

/**
 * @struct IR_DEMO_FLAG
 * @struct IR_DEMO_FLAG
 * @ingroup global
 */
extern IR_DEMO_RX rx_byte;
extern IR_DEMO_FLAG rx_tx_flag;

/**
 * @brief ISR for the 8bit Demo which checks the following interrupts
 *  - mTouch
 *  - IrDA
 *      # check if UART start bit is low
 *      # if so, then get the whole byte
 *  - MCP2200
 *
 * @note Should disable all interrupts when writing to LCD or other time senstive funcitons
 */

void interrupt ISR(void) {
    //mTouch Interrupt
    if (EDF_INT_SourceFlagGet(INT_TMR_2)) {
        EDF_INT_SourceFlagClear(INT_TMR_2); //clear flag
        mTouch_Scan(); //scan the touch buttons
        EDF_TMR_CounterSet(TMR_2, 255);
        
    }

    //IrDA Demo - Poll RX pin
    if (EDF_INT_SourceFlagGet(INT_TMR_4)) {
        EDF_INT_SourceFlagClear(INT_TMR_4);
        if (!IRDA_RX) { //UART Start Bit is active LOW
            rx_byte.irda_rx = irda_getch(); //get the byte NOW!
            rx_tx_flag.IRDA_RX_FLAG = 1; //print to the LCD out of the ISR
        }
    }
    //IrDA Demo - MCP2200 RX flag
    if (UART_RX_INTERRUPT) {
        UART_RX_INTERRUPT = 0;
        rx_byte.mcp2200_rx = UART_RX_REGISTER; //must do this to clear interrupt (empty RCREG)
        rx_tx_flag.MCP2200_RX_FLAG = 1;
    }
}
#endif

/** @}*/
