/**
* @file OnePIC_8bit_demo_main.h
* @ingroup eight_bit_demo
* @{
*
*/


#ifndef __ONEPIC_8BIT_DEMO_MAIN_H
#define __ONEPIC_8BIT_DEMO_MAIN_H


/**
 * @struct IR_DEMO_FLAG demo_main.h demo_main
 * @brief Contains the RX/TX flags
 * @var IR_DEMO_FLAG::IRDA_RX_FLAG
 * contains the IR RX Flag
 * @var IR_DEMO_FLAG::MCP2200_RX_FLAG
 * contains the USB RX Flag
 * @var IR_DEMO_FLAG::TOUCH_TX_FLAG
 * contains the touch buttons RX Flag
 *
 * @file demo_main.h
 *
 */
typedef struct {
    bool IRDA_RX_FLAG;
    bool MCP2200_RX_FLAG;
    bool TOUCH_TX_FLAG;
} IR_DEMO_FLAG;


/**
 * @struct IR_DEMO_RX
 * @brief Global variables that are written to in the ISR
 * @var IR_DEMO_RX::mcp2200_rx
 * UART received byte from PC
 * @var IR_DEMO_RX::irda_rx
 * IR received byte from PICtail
 * @var IR_DEMO_RX::touch_tx
 * Saves which button pressed in mTouch ISR
 * @var IR_DEMO_RX::rx_tx_old_byte
 * Don't continuosly update the LCD unless new data received or sent!
 *
 */
typedef struct {
    char mcp2200_rx; 
    char irda_rx; 
    char touch_tx; 
    char rx_tx_old_byte; 
}IR_DEMO_RX;


//prototypes for main program
void demo_main_loop(void);
void irda_demo_init(void);
void irda_tx(char c);
void mcp2200_tx(char c);

void touch_service();
void mcp2200_service();
void ir_service();


#endif
/** @}*/