/**
 * @brief Connects to the MCP2200 using the Hardware UART module
 * @file OnePIC_8bit_demo_mcp2200.c
 * @defgroup mcp2200 MCP2200 - USB
 * @ingroup eight_bit_demo
 * @{
 *  
 */
#include <htc.h>   //PIC specific hardware
#include <stdio.h> //Used for printf
#include <string.h> //Used for sprintf

#include "bsp.h"   //XTAL Frequ
#include "OnePIC_8bit_demo_mcp2200.h" //prototypes


/**
 * @brief Configures the UART module for 9600 baud
 * @note If this is changed, you must change it in the GUI as well
 */
void mcp2200_init(void){
        MCP2200_RX_TRIS = 1; //hardware UART module requires RX/TX to be inputs
        MCP2200_TX_TRIS = 1; //MUST be an input
        
	SPBRG = DIVIDER;
        SYNC = 0;
        BRGH = 0;
        BRG16 = 0;
	SPBRG = 51;             //9600 baud
	RCSTA = (0x90);
        SPEN = 1;
        TXEN = 1;
        TX9 = 0;
        RCIE = 1;               //enable RX interrupt
}
/**
 * Outputs a message to prompt the user for a keypress through COM port
 */
void mcp2200_prompt(void) {
    char data_byte;
    char buffer[12];
    
    printf("\rPress a key and I will echo it back:\n");
    while (1) {
        data_byte = mcp2200_getch(); // read a response from the user
        printf("\rI detected [%c]", data_byte); // echo it back
        sprintf(buffer, "Number: %c", data_byte);
    }

}

/**
 * Prints a single character
 * If using PRINTF, this MUST be avaliable!
 *
 * @param byte unsigned char
 */
void putch(uint8_t byte) { 
    //First check to see if currently transmitting
    while (!TXIF) // will be set when register is empty
        continue; //wait until finished sending
    TXREG = byte; //now send the byte
    
}

/**
 * Receive one character from MCP2200
 * @return unsigned char RCREG
 */
char mcp2200_getch(void) {
    //First check to see if currently receiving
    while (!RCIF) // set when register is not empty
        continue; //wait until finished receiving the byte
    return RCREG; //we got the byte, lets return it
                  //if RCIE is enabled, RCIF will be set and an interrupt issued now
}

/**
 * A simple 'echo back what was sent'
 * @return unsigned char character sent
 */
char mcp2200_getche(void) {
    char c;
    
    putch(c = mcp2200_getch()); //wait until a byte is received and then echo it back
    return c;
}
/** @}*/