/**
 * @brief inits the LEDs on the board and translates POT values to LED bar graph
 * @file OnePIC_led.c
 * @defgroup led LED
 * @ingroup base
 * @{
 */

#include "OnePIC_led.h"
#include "Compiler.h"
#include "bsp.h"

/**
 * @brief init all LEDs by turning them all OFF
 */
void EDF_LEDInit()
{

    gpLED_8_TRIS = 0;
    gpLED_7_TRIS = 0;
    gpLED_6_TRIS = 0;
    gpLED_5_TRIS = 0;
    gpLED_4_TRIS = 0;
    gpLED_3_TRIS = 0;
    gpLED_2_TRIS = 0;
    gpLED_1_TRIS = 0;

    gpLED_UP_TRIS    = 0;
    gpLED_DOWN_TRIS  = 0;
    gpLED_RIGHT_TRIS = 0;
    gpLED_LEFT_TRIS  = 0;
}

/**
 * @brief maps all LED definitions to readable name and turns it ON/OFF
 * @param d specific LED pin
 * @param s LED_ON or LED_OFF
 *
 */
static void EDF_LEDMod(EDF_LED d, uint8_t s)
{
   
    switch(d)
    {
        case LED_UP:    gpLED_UP     = s; break;
        case LED_DOWN:  gpLED_DOWN   = s; break;
        case LED_LEFT:  gpLED_LEFT   = s; break;
        case LED_RIGHT: gpLED_RIGHT  = s; break;
        case LED_1:     gpLED_1      = s; break;
        case LED_2:     gpLED_2      = s; break;
        case LED_3:     gpLED_3      = s; break;
        case LED_4:     gpLED_4      = s; break;
        case LED_5:     gpLED_5      = s; break;
        case LED_6:     gpLED_6      = s; break;
        case LED_7:     gpLED_7      = s; break;
        case LED_8:     gpLED_8      = s; break;
        default: break;
    }
}

/**
 * @brief set a single LED
 * @param d a specific LED to turn ON
 */
void EDF_LEDSet(EDF_LED d) 
{ 
    EDF_LEDMod(d,LED_ON);
}

/**
 * @brief clear a single LED
 * @param d A specific LED to turn OFF
 */
void EDF_LEDClr(EDF_LED d) 
{ 
    EDF_LEDMod(d,LED_OFF);
}

/**
 *
 * @param n POT reading
 * @note 8bit does not have access to LEDs 7 & 8
 */
void EDF_pot2bar(uint16_t n)
{
  //  if(n < 1024 && n >= 896)     { EDF_LEDBarSet(0xFF); }
  //  else if(n < 896 && n >= 768) { EDF_LEDBarSet(0x7F); }
    if(n < 1024 && n >= 850) { EDF_LEDBarSet(0x3F); }
    else if(n < 850 && n >= 680) { EDF_LEDBarSet(0x1f); }
    else if(n < 680 && n >= 510) { EDF_LEDBarSet(0x0F); }
    else if(n < 510 && n >= 240) { EDF_LEDBarSet(0x07); }
    else if(n < 340 && n >= 170) { EDF_LEDBarSet(0x03); }
    else if(n < 170 && n > 5)    { EDF_LEDBarSet(0x01); }
    else if(n <= 5) {EDF_LEDBarSet(0x00); }
}

/**
 * @brief light up the bar display
 * @param bf
 * This displays hex values on the LED bar
 */
void EDF_LEDBarSet(uint8_t bf)
{
    if((bf&0x80)!=0){ EDF_LEDSet(LED_8); } else { EDF_LEDClr(LED_8); }
    if((bf&0x40)!=0){ EDF_LEDSet(LED_7); } else { EDF_LEDClr(LED_7); }
    if((bf&0x20)!=0){ EDF_LEDSet(LED_6); } else { EDF_LEDClr(LED_6); }
    if((bf&0x10)!=0){ EDF_LEDSet(LED_5); } else { EDF_LEDClr(LED_5); }
    if((bf&0x08)!=0){ EDF_LEDSet(LED_4); } else { EDF_LEDClr(LED_4); }
    if((bf&0x04)!=0){ EDF_LEDSet(LED_3); } else { EDF_LEDClr(LED_3); }
    if((bf&0x02)!=0){ EDF_LEDSet(LED_2); } else { EDF_LEDClr(LED_2); }
    if((bf&0x01)!=0){ EDF_LEDSet(LED_1); } else { EDF_LEDClr(LED_1); }
}
/** @}*/
