/**
 * @file OnePIC_mtouch.c
 * @brief Contains touch routine which lights up the corresponding mTouch LED when pressed
 * Enter here every mTouch interrupt for polling of buttons
 * @defgroup mtouch mTouch
 * @ingroup base
 * @{
 *
 */

#include "OnePIC_mtouch.h"
#ifdef __PIC24FJ256GA106__
#include "mTouch.h"
#elif defined (__PICC__)
#include "mTouchCVD.h"
#elif defined (__PIC32MX__)
#include "mTouchCapAPI.h"
#include "mTouchConfig.h"
#include "mTouchCapLED.h"
#include "mTouchCapApp_DirectKeys.h"
typedef unsigned char uint8_t; /**<@brief  quick typedef*/
#endif
#include <stdint.h>
#include "bsp.h"
#include "OnePIC_led.h"
//#include "OnePIC_lcd.h" //requried for LCDUpdate
//#include "OnePIC_rtcc.h" //requried for _time_str


/**
 * @var _bannerState
 * @brief indicates whether or not to print:
 *  - mTouch key (up,right,down,left,enter)
 *  - POT reading
 *
 * @note LCD will ALWAYS be displaying the time
 */
BANNER_STATE _bannerState;
uint8_t *p_capString; /**<@brief  pointer to the actuall sting to be printed to the LCD*/
uint8_t p_capStringUp[] = "Up"; /**<@brief   'UP' text*/
uint8_t p_capStringDown[] = "Down"; /**<@brief 'DOWN' text */
uint8_t p_capStringLeft[] = "Left"; /**<@brief 'LEFT' text*/
uint8_t p_capStringRight[] = "Right"; /**<@brief  'RIGHT' text*/
uint8_t p_capStringEnter[] = "Enter"; /**<@brief 'ENTER' text*/
uint8_t p_capStringBlank[] = " "; /**<@brief  ' ' blank text*/

uint8_t printed = 0; //this is so the LCD will print even the SLIGHTEST touch press
/**
 * @brief Polls the touch buttons
 *
 * @note increase interrupt frequency for better resolution
 */
void EDF_mTouchPoll(void) {
#ifdef __PICC__
    if (mTouch_isDataReady()) { // Is new information ready?
        mTouch_Service();
#elif defined __PIC24FJ256GA106__
    if (MTouchIsDataReady()) {
        //MTouchDecode();
#else
        {
#endif
        if (EDF_CapTouchGetButtonState(CAP_UP) == EDF_MTOUCH_PRESSED) {
            p_capString = p_capStringUp;
            _bannerState = BANNER_CAP;
            EDF_LEDSet(LED_UP);
            EDF_LEDClr(LED_RIGHT);
            EDF_LEDClr(LED_DOWN);
            EDF_LEDClr(LED_LEFT);
            printed = 2;

        } else if (EDF_CapTouchGetButtonState(CAP_DOWN) == EDF_MTOUCH_PRESSED) {
            p_capString = p_capStringDown;
            _bannerState = BANNER_CAP;
            EDF_LEDSet(LED_DOWN);
            EDF_LEDClr(LED_UP);
            EDF_LEDClr(LED_RIGHT);
            EDF_LEDClr(LED_LEFT);
            printed = 2;

        } else if (EDF_CapTouchGetButtonState(CAP_LEFT) == EDF_MTOUCH_PRESSED) {
            p_capString = p_capStringLeft;
            _bannerState = BANNER_CAP;
            EDF_LEDSet(LED_LEFT);
            EDF_LEDClr(LED_UP);
            EDF_LEDClr(LED_RIGHT);
            EDF_LEDClr(LED_DOWN);
            printed = 2;

        } else if (EDF_CapTouchGetButtonState(CAP_RIGHT) == EDF_MTOUCH_PRESSED) {
            p_capString = p_capStringRight;
            _bannerState = BANNER_CAP;
            EDF_LEDSet(LED_RIGHT);
            EDF_LEDClr(LED_UP);
            EDF_LEDClr(LED_DOWN);
            EDF_LEDClr(LED_LEFT);
            printed = 2;

        } else if (EDF_CapTouchGetButtonState(CAP_ENTER) == EDF_MTOUCH_PRESSED) {
            p_capString = p_capStringEnter;
            _bannerState = BANNER_CAP;
            EDF_LEDSet(LED_UP);
            EDF_LEDSet(LED_RIGHT);
            EDF_LEDSet(LED_LEFT);
            EDF_LEDSet(LED_DOWN);
            printed = 2;

        } else {
            if (printed == 0) {
                _bannerState = BANNER_POT;
                p_capString = p_capStringBlank;
            }
            EDF_LEDClr(LED_UP);
            EDF_LEDClr(LED_RIGHT);
            EDF_LEDClr(LED_DOWN);
            EDF_LEDClr(LED_LEFT);
        }


    }
}
/** @}*/
