/**
 * @file OnePIC_switch.c
 * @defgroup base Base Code
 * @{
 */
#include "TimeDelay.h"
#include "OnePIC_switch.h"
#include "OnePIC_int.h"

#ifdef __PIC24FJ256GA106__
//#include "Controller.h"
#endif

uint8_t _pbState; /**<@brief _pbState what state push button is in */
uint8_t ON_ENTRY; /**<@brief 1 if held down already */
swState _swState; /**<@brief holds SW1 info*/

/**
 * @brief polls and debounces the switch for a press
 * @return bool 1 if button pressed, else 0
 *
 * @note the switch is duplexed with gpLED1 which is why port is being set as input briefly and then back to output
 */
uint8_t switchPressed(void) {
    uint8_t rVal = 0;
    bool sw1_state;

    sw1_state = SW1_LATCH; //save existing LED state
    SW1_LATCH = 0;        //stop driving the LED/SW1 pin
    SW1_TRIS = 1;
    Delay10us(1);
    if (SW1_PORT) { //read from port....if held down, the pin is connected to VDD
        Delay10us(1); //Debounce if switch is held down
        if(SW1_PORT) //still down?
            rVal = SW1_PORT;
    }
    SW1_TRIS = 0;   //configure as output
    SW1_LATCH = sw1_state;  //reload prior state

    return rVal;
}

/**
 * @brief when switch is pressed, toggle all of the LEDs once
 *
 * @note if held down, the state of the LEDs will remain constant
 */
void switchTasks(void) {
    switch (_swState) {
        case SW_PRESSED:
            if (ON_ENTRY) {
                _pbState ^= 1;
                ON_ENTRY = 0;
            }
            if (switchPressed() == 0) {
                _swState = SW_RELEASED;
            }
            break;
        case SW_RELEASED:
            if (switchPressed() == 1) {
                ON_ENTRY = 1;
                _swState = SW_PRESSED;
            }
            break;
    }
}

/** @}*/
