/**
 * @file OnePIC_int.h
 * @ingroup base
 * @{
 *
 */

#ifndef _ONEPIC_INT_H
#define _ONEPIC_INT_H
#include "bsp.h"
#include <stdbool.h>

/**
 * @enum EDF_INT_SOURCE_tag
 * @brief Contains all of the interrupt sources for the 8bit
 * @var EDF_INT_SOURCE::INT_TMR_0
 * TMR0
 * @var EDF_INT_SOURCE::INT_TMR_1
 * TMR1
 * @var EDF_INT_SOURCE::INT_TMR_2
 * TMR2
 * @var EDF_INT_SOURCE::INT_TMR_4
 * TMR5
 * @var EDF_INT_SOURCE::INT_TMR_6
 * TMR6
 * @var EDF_INT_SOURCE::INT_SSP
 * SSP
 *
 */
typedef enum EDF_INT_SOURCE_tag {
    INT_TMR_0,
    INT_TMR_1,
    INT_TMR_2,
    INT_TMR_4,
    INT_TMR_6,
    INT_SSP
} EDF_INT_SOURCE;
// Move to into OnePIC_isr.h
void EDF_INT_SourceFlagClear(EDF_INT_SOURCE source);
bool EDF_INT_SourceFlagGet(EDF_INT_SOURCE source);
void EDF_INT_SourceEnable(EDF_INT_SOURCE source);
bool EDF_INT_SourceEnableGet(EDF_INT_SOURCE source);
void EDF_INT_SourceDisable(EDF_INT_SOURCE source);
void EDF_INT_Enable(void);
void EDF_INT_Disable(void);
#endif

/** @}*/
/*******************************************************************************
 End of File
 */



