/**
 * @file OnePIC_lcd.h
 * @ingroup base
 * @{
 */

#ifndef _ONEPIC_LCD_H_
#define _ONEPIC_LCD_H_


#include "bsp.h"
#include <stdint.h>

#define LCD_LINES 2  /**<@brief  */
#define LCD_LINE_LENGTH 40 /**<@brief  */


/**********************************************************************
 * Commands 
 **********************************************************************/
#define CLEAR_DISPLAY			0x01	 /**<@brief clear display */

#define RETURN_HOME			0x02	 /**<@brief  return to (0,0) or rather top-left */

#define ENTRY_MODE_SET			0x04 /**<@brief  specify direction of cursor movement and display shift mode */
#define INCREMENT			0x02 /**<@brief  increment after each write */
#define DECREMENT			0x00 /**<@brief  decrement after each write */
#define SHIFT_ON			0x01 /**<@brief  shift display or move cursor?*/
#define SHIFT_OFF			0x00 /**<@brief  shift display or move cursor?*/

#define DISPLAY_CONTROL			0x08 /**<@brief  */
#define DISPLAY_ON			0x04 /**<@brief  */
#define DISPLAY_OFF			0x00 /**<@brief  */
#define CURSOR_ON			0x02 /**<@brief  */
#define CURSOR_OFF			0x00 /**<@brief  */
#define BLINKING_ON			0x01 /**<@brief  */
#define BLINKING_OFF			0x00 /**<@brief  */

#define CURSOR_DISPLAY_SHIFT            0x10 /**<@brief  */
#define DISPLAY_SHIFT_ON		0x08 /**<@brief  */
#define DISPLAY_SHIFT_OFF		0x00 /**<@brief  */
#define CURSOR_SHIFT_OFF		0x08 /**<@brief  */
#define CURSOR_SHIFT_ON			0x00 /**<@brief  */
#define SHIFT_RIGHT			0x04 /**<@brief  */
#define SHIFT_LEFT			0x00 /**<@brief  */

#define FUNCTION_SET			0x20 /**<@brief  */
#define EIGHT_BITS			0x10 /**<@brief  */
#define FOUR_BITS			0x00 /**<@brief  */
#define TWO_LINES			0x08 /**<@brief  */
#define ONE_LINE			0x00 /**<@brief  */
#define FIVEXTEN			0x04 /**<@brief  */
#define FIVEXEIGHT			0x00 /**<@brief  */

#define SET_CGRAM_ADDR			0x40 /**<@brief  */
#define SET_DDRAM_ADDR			0x80 /**<@brief  */

#define READ_BUSY_FLAG_ADDR
#define WRITE_DATA		
#define READ_DATA		



/**********************************************************************
 * Data
 **********************************************************************/
extern char LCDBuffer[LCD_LINES][LCD_LINE_LENGTH];

/**********************************************************************
 * Function Prototypes
 **********************************************************************/
void LCDStrobeEnable(void);
void LCDDataPut(unsigned char data);
void LCDCommand(unsigned char command);
void LCDPut(unsigned char character);
void LCDInit(void);
void LCDUpdate(void);
void LCDPutLine(uint8_t lineNo);


#endif // _ONEPIC_LCD_H_
/** @}*/
