/**
 *@file OnePIC_led.h
 * @ingroup base
 * @{
 * 
 */

#ifndef _EDF_LED_H
#define _EDF_LED_H
#include "Compiler.h"
#include <stdint.h>

/**
 * @enum EDF_LED
 * @brief Contains all LEDs on board
 *  - General Purpose (gpLEDs) below LCD
 *  - mTouch
 * @var EDF_LED::LED_UP
 * mtouch up
 * @var EDF_LED::LED_DOWN
 * mtouch down
 * @var EDF_LED::LED_LEFT
 * mtouch left
 * @var EDF_LED::LED_RIGHT
 * mtouch right
 * @var EDF_LED::LED_ENTER
 * mtouch enter
 * @var EDF_LED::LED_1
 * general purpose leftmost
 * @var EDF_LED::LED_2
 * general purpose 2
 * @var EDF_LED::LED_3
 * general purpose 3
 * @var EDF_LED::LED_4
 * general purpose 4
 * @var EDF_LED::LED_5
 * general purpose 5
 * @var EDF_LED::LED_6
 * general purpose 6
 * @var EDF_LED::LED_7
 * general purpose 7
 * @var EDF_LED::LED_8
 * general purpose rightmost
 *
 * @note the 8bit PIC16LF1939 does not have access to the 2 rightmost LEDs (LED 7&8)
 */
typedef enum
{
    LED_UP,
    LED_DOWN,
    LED_LEFT,
    LED_RIGHT,
    LED_ENTER,
    LED_1,
    LED_2,
    LED_3,
    LED_4,
    LED_5,
    LED_6,
    LED_7,
    LED_8
    
}EDF_LED;

/**
 * @brief Turns on an LED
 * @param d Led Instance (ex: LED_5)   
 *
 * @return None.
 */
void EDF_LEDSet(EDF_LED d);

/** 
 * @brief Turns off an LED
 * @param d Led Instance (ex: LED_5)   
 *
 * @return None.
 */
void EDF_LEDClr(EDF_LED d);

/**
 * @brief Displays POT value on the LEDs
 * @param n POT value
 * @return None.
 */
void EDF_pot2bar(uint16_t n);

/**
 * @brief Displays a bit pattern on the LEDs
 * @param bf LED bit-pattern
 * @return None.
 */
void EDF_LEDBarSet(uint8_t bf);

/**
 * @brief Initializes the LEDs
 * @return None.
 */
void EDF_LEDInit(void);
#endif // _EDF_LED_H

/** @}*/

