/**
 * @file OnePIC_timer.h
 * @ingroup base
 * @{
 */
#ifndef _ONEPIC_TIMER_H
#define _ONEPIC_TIMER_H
#include <stdint.h>
#include "Compiler.h"

/**
 * @enum TMR_MODULE_ID_tag
 * @brief Contains ONLY timer related sources for the 8bit
 * @var TMR_MODULE_ID::TMR_0
 * TMR0
 * @var TMR_MODULE_ID::TMR_1
 * TMR1
 * @var TMR_MODULE_ID::TMR_2
 * TMR2
 * @var TMR_MODULE_ID::TMR_4
 * TMR5
 * @var TMR_MODULE_ID::TMR_6
 * TMR6
 *
 */
typedef enum TMR_MODULE_ID_tag {
    TMR_0,
    TMR_1,
    TMR_2,
    TMR_4,
    TMR_6
} TMR_MODULE_ID;

/**
 * @enum EDF_TMR_CLOCK_SOURCE_tag
 * @brief Contains the source of the clocks such as:
 *  - Fosc
 *  - capactivie sensing oscillator
 *  - Fosc/4
 *  - dedicated oscillator circuit
 * @var TMR_MODULE_ID::TMR_SOURCE_INS_CLK
 * internal source
 * @var TMR_MODULE_ID::TMR_SOURCE_EXT_CLK
 * extenral source
 * 
 *
 */
typedef enum EDF_TMR_CLOCK_SOURCE_tag {
    TMR_SOURCE_INS_CLK = 0, // FOSC/2:P24 FOSC/4
    TMR_SOURCE_EXT_CLK = 1
} EDF_TMR_CLOCK_SOURCE;


/**
 * @enum EDF_TMR_PRESCALE_tag
 * @brief Contains all prescaler settings for All timers
 * @var EDF_TMR_PRESCALE::TMR246_PRESCALE_1_64
 * timer 2,4, & 6 1:64 prescaler
 * @var EDF_TMR_PRESCALE::TMR246_PRESCALE_1_16
 * timer 2,4, & 6 1:16 prescaler
 * @var EDF_TMR_PRESCALE::TMR246_PRESCALE_1_4
 * timer 2,4, & 6 1:4 prescaler
 * @var EDF_TMR_PRESCALE::TMR246_PRESCALE_1_1
 * timer 2,4, & 6 1:1 prescaler
 * @var EDF_TMR_PRESCALE::TMR1_PRESCALE_1_8
 * timer 1 1:8 prescaler
 * @var EDF_TMR_PRESCALE::TMR1_PRESCALE_1_4
 * timer 1 1:4 prescaler
 * @var EDF_TMR_PRESCALE::TMR1_PRESCALE_1_2
 * timer 1 1:2 prescaler
 * @var EDF_TMR_PRESCALE::TMR1_PRESCALE_1_1
 * timer 1 1:1 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_256
 * timer 0 1:256 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_128
 * timer 0 1:128 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_64
 * timer 0 1:64 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_32
 * timer 0 1:32 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_16
 * timer 0 1:16 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_8
 * timer 0 1:8 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_4
 * timer 0 1:4 prescaler
 * @var EDF_TMR_PRESCALE::TMR0_PRESCALE_1_2
 * timer 0 1:2 prescaler
 *
 *
 */
typedef enum EDF_TMR_PRESCALE_tag {
    TMR_PRESCALE_1_256 = 0x03,
    TMR_PRESCALE_1_64 = 0x02,
    TMR_PRESCALE_1_8 = 0x01,
    TMR_PRESCALE_1_1 = 0x00,

    TMR246_PRESCALE_1_64 = 0b11,
    TMR246_PRESCALE_1_16 = 0b10,
    TMR246_PRESCALE_1_4 = 0b01,
    TMR246_PRESCALE_1_1 = 0b00,

    TMR1_PRESCALE_1_8 = 0b11,
    TMR1_PRESCALE_1_4 = 0b10,
    TMR1_PRESCALE_1_2 = 0b01,
    TMR1_PRESCALE_1_1 = 0b00,

    TMR0_PRESCALE_1_256 = 0b111,
    TMR0_PRESCALE_1_128 = 0b110,
    TMR0_PRESCALE_1_64 = 0b101,
    TMR0_PRESCALE_1_32 = 0b100,
    TMR0_PRESCALE_1_16 = 0b011,
    TMR0_PRESCALE_1_8 = 0b010,
    TMR0_PRESCALE_1_4 = 0b001,
    TMR0_PRESCALE_1_2 = 0b000,


//#endif
} EDF_TMR_PRESCALE;
/** @}*/
void EDF_TMR_ClockSourceSelect(TMR_MODULE_ID index, EDF_TMR_CLOCK_SOURCE source);
void EDF_TMR_StopInIdleDisable(TMR_MODULE_ID index);
void EDF_TMR_GateDisable(TMR_MODULE_ID index);
void EDF_TMR_PrescalerSet(TMR_MODULE_ID index, EDF_TMR_PRESCALE prescaler);
void EDF_TMR_Start(TMR_MODULE_ID index);
void EDF_TMR_Stop(TMR_MODULE_ID index);
void EDF_TMR_CounterSet(TMR_MODULE_ID index, uint16_t value);
#endif // #ifndef _EDF_TMR_H

