/*************************************************************************
 *   2011 Microchip Technology Inc.                                       
 *  
 *  Project Name:    PIC24 EDF One PIC board
 *  FileName:        main16.c
 *  Processor:       PIC24FJ256GA106
 *  Compiler:        C30(3.30vb) or later
 *  IDE:             MPLABX IDE v1.00 or later
 *  Hardware:        One PIC Microcontroller Platform
 *************************************************************************/
/*************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and
 * any derivatives created by any person or entity by or on your behalf,
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors
 * retain all ownership and intellectual property rights in the
 * accompanying software and in all derivatives hereto.
 *
 * This software and any accompanying information is for suggestion only.
 * It does not modify Microchip's standard warranty for its products. You
 * agree that you are solely responsible for testing the software and
 * determining its suitability. Microchip has no obligation to modify,
 * test, certify, or support the software.
 *
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY
 * APPLICATION.
 *
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE,
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE,
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW,
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF
 * THESE TERMS.
 *************************************************************************/

/**
 * @file main16.c
 * @brief main loop for the 16bit code + base code
 * @subpage sixteen_bit
 * @defgroup sixteen_bit 16bit
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "OnePIC_int.h"
#include "OnePIC_timer.h"
#include "OnePIC_lcd.h"
#include "OnePIC_led.h"
#include "OnePIC_isr.h"
#include "OnePIC_switch.h"
#include "OnePIC_pot.h"
#include "OnePIC_i2c.h"
#include "Compiler.h"

#include "mTouch.h"
//#include "Controller.h"

#include "OnePIC_mtouch.h"
#include "OnePIC_rtcc.h"
#include "TimeDelay.h"

_CONFIG1(JTAGEN_OFF & GCP_OFF & GWRP_OFF & ICS_PGx2 & FWDTEN_OFF)
_CONFIG2(IESO_OFF & FNOSC_FRCPLL & FNOSC_PRIPLL & FCKSM_CSDCMD & IOL1WAY_OFF & POSCMOD_HS)

extern uint8_t printed; //used in OnePIC_mtouch.c

/**
 * @brief main loop for the 8bit. Tasks include:
 *  - Init
 *  - take a POT reading
 *  - poll mTouch buttons
 *  - process the real-time clock & calender (RTCC)
 */
int main(void) {
    uint16_t potReading;


    EDF_TMR_ClockSourceSelect(TMR_1, TMR_SOURCE_INS_CLK);
    EDF_TMR_StopInIdleDisable(TMR_1);
    EDF_TMR_GateDisable(TMR_1);
    EDF_TMR_PrescalerSet(TMR_1, TMR_PRESCALE_1_8);
    EDF_INT_SourceEnable(INT_TMR_1);
    EDF_INT_SourceFlagClear(INT_TMR_1);
    EDF_mTouchInit();


#if !defined(WIRELESS_DEMO)
    I2CInit();
    RTCCInit();
#endif
    LCDInit();
    EDF_LEDInit();
    sprintf(LCDBuffer[0], "   MICROCHIP    ");
    sprintf(LCDBuffer[1], "  One PIC MCU   ");
    LCDUpdate();
    _pbState = 1;
    _swState = SW_PRESSED;
    DelayMs(1000);
    sprintf(LCDBuffer[0], "PIC24FJ256GA106");
    uint8_t tick;
    potReading = 1000;


    EDF_TMR_Start(TMR_1); //Enable Touch now
#if defined(WIRELESS_DEMO)
    demo_main_loop();
#endif

    //while (1) {
    //   Nop();
#if !defined(WIRELESS_DEMO)
    RTCCProcessEvents();
#endif

    _bannerState = BANNER_POT; //start by printing the POT value
    potReading = 0; //init to 0
    EDF_pot2bar(potReading); //take the reading
    while (1) {

        potReading = EDF_potRead(); //read the POT
        if (_pbState == 1)
            EDF_pot2bar(potReading);
        else
            EDF_pot2bar(0x00); //if button is in it's DOWN state, don't light up the LEDs

        EDF_TMR_Stop(TMR_1);
        RTCCProcessEvents(); //get the time
        EDF_TMR_Start(TMR_1);
        if (_bannerState != BANNER_CAP) {
            sprintf(LCDBuffer[1], "%4d    %8s", potReading, _time_str); //print the POT reading
        } else
            sprintf(LCDBuffer[1], "%-5s   %8s", p_capString, _time_str); //print what button was pressed

        LCDUpdate();
        if (printed != 0) //use this so we don't miss the update even for the slightest touch
            printed--;
        switchTasks(); //poll the button
    }
    return 1;
}
/** @}*/

