/*************************************************************************
 *   2011 Microchip Technology Inc.
 *
 *  Project Name:    PIC32 EDF One PIC board
 *  FileName:        main32.c
 *  Processor:       PIC32MX795F512L
 *  Compiler:        C32(2.01) or later
 *  IDE:             MPLABX IDE v1.00 or later
 *  Hardware:        One PIC Microcontroller Platform
 *************************************************************************/
/*************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and
 * any derivatives created by any person or entity by or on your behalf,
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors
 * retain all ownership and intellectual property rights in the
 * accompanying software and in all derivatives hereto.
 *
 * This software and any accompanying information is for suggestion only.
 * It does not modify Microchip's standard warranty for its products. You
 * agree that you are solely responsible for testing the software and
 * determining its suitability. Microchip has no obligation to modify,
 * test, certify, or support the software.
 *
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY
 * APPLICATION.
 *
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE,
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE,
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW,
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS
// * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF
 * THESE TERMS.
 *************************************************************************/



/**
 * @brief Main loop for the 32bit + base code
 * @file main32.c
 */

/**
 * @subpage thirtytwo_bit
 * @defgroup thirtytwo_bit 32bit
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "OnePIC_int.h"
#include "OnePIC_timer.h"
#include "OnePIC_lcd.h"
#include "OnePIC_led.h"
#include "OnePIC_isr.h"
#include "OnePIC_switch.h"
#include "OnePIC_pot.h"
#include "OnePIC_i2c.h"
#include "Compiler.h"
#include "mTouchCapAPI.h"
#include "mTouchConfig.h"
#include "mTouchCapLED.h"
#include "mTouchCapApp_DirectKeys.h"
#include "OnePIC_mtouch.h"
#include "OnePIC_rtcc.h"
#include "TimeDelay.h"

#pragma config FNOSC    = FRCPLL   // Oscillator Selection: Internal FRC with PLL
#pragma config FPLLIDIV = DIV_2    // PLL Input Divider: Divide by 2
#pragma config FPLLMUL  = MUL_20   // PLL Multiplier: Multiply by 20
#pragma config FPLLODIV = DIV_4    // PLL Output Divider: Divide by 1
#pragma config POSCMOD  = HS       // Primary Oscillator: HS oscillator
#pragma config FCKSM    = CSDCMD   // Clock Switching & Fail Safe Monitor: Clock Switching Disabled, Clock Monitoring Disabled
#pragma config OSCIOFNC = OFF      // CLKO Enable: Disabled
#pragma config IESO     = OFF      // Internal/External Switch-over: Disabled
#pragma config FSOSCEN  = OFF      // Secondary Oscillator Enable: Disabled
#pragma config FPBDIV   = DIV_1    // Peripheral Clock divisor: Divide by 1
#pragma config UPLLEN   = OFF      // USB PLL Disabled
#pragma config UPLLIDIV = DIV_2    // USB PLL Input Divider
#pragma config FWDTEN   = OFF       // Watchdog Timer: Disabled
#pragma config WDTPS    = PS1      // Watchdog Timer Postscale: 1:1
#pragma config CP       = OFF      // Code Protect: OFF
#pragma config BWP      = OFF      // Boot Flash Write Protect: OFF
#pragma config PWP      = OFF      // Program Flash Write Protect: OFF
#pragma config ICESEL   = ICS_PGx2 // ICE/ICD Comm Channel Select: ICE pins shared with PGC1,PGD1
#pragma config DEBUG    = OFF      // Background Debugger Enable: OF

/**
 * @brief main loop for the 32bit. Tasks include:
 *  - Init
 *  - take a POT reading
 *  - poll mTouch buttons
 *  - process the real-time clock & calender (RTCC)
 */
int main(void) {
    uint16_t reading = 0;
    uint16_t tick = 0;

    EDF_mTouchInit();
    LCDInit();


    EDF_LEDInit();
    sprintf(LCDBuffer[0], "   MICROCHIP    ");
    sprintf(LCDBuffer[1], "  One PIC MCU   ");
    LCDUpdate();
    DelayMs(1000);
    sprintf(LCDBuffer[0], "PIC32MX795F512L");
    I2CInit();
    RTCCInit();
    _pbState = 1;
    _swState = SW_PRESSED;

    while (1) {

        if (tick++ == 30) { //30
            tick = 0;
            reading = EDF_potRead();
            switchTasks();
            RTCCProcessEvents();
            if (_pbState == 1) {
                EDF_pot2bar(reading);
            } else {
                EDF_pot2bar(0x00);
            }

            if (_bannertest == 0) {
                sprintf(LCDBuffer[1], "%4d    %8s", reading, _time_str);
                EDF_LEDClr(LED_UP);
                EDF_LEDClr(LED_DOWN);
                EDF_LEDClr(LED_LEFT);
                EDF_LEDClr(LED_RIGHT);
                LCDUpdate();
            } else if (_bannertest == 1) {
                if (edf_u == 1) {
                    sprintf(LCDBuffer[1], "%-5s   %8s", "Up", _time_str);
                } else if (edf_d == 1) {
                    sprintf(LCDBuffer[1], "%-5s   %8s", "Down", _time_str);
                } else if (edf_l == 1) {
                    sprintf(LCDBuffer[1], "%-5s   %8s", "Left", _time_str);
                } else if (edf_r == 1) {
                    sprintf(LCDBuffer[1], "%-5s   %8s", "Right", _time_str);
                } else if (edf_e == 1) {

                    sprintf(LCDBuffer[1], "%-5s   %8s", "Enter", _time_str);
                }
                RTCCProcessEvents();
                _bannertest = 0;
                LCDUpdate();
            }
        }
        mTouchTasks();
        DelayMs(1); //requried delay for ADC from mtouch

        //  }


    } //end while (1)
} // end main()
