/*************************************************************************
 *   2011 Microchip Technology Inc.
 *
 *  Project Name:    PIC16 EDF One PIC board
 *  FileName:        main8.c
 *  Processor:       PIC16LF1939
 *  Compiler:        HI-TECH PRO Ver. 9.83 or later
 *  IDE:             MPLABX IDE v1.00 (or later) 
 *  Hardware:        One PIC Microcontroller Platform
 *************************************************************************/
/*************************************************************************
 * MICROCHIP SOFTWARE NOTICE AND DISCLAIMER: You may use this software, and
 * any derivatives created by any person or entity by or on your behalf,
 * exclusively with Microchip's products in accordance with applicable
 * software license terms and conditions, a copy of which is provided for
 * your referencein accompanying documentation. Microchip and its licensors
 * retain all ownership and intellectual property rights in the
 * accompanying software and in all derivatives hereto.
 *
 * This software and any accompanying information is for suggestion only.
 * It does not modify Microchip's standard warranty for its products. You
 * agree that you are solely responsible for testing the software and
 * determining its suitability. Microchip has no obligation to modify,
 * test, certify, or support the software.
 *
 * THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
 * EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED
 * WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE, ITS INTERACTION WITH
 * MICROCHIP'S PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY
 * APPLICATION.
 *
 * IN NO EVENT, WILL MICROCHIP BE LIABLE, WHETHER IN CONTRACT, WARRANTY,
 * TORT (INCLUDING NEGLIGENCE OR BREACH OF STATUTORY DUTY), STRICT
 * LIABILITY, INDEMNITY, CONTRIBUTION, OR OTHERWISE, FOR ANY INDIRECT,
 * SPECIAL, PUNITIVE, EXEMPLARY, INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE,
 * FOR COST OR EXPENSE OF ANY KIND WHATSOEVER RELATED TO THE SOFTWARE,
 * HOWSOEVER CAUSED, EVEN IF MICROCHIP HAS BEEN ADVISED OF THE POSSIBILITY
 * OR THE DAMAGES ARE FORESEEABLE. TO THE FULLEST EXTENT ALLOWABLE BY LAW,
 * MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN ANY WAY RELATED TO THIS
 * SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY, THAT YOU HAVE PAID
 * DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *
 * MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE OF
 * THESE TERMS.
 *************************************************************************/



/**
 * @brief Main loop for the 8bit + base code
 * @file main8.c
 */

/**
 * @subpage eight_bit
 * @defgroup eight_bit 8bit
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "OnePIC_int.h"
#include "OnePIC_timer.h"
#include "OnePIC_lcd.h"
#include "OnePIC_led.h"
#include "OnePIC_isr.h"
#include "OnePIC_switch.h"
#include "OnePIC_pot.h"
#include "OnePIC_i2c.h"
#include "Compiler.h"
#if defined(WIRELESS_DEMO)
#include "OnePIC_8bit_demo_main.h" //Custom Demo
#endif
#include "OnePIC_mtouch.h"
#include "OnePIC_rtcc.h"
#include "TimeDelay.h"

//config bits for the 8bit
__CONFIG(FOSC_INTOSC & WDTE_OFF & PWRTE_OFF & MCLRE_ON & CP_OFF & CPD_OFF & BOREN_ON & CLKOUTEN_OFF & IESO_OFF & FCMEN_OFF);
__CONFIG(WRT_OFF & PLLEN_ON & STVREN_OFF & LVP_OFF);
extern RTCC _time;
extern uint8_t printed; /**<@var printed used to not miss LCD updates if touch button just pressed*/

/**
 * @brief main loop for the 8bit. Tasks include:
 *  - Init
 *  - take a POT reading
 *  - poll mTouch buttons
 *  - process the real-time clock & calender (RTCC)
 */
int main() {
    uint16_t potReading;

    EDF_mTouchInit();
    EDF_LEDInit();
    LCDInit();
    sprintf(LCDBuffer[0], "   MICROCHIP    ");
    sprintf(LCDBuffer[1], "  One PIC MCU   ");
    LCDUpdate();
    DelayMs(1000); //Pause for 1 sec for flash screen
    _pbState = 1;
    _swState = SW_PRESSED;
    sprintf(LCDBuffer[0], "  PIC16LF1939   ");


    EDF_TMR_ClockSourceSelect(TMR_2, TMR_SOURCE_INS_CLK); //if Change mTouch Timer Interrupt, must change it int mtouch_config/h
    EDF_TMR_StopInIdleDisable(TMR_2);
    //xxx    EDF_TMR_GateDisable(TMR_1);
    EDF_TMR_PrescalerSet(TMR_2, TMR246_PRESCALE_1_4);
    EDF_TMR_Start(TMR_2); //mTouch
    EDF_INT_SourceEnable(INT_TMR_2);
    EDF_INT_SourceFlagClear(INT_TMR_2);
    EDF_INT_Enable();
#ifdef WIRELESS_DEMO
    //This is defined in the 8bit_IR configuration in MPLABX under compiler->preprocessor
    demo_main_loop(); //enter demo if building the demo
#endif
    EDF_INT_SourceEnable(INT_SSP); //I2C

    I2CInit();  //I2C
    RTCCInit(); //RTC

    _bannerState = BANNER_POT;      //start by printing the POT value
    potReading = 0;                 //init to 0
    EDF_pot2bar(potReading);        //take the reading
    while (1) {
            //EDF_INT_SourceEnable(INT_TMR_2);
            potReading = EDF_potRead();     //read the POT
            if (_pbState == 1)
                EDF_pot2bar(potReading);
            else
                EDF_pot2bar(0x00);          //if button is in it's DOWN state, don't light up the LEDs

            //EDF_INT_SourceDisable(INT_TMR_2);
            RTCCProcessEvents();            //get the time
            if (_bannerState != BANNER_CAP) {
                sprintf(LCDBuffer[1], "%4d    %8s", potReading, _time_str); //print the POT reading
            } else
                sprintf(LCDBuffer[1], "%-5s   %8s", p_capString, _time_str); //print what button was pressed

            LCDUpdate();

            if(printed !=0) //use this so we don't miss the update even for the slightest touch
                printed--;
        switchTasks(); //poll the button
    }
    return 1;
}

